@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  config_file.c - config file routines                                     */
-/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp.h"
-static const char cvsid[] = "$Id: config_file.c,v 1.46 2004/11/03 10:51:32 masneyb Exp $";
+static const char cvsid[] = "$Id: config_file.c 950 2008-01-24 23:28:30Z masneyb $";
 
 void
 gftp_add_bookmark (gftp_bookmarks_var * newentry)
@@ -125,7 +125,7 @@ copyfile (char *source, char *dest)
     {
       printf (_("Error: Cannot open local file %s: %s\n"),
               source, g_strerror (errno));
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   if ((destfd = gftp_fd_open (NULL, dest, O_WRONLY | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR)) == -1)
@@ -133,7 +133,7 @@ copyfile (char *source, char *dest)
       printf (_("Error: Cannot open local file %s: %s\n"),
               dest, g_strerror (errno));
       close (srcfd);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   while ((n = read (srcfd, buf, sizeof (buf))) > 0)
@@ -142,26 +142,26 @@ copyfile (char *source, char *dest)
         {
           printf (_("Error: Could not write to socket: %s\n"), 
                   g_strerror (errno));
-          exit (1);
+          exit (EXIT_FAILURE);
         }
     }
 
   if (n == -1)
     {
       printf (_("Error: Could not read from socket: %s\n"), g_strerror (errno));
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   if (close (srcfd) == -1)
     {
       printf (_("Error closing file descriptor: %s\n"), g_strerror (errno));
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   if (close (destfd) == -1)
     {
       printf (_("Error closing file descriptor: %s\n"), g_strerror (errno));
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   return (1);
@@ -181,7 +181,7 @@ gftp_read_bookmarks (char *global_data_path)
   if ((tempstr = gftp_expand_path (NULL, BOOKMARKS_FILE)) == NULL)
     {
       printf (_("gFTP Error: Bad bookmarks file name %s\n"), BOOKMARKS_FILE);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   if (access (tempstr, F_OK) == -1)
@@ -202,7 +202,7 @@ gftp_read_bookmarks (char *global_data_path)
     {
       printf (_("gFTP Error: Cannot open bookmarks file %s: %s\n"), tempstr,
 	      g_strerror (errno));
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   g_free (tempstr);
 
@@ -303,7 +303,7 @@ gftp_read_bookmarks (char *global_data_path)
 
           newentry->num_local_options_vars++;
           newentry->local_options_vars = g_realloc (newentry->local_options_vars,
-                                                    sizeof (gftp_config_vars) * newentry->num_local_options_vars);
+                                                    (gulong) sizeof (gftp_config_vars) * newentry->num_local_options_vars);
 
           memcpy (&newentry->local_options_vars[newentry->num_local_options_vars - 1], global_entry, 
                   sizeof (newentry->local_options_vars[newentry->num_local_options_vars - 1]));
@@ -363,7 +363,7 @@ gftp_config_parse_args (char *str, int numargs, int lineno, char **first, ...)
       else
 	endpos = curpos + strlen (curpos);
 
-      *dest = g_malloc (endpos - curpos + 1);
+      *dest = g_malloc0 ((gulong) (endpos - curpos + 1));
       tempchar = *endpos;
       *endpos = '\0';
       strcpy (*dest, curpos);
@@ -395,8 +395,7 @@ gftp_config_parse_args (char *str, int numargs, int lineno, char **first, ...)
   while (numargs > 1)
     {
       dest = va_arg (argp, char **);
-      *dest = g_malloc (1);
-      **dest = '\0';
+      *dest = g_malloc0 (1);
       numargs--;
     }
   va_end (argp);
@@ -477,7 +476,7 @@ gftp_config_read_ext (char *buf, int line)
 {
   gftp_file_extensions * tempext;
 
-  tempext = g_malloc (sizeof (*tempext));
+  tempext = g_malloc0 (sizeof (*tempext));
   gftp_config_parse_args (buf, 4, line, &tempext->ext, &tempext->filename,
                           &tempext->ascii_binary, &tempext->view_program);
  
@@ -556,7 +555,8 @@ gftp_read_config_file (char *global_data_path)
     {
       if (gftp_protocols[i].shown)
         {
-          protocol_list = g_realloc (protocol_list, sizeof (char *) * (j + 2));
+          protocol_list = g_realloc (protocol_list,
+                                     (gulong) sizeof (char *) * (j + 2));
           protocol_list[j] = gftp_protocols[i].name;
           protocol_list[j + 1] = NULL;
           j++;
@@ -585,7 +585,7 @@ gftp_read_config_file (char *global_data_path)
   if ((tempstr = gftp_expand_path (NULL, CONFIG_FILE)) == NULL)
     {
       printf (_("gFTP Error: Bad config file name %s\n"), CONFIG_FILE);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   if (access (tempstr, F_OK) == -1)
@@ -597,7 +597,7 @@ gftp_read_config_file (char *global_data_path)
 	    {
 	      printf (_("gFTP Error: Could not make directory %s: %s\n"),
 		      temp1str, g_strerror (errno));
-	      exit (1);
+	      exit (EXIT_FAILURE);
 	    }
 	}
       g_free (temp1str);
@@ -608,7 +608,7 @@ gftp_read_config_file (char *global_data_path)
 	  printf (_("gFTP Error: Cannot find master config file %s\n"),
 		  temp1str);
 	  printf (_("Did you do a make install?\n"));
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
       copyfile (temp1str, tempstr);
       g_free (temp1str);
@@ -618,7 +618,7 @@ gftp_read_config_file (char *global_data_path)
     {
       printf (_("gFTP Error: Cannot open config file %s: %s\n"), CONFIG_FILE,
 	      g_strerror (errno));
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   g_free (tempstr);
  
@@ -656,7 +656,7 @@ gftp_read_config_file (char *global_data_path)
                                 tmpconfigvar, line) != 0)
             {
               printf (_("Terminating due to parse errors at line %d in the config file\n"), line);
-              exit (1);
+              exit (EXIT_FAILURE);
             }
         }
       else
@@ -669,7 +669,7 @@ gftp_read_config_file (char *global_data_path)
   if ((tempstr = gftp_expand_path (NULL, LOG_FILE)) == NULL)
     {
       printf (_("gFTP Error: Bad log file name %s\n"), LOG_FILE);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   if ((gftp_logfd = fopen (tempstr, "w")) == NULL)
@@ -704,14 +704,14 @@ write_comment (FILE * fd, const char *comment)
 	       endpos++);
 	}
       fwrite (pos, 1, endpos - pos, fd);
-      fwrite ("\n# ", 1, 3, fd);
+      fwrite ("\n", 1, 1, fd);
       if (*endpos == '\0')
 	{
 	  pos = endpos;
 	  break;
 	}
-      else
-	pos = endpos + 1;
+      pos = endpos + 1;
+      fwrite ("# ", 1, 2, fd);
     }
   if (strlen (pos) > 1)
     {
@@ -729,20 +729,20 @@ gftp_write_bookmarks_file (void)
   FILE * bmfile;
   int i;
 
-  bmhdr = N_("Bookmarks file for gFTP. Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>. Warning: Any comments that you add to this file WILL be overwritten");
+  bmhdr = N_("Bookmarks file for gFTP. Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>. Warning: Any comments that you add to this file WILL be overwritten");
   pwhdr = N_("Note: The passwords contained inside this file are scrambled. This algorithm is not secure. This is to avoid your password being easily remembered by someone standing over your shoulder while you're editing this file. Prior to this, all passwords were stored in plaintext.");
 
   if ((tempstr = gftp_expand_path (NULL, BOOKMARKS_FILE)) == NULL)
     {
       printf (_("gFTP Error: Bad bookmarks file name %s\n"), CONFIG_FILE);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   if ((bmfile = fopen (tempstr, "w+")) == NULL)
     {
       printf (_("gFTP Error: Cannot open bookmarks file %s: %s\n"),
 	      CONFIG_FILE, g_strerror (errno));
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   g_free (tempstr);
@@ -776,7 +776,7 @@ gftp_write_bookmarks_file (void)
             password = NULL;
 
           fprintf (bmfile,
-    	       "[%s]\nhostname=%s\nport=%d\nprotocol=%s\nremote directory=%s\nlocal directory=%s\nusername=%s\npassword=%s\naccount=%s\n",
+    	       "[%s]\nhostname=%s\nport=%u\nprotocol=%s\nremote directory=%s\nlocal directory=%s\nusername=%s\npassword=%s\naccount=%s\n",
     	       tempstr, tempentry->hostname == NULL ? "" : tempentry->hostname,
     	       tempentry->port, tempentry->protocol == NULL
     	       || *tempentry->protocol ==
@@ -830,19 +830,19 @@ gftp_write_config_file (void)
   if ((tempstr = gftp_expand_path (NULL, CONFIG_FILE)) == NULL)
     {
       printf (_("gFTP Error: Bad config file name %s\n"), CONFIG_FILE);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   if ((conffile = fopen (tempstr, "w+")) == NULL)
     {
       printf (_("gFTP Error: Cannot open config file %s: %s\n"), CONFIG_FILE,
 	      g_strerror (errno));
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   g_free (tempstr);
 
-  write_comment (conffile, _("Config file for gFTP. Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>. Warning: Any comments that you add to this file WILL be overwritten. If a entry has a (*) in it's comment, you can't change it inside gFTP"));
+  write_comment (conffile, _("Config file for gFTP. Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>. Warning: Any comments that you add to this file WILL be overwritten. If a entry has a (*) in it's comment, you can't change it inside gFTP"));
 
   for (templist = gftp_options_list;
        templist != NULL;
@@ -1120,7 +1120,7 @@ gftp_config_file_read_color (char *str, gftp_config_vars * cv, int line)
 
   gftp_config_parse_args (str, 3, line, &red, &green, &blue);
 
-  color = g_malloc (sizeof (*color));
+  color = g_malloc0 (sizeof (*color));
   color->red = strtol (red, NULL, 16);
   color->green = strtol (green, NULL, 16);
   color->blue = strtol (blue, NULL, 16);
@@ -1153,7 +1153,7 @@ gftp_config_file_copy_color (gftp_config_vars * cv, gftp_config_vars * dest_cv)
   if (dest_cv->flags & GFTP_CVARS_FLAGS_DYNMEM && dest_cv->value != NULL)
     g_free (dest_cv->value);
 
-  dest_cv->value = g_malloc (sizeof (gftp_color));
+  dest_cv->value = g_malloc0 (sizeof (gftp_color));
   memcpy (dest_cv->value, cv->value, sizeof (gftp_color));
   dest_cv->flags |= GFTP_CVARS_FLAGS_DYNMEM;
 }
@@ -1287,7 +1287,7 @@ gftp_lookup_global_option (const char * key, void *value)
   else
     {
       fprintf (stderr, _("FATAL gFTP Error: Config option '%s' not found in global hash table\n"), key);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 }
 
@@ -1350,7 +1350,7 @@ gftp_set_global_option (const char * key, const void *value)
   else
     {
       fprintf (stderr, _("FATAL gFTP Error: Config option '%s' not found in global hash table\n"), key);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 }
 
@@ -1392,12 +1392,12 @@ gftp_set_request_option (gftp_request * request, const char * key,
                                                key)) == NULL)
         {
           fprintf (stderr, _("FATAL gFTP Error: Config option '%s' not found in global hash table\n"), key);
-          exit (1);
+          exit (EXIT_FAILURE);
         }
       
       request->num_local_options_vars++;
       request->local_options_vars = g_realloc (request->local_options_vars, 
-                                               sizeof (gftp_config_vars) * request->num_local_options_vars);
+                                               (gulong) sizeof (gftp_config_vars) * request->num_local_options_vars);
 
       memcpy (&request->local_options_vars[request->num_local_options_vars - 1], tmpconfigvar, sizeof (*tmpconfigvar));
       _gftp_set_option_value (&request->local_options_vars[request->num_local_options_vars - 1], value);
@@ -1425,7 +1425,7 @@ gftp_set_bookmark_option (gftp_bookmarks_var * bm, const char * key,
                                                key)) == NULL)
         {
           fprintf (stderr, _("FATAL gFTP Error: Config option '%s' not found in global hash table\n"), key);
-          exit (1);
+          exit (EXIT_FAILURE);
         }
 
       /* Check to see if this is set to the same value as the global option. 
@@ -1444,7 +1444,7 @@ gftp_set_bookmark_option (gftp_bookmarks_var * bm, const char * key,
 
       bm->num_local_options_vars++;
       bm->local_options_vars = g_realloc (bm->local_options_vars, 
-                                          sizeof (gftp_config_vars) * bm->num_local_options_vars);
+                                          (gulong) sizeof (gftp_config_vars) * bm->num_local_options_vars);
 
       memcpy (&bm->local_options_vars[bm->num_local_options_vars - 1], tmpconfigvar, sizeof (*tmpconfigvar));
       _gftp_set_option_value (&bm->local_options_vars[bm->num_local_options_vars - 1], value);
@@ -1482,7 +1482,7 @@ gftp_copy_local_options (gftp_config_vars ** new_options_vars,
   *new_options_hash = g_hash_table_new (string_hash_function,
                                         string_hash_compare);
 
-  *new_options_vars = g_malloc (sizeof (gftp_config_vars) * num_local_options_vars);
+  *new_options_vars = g_malloc0 ((gulong) sizeof (gftp_config_vars) * num_local_options_vars);
   memcpy (*new_options_vars, orig_options,
           sizeof (gftp_config_vars) * num_local_options_vars);
 
