@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  fsp.c - functions interfacing with  FSP v2 protocol library              */
-/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
 /*  Copyright (C) 2004 Radim Kolar <hsn@netmag.cz>                           */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
@@ -19,13 +19,12 @@
 /*****************************************************************************/
 
 #include "gftp.h"
+static const char cvsid[] = "$Id: fsp.c 903 2007-03-16 02:44:07Z masneyb $";
 
 #define FSP_USE_SHAREMEM_AND_SEMOP 1
 #include "fsplib/fsplib.h"
 #include "fsplib/lock.h"
 
-static const char cvsid[] = "$Id: fsp.c,v 1.2 2005/01/16 16:15:05 masneyb Exp $";
-
 typedef struct fsp_protocol_data_tag
 {
   FSP_SESSION *fsp;
@@ -95,7 +94,7 @@ fsp_disconnect (gftp_request * request)
 }
 
 static off_t
-fsp_get_file (gftp_request * request, const char *filename, int fd,
+fsp_get_file (gftp_request * request, const char *filename,
                 off_t startsize)
 {
   fsp_protocol_data * lpd;
@@ -165,7 +164,7 @@ static ssize_t fsp_write_function(gftp_request *request, const char *buf, size_t
 }
 
 static int
-fsp_put_file (gftp_request * request, const char *filename, int fd,
+fsp_put_file (gftp_request * request, const char *filename,
                 off_t startsize, off_t totalsize)
 {
   fsp_protocol_data * lpd;
@@ -279,10 +278,10 @@ fsp_abort_transfer (gftp_request * request)
 
 static int
 fsp_stat_filename (gftp_request * request, const char *filename,
-                     mode_t * mode)
+                   mode_t * mode, off_t * filesize)
 {
-  struct stat st;
   fsp_protocol_data * lpd;
+  struct stat st;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->protonum == GFTP_FSP_NUM, GFTP_EFATAL);
@@ -295,6 +294,8 @@ fsp_stat_filename (gftp_request * request, const char *filename,
     return (GFTP_ERETRYABLE);
 
   *mode = st.st_mode;
+  *filesize = st.st_size;
+
   return (0);
 }
 
@@ -418,19 +419,16 @@ fsp_chdir (gftp_request * request, const char *directory)
   
   olddir=NULL;
   /* build new directory string */
-  if (request->directory != directory)
-    {
-      olddir = request->directory;
+  olddir = request->directory;
 
-      if (*directory != '/' && request->directory != NULL)
-        {
-          tempstr = g_strconcat (request->directory, "/", directory, NULL);
-          request->directory = gftp_expand_path (request, tempstr);
-          g_free (tempstr);
-        }
-      else
-        request->directory = gftp_expand_path (request, directory);
+  if (*directory != '/' && request->directory != NULL)
+    {
+      tempstr = g_strconcat (request->directory, "/", directory, NULL);
+      request->directory = gftp_expand_path (request, tempstr);
+      g_free (tempstr);
     }
+  else
+    request->directory = gftp_expand_path (request, directory);
 
   if (fsp_getpro (lpd->fsp,request->directory,NULL) == 0)
     {
@@ -634,6 +632,7 @@ fsp_init (gftp_request * request)
   request->need_password = 0;
   request->use_cache = 1;
   request->always_connected = 0;
+  request->use_local_encoding = 0;
 
   lpd = g_malloc0 (sizeof (*lpd));
   request->protocol_data = lpd;
