@@ -17,7 +17,7 @@
 /*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
 /*****************************************************************************/
 
-/* $Id: ftpcommon.h,v 1.6 2004/12/21 01:30:20 masneyb Exp $ */
+/* $Id: ftpcommon.h 855 2006-11-03 03:23:19Z masneyb $ */
 
 #include "gftp.h"
 
@@ -39,5 +39,7 @@ typedef struct rfc959_params_tag rfc959_parms;
 
 int rfc959_send_command 		( gftp_request * request,
 					  const char *command,
-					  int read_response );
+					  ssize_t command_len,
+					  int read_response,
+					  int dont_try_to_reconnect );
 
