@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  ftps.c - General purpose routines for the FTPS protocol                  */
-/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -20,7 +20,7 @@
 #include "gftp.h"
 #include "ftpcommon.h"
 
-static const char cvsid[] = "$Id: ftps.c,v 1.9 2004/09/08 00:18:09 masneyb Exp $";
+static const char cvsid[] = "$Id: ftps.c 977 2008-11-30 14:41:36Z masneyb $";
 
 #ifdef USE_SSL
 static int
@@ -60,7 +60,7 @@ ftps_auth_tls_start (gftp_request * request)
 
   params = request->protocol_data;
 
-  ret = rfc959_send_command (request, "AUTH TLS\r\n", 1);
+  ret = rfc959_send_command (request, "AUTH TLS\r\n", -1, 1, 0);
   if (ret < 0)
     return (ret);
   else if (ret != '2')
@@ -72,12 +72,11 @@ ftps_auth_tls_start (gftp_request * request)
   request->read_function = gftp_ssl_read;
   request->write_function = gftp_ssl_write;
 
-  ret = rfc959_send_command (request, "PBSZ 0\r\n", 1);
+  ret = rfc959_send_command (request, "PBSZ 0\r\n", -1, 1, 0);
   if (ret < 0)
     return (ret);
 
-  ret = '5'; /* FIXME */
-  /* ret = rfc959_send_command (request, "PROT P\r\n", 1); */
+  ret = rfc959_send_command (request, "PROT P\r\n", -1, 1, 0);
   if (ret < 0)
     return (ret);
   else if (ret == '2')
@@ -87,7 +86,7 @@ ftps_auth_tls_start (gftp_request * request)
     }
   else
     {
-      ret = rfc959_send_command (request, "PROT C\r\n", 1);
+      ret = rfc959_send_command (request, "PROT C\r\n", -1, 1, 0);
       if (ret < 0)
         return (ret);
       else if (ret != '2')
@@ -102,19 +101,8 @@ ftps_auth_tls_start (gftp_request * request)
 
   return (0);
 }
-#endif
 
-
-void
-ftps_register_module (void)
-{
-#ifdef USE_SSL
-  ssl_register_module ();
-#endif
-}
-
-
-static int
+/*@unused@*/ static int
 ftps_connect (gftp_request * request)
 {
   if (request->datafd > 0)
@@ -126,6 +114,17 @@ ftps_connect (gftp_request * request)
   return (rfc959_connect (request));
 }
 
+#endif
+
+
+void
+ftps_register_module (void)
+{
+#ifdef USE_SSL
+  ssl_register_module ();
+#endif
+}
+
 
 int
 ftps_init (gftp_request * request)
