@@ -17,7 +17,7 @@
 /*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
 /*****************************************************************************/
 
-/* $Id: gftp.h,v 1.113 2005/01/16 15:48:24 masneyb Exp $ */
+/* $Id: gftp.h 960 2008-03-28 11:44:36Z masneyb $ */
 
 #ifndef __GFTP_H
 #define __GFTP_H
@@ -105,13 +105,6 @@
 #define AF_LOCAL AF_UNIX
 #endif
 
-#ifdef HAVE_GETADDRINFO
-#define HAVE_IPV6
-#define GFTP_GET_AI_FAMILY(request)	((request) != NULL && (request)->hostp != NULL ? (request)->hostp->ai_family : -1)
-#else
-#define GFTP_GET_AI_FAMILY(request)	AF_INET
-#endif
-
 /* Solaris needs this included for major()/minor() */
 #ifdef HAVE_SYS_MKDEV_H
 #include <sys/mkdev.h>
@@ -161,18 +154,24 @@
 # define GFTP_LOG_FUNCTION_ATTRIBUTES
 #endif
 
+#if defined (HAVE_GETADDRINFO) && defined (HAVE_GAI_STRERROR)
+# define HAVE_IPV6
+#endif
+
 #if defined (_LARGEFILE_SOURCE) && !defined (__hppa__) && !defined (__hppa)
 #define GFTP_OFF_T_HEX_PRINTF_MOD	"%llx"
 #define GFTP_OFF_T_INTL_PRINTF_MOD	"%'lld"
 #define GFTP_OFF_T_PRINTF_MOD		"%lld"
 #define GFTP_OFF_T_11PRINTF_MOD		"%11lld"
 #define gftp_parse_file_size(str)	strtoll (str, NULL, 10)
+#define GFTP_OFF_T_PRINTF_CONVERSION	long long
 #else
 #define GFTP_OFF_T_HEX_PRINTF_MOD	"%lx"
 #define GFTP_OFF_T_INTL_PRINTF_MOD	"%'ld"
 #define GFTP_OFF_T_PRINTF_MOD		"%ld"
 #define GFTP_OFF_T_11PRINTF_MOD		"%11ld"
 #define gftp_parse_file_size(str)	strtol (str, NULL, 10)
+#define GFTP_OFF_T_PRINTF_CONVERSION	off_t
 #endif
 
 /* Server types (used by FTP protocol from SYST command) */
@@ -231,13 +230,10 @@ typedef struct gftp_file_tag gftp_file;
 
 struct gftp_file_tag 
 {
-  char *file,			/* Our filename */
-       *utf8_file,		/* UTF-8 encoded filename for display purposes
-				   only. This is only set if file is not in 
-				   UTF-8 */
-       *user,			/* User that owns it */
-       *group,			/* Group that owns it */
-       *destfile;		/* Full pathname to the destination for the 
+  /*@null@*/ char *file,	/* Our filename */
+                  *user,	/* User that owns it */
+                  *group,	/* Group that owns it */
+                  *destfile;	/* Full pathname to the destination for the 
                                    file transfer */
 
   int fd;			/* Already open fd for this file */
@@ -247,6 +243,11 @@ struct gftp_file_tag
   off_t size,			/* Size of the file */
         startsize;		/* Size to start the transfer at */
   mode_t st_mode;		/* File attributes */
+
+  dev_t st_dev;			/* The device and associated inode. These */
+  ino_t st_ino;			/* two fields are used for detecting loops */
+				/* with symbolic links. */
+
   unsigned int selected : 1,	/* Is this file selected? */
                was_sel : 1,	/* Was this file selected before  */
                shown : 1,	/* Is this file shown? */
@@ -254,9 +255,14 @@ struct gftp_file_tag
                done_edit : 1,	/* Edit the file when done transfering? */
                done_rm : 1,	/* Remove the file when done */
                transfer_done : 1, /* Is current file transfer done? */
-               is_fd : 1;	/* Is this a file descriptor? */
+               retry_transfer : 1, /* Is current file transfer done? */
+               exists_other_side : 1, /* The file exists on the other side
+                                         during the file transfer */
+               filename_utf8_encoded : 1; /* Is the filename properly UTF8
+                                             encoded? */
+
   char transfer_action;		/* See the GFTP_TRANS_ACTION_* vars above */
-  void *user_data;
+  /*@null@*/ void *user_data;
 };
 
 
@@ -309,26 +315,26 @@ typedef struct gftp_config_list_vars_tag
 
 typedef struct gftp_config_vars_tag
 {
-  char *key,			/* variable name */
-       *description;		/* How this field will show up in the dialog */
+  /*@null@*/ char *key,		/* variable name */
+                  *description;	/* How this field will show up in the dialog */
   int otype;			/* Type of option this is */
-  void *value;
-  void *listdata;		/* For options that have several different 
+  /*@null@*/ void *value;
+  /*@null@*/ void *listdata;	/* For options that have several different 
 				   options, this is a list of all the options.
 				   Each option_type that uses this will use this
 				   field differently */
   int flags;			/* See GFTP_CVARS_FLAGS_* above */
-  char *comment;                /* Comment to write out to the config file */
+  /*@null@*/ char *comment;     /* Comment to write out to the config file */
   int ports_shown;		/* What ports of gFTP is this option shown in */
-  void *user_data;		/* Data that the GUI can store here (Widget in gtk+) */
+  /*@null@*/ void *user_data;	/* Data that the GUI can store here (Widget in gtk+) */
 } gftp_config_vars;
 
 
 typedef struct gftp_option_type_tag
 {
   int (*read_function) (char *str, gftp_config_vars * cv, int line);
-  int (*write_function) (gftp_config_vars * cv, char *buf, size_t buflen,
-                         int to_config_file);
+  int (*write_function) (gftp_config_vars * cv, /*@out@*/ char *buf,
+                         size_t buflen, int to_config_file);
   void (*copy_function) (gftp_config_vars * cv, gftp_config_vars * dest_cv);
   int (*compare_function) (gftp_config_vars * cv1, gftp_config_vars * cv2);
   void *(*ui_print_function) (gftp_config_vars * cv, void *user_data, void *value);
@@ -342,8 +348,8 @@ typedef struct gftp_option_type_tag
 
 typedef struct gftp_textcomboedt_data_tag
 {
-  char *description,
-       *text;
+  /*@null@*/ char *description,
+                  *text;
   int flags;
 } gftp_textcomboedt_data;
 
@@ -351,7 +357,7 @@ typedef struct gftp_textcomboedt_data_tag
 typedef struct gftp_request_tag gftp_request;
 
 typedef void (*gftp_logging_func)		( gftp_logging_level level, 
-						  gftp_request * request, 
+						  /*@null@*/ gftp_request * request, 
 						  const char *string, ... );
 
 #define GFTP_ANONYMOUS_USER			"anonymous"
@@ -379,22 +385,13 @@ struct gftp_request_tag
   int wakeup_main_thread[2];	/* FD that gets written to by the threads
                                    to wakeup the parent */
 
-  /* One of these are used to lookup the IP address of the host we are
-     connecting to */
-#if defined (HAVE_GETADDRINFO) && defined (HAVE_GAI_STRERROR)
-  struct addrinfo *hostp,
-                  *current_hostp;
-#else
-  struct hostent host, *hostp;
-  int curhost;
-#endif
-
+  void *remote_addr;
+  size_t remote_addr_len;
   int ai_family;
+
   int server_type;		/* The type of server we are connected to.
                                    See GFTP_DIRTYPE_* above */
-  unsigned int free_hostp : 1, 		/* Should we free the hostp structure 
-					   in gftp_destroy_request() */
-               use_proxy : 1,
+  unsigned int use_proxy : 1,
                always_connected : 1,
                need_hostport : 1,
                need_username : 1,
@@ -403,7 +400,9 @@ struct gftp_request_tag
                cached : 1,              /* Is this directory listing cached? */
                cancel : 1,		/* If a signal is received, should
 					   we cancel this operation */
-               stopable : 1;
+               stopable : 1,
+               refreshing : 1,
+               use_local_encoding : 1;
 
   off_t gotbytes;
  
@@ -429,14 +428,12 @@ struct gftp_request_tag
   void (*disconnect) 			( gftp_request * request );
   off_t (*get_file) 			( gftp_request * request, 
 					  const char *filename, 
-					  int fd,
 					  off_t startsize );
   int (*put_file) 			( gftp_request * request, 
 					  const char *filename, 
-					  int fd,
 					  off_t startsize,
 					  off_t totalsize );
-  long (*transfer_file) 		( gftp_request * fromreq, 
+  off_t (*transfer_file) 		( gftp_request * fromreq, 
 					  const char *fromfile, 
 					  off_t fromsize, 
 					  gftp_request * toreq, 
@@ -452,7 +449,8 @@ struct gftp_request_tag
   int (*abort_transfer) 		( gftp_request * request );
   int (*stat_filename) 			( gftp_request * request,
 					  const char *filename,
-					  mode_t * mode );
+					  mode_t * mode,
+					  off_t * filesize );
   int (*list_files) 			( gftp_request * request );
   int (*get_next_file)			( gftp_request * request, 
 					  gftp_file *fle, 
@@ -497,8 +495,10 @@ struct gftp_request_tag
 #endif
 
 #if GLIB_MAJOR_VERSION > 1
-  GIConv iconv; 
-  unsigned int iconv_initialized : 1;
+  GIConv iconv_to, iconv_from; 
+  unsigned int iconv_from_initialized : 1,
+               iconv_to_initialized : 1;
+  char *iconv_charset;
 #endif
 };
 
@@ -549,6 +549,7 @@ typedef struct gftp_transfer_tag
                structmutex;
 
   void *user_data;
+  void *thread_id;
   void *clist;
 } gftp_transfer;
 
@@ -562,10 +563,10 @@ typedef struct gftp_log_tag
 
 typedef struct supported_gftp_protocols_tag 
 {
-  char *name;					/* Description of protocol */
-  int (*init) (gftp_request * request);		/* Init function */
-  void (*register_options) (void);		/* Protocol options */
-  char *url_prefix;				/* URL Prefix */
+  /*@null@*/ char *name;			/* Description of protocol */
+  /*@null@*/ int (*init) (gftp_request * request); /* Init function */
+  /*@null@*/ void (*register_options) (void);	/* Protocol options */
+  /*@null@*/ char *url_prefix;			/* URL Prefix */
   unsigned int default_port;			/* Default port */
   unsigned int shown : 1,			/* Whether this protocol is 
                                                    shown or not to the user in 
@@ -634,15 +635,15 @@ typedef struct gftp_getline_buffer_tag
 
 
 /* Global config options. These are defined in options.h */
-extern GList * gftp_file_transfers, * gftp_file_transfer_logs,
-             * gftp_options_list;
-extern GHashTable * gftp_global_options_htable, * gftp_bookmarks_htable, 
-                  * gftp_config_list_htable;
+/*@null@*/ extern GList * gftp_file_transfers, * gftp_file_transfer_logs,
+                        * gftp_options_list;
+/*@null@*/ extern GHashTable * gftp_global_options_htable, * gftp_bookmarks_htable, 
+                             * gftp_config_list_htable;
+/*@null@*/ extern gftp_bookmarks_var * gftp_bookmarks;
+/*@null@*/ extern FILE * gftp_logfd;
 extern gftp_config_vars gftp_global_config_vars[];
 extern supported_gftp_protocols gftp_protocols[];
-extern gftp_bookmarks_var * gftp_bookmarks;
 extern char gftp_version[];
-extern FILE * gftp_logfd;
 extern int gftp_configuration_changed;
 
 /* This is defined in config_file.c */
@@ -651,7 +652,7 @@ extern gftp_option_type_var gftp_option_types[];
 
 /* cache.c */
 void gftp_generate_cache_description 	( gftp_request * request, 
-					  char *description,
+					  /*@out@*/ char *description,
 					  size_t len, 
 					  int ignore_directory );
 
@@ -665,11 +666,29 @@ void gftp_delete_cache_entry 		( gftp_request * request,
 					  char *descr,
 					  int ignore_directory );
 
+/* charset-conv.c */
+/*@null@*/ char * gftp_string_to_utf8	( gftp_request * request, 
+					  const char *str,
+					  size_t *dest_len );
+
+/*@null@*/ char * gftp_string_from_utf8	( gftp_request * request, 
+					  int force_local,
+					  const char *str,
+					  size_t *dest_len );
+
+/*@null@*/ char * gftp_filename_to_utf8	( gftp_request * request, 
+					  const char *str,
+					  size_t *dest_len );
+
+/*@null@*/ char * gftp_filename_from_utf8 ( gftp_request * request, 
+					  const char *str,
+					  size_t *dest_len );
+
 /* config_file.c */
 int gftp_config_parse_args 		( char *str, 
 					  int numargs, 
 					  int lineno, 
-					  char **first, 
+					  /*@out@*/ char **first, 
 					  ... );
 
 void gftp_add_bookmark 			( gftp_bookmarks_var * newentry );
@@ -685,11 +704,12 @@ GHashTable * build_bookmarks_hash_table	( gftp_bookmarks_var * entry );
 void print_bookmarks 			( gftp_bookmarks_var * bookmarks );
 
 void gftp_lookup_global_option 		( const char * key, 
-					  void *value );
+					  /*@out@*/ void *value );
 
 void gftp_lookup_request_option 	( gftp_request * request, 
 					  const char * key, 
-					  void *value );
+					  /*@out@*/ void *value );
+
 
 void gftp_lookup_bookmark_option 	( gftp_bookmarks_var * bm, 
 					  const char * key, 
@@ -725,7 +745,7 @@ void gftp_free_proxy_hosts 		( GList * proxy_hosts );
 GList * gftp_copy_proxy_hosts 		( GList * proxy_hosts );
 
 /* misc.c */
-char *insert_commas 			( off_t number, 
+/*@null@*/ char *insert_commas 		( off_t number, 
 					  char *dest_str, 
 					  size_t dest_len );
 
@@ -736,7 +756,8 @@ char *gftp_expand_path 			( gftp_request * request,
 
 void make_nonnull 			( char **str );
 
-int gftp_match_filespec 		( const char *filename, 
+int gftp_match_filespec 		( gftp_request * request,
+					  const char *filename, 
 					  const char *filespec );
 
 int gftp_parse_command_line 		( int *argc,
@@ -749,6 +770,11 @@ gint string_hash_compare 		( gconstpointer path1,
 
 guint string_hash_function		( gconstpointer key );
 
+gint uint_hash_compare 			( gconstpointer path1,
+					  gconstpointer path2 );
+
+guint uint_hash_function		( gconstpointer key );
+
 void free_file_list			( GList * filelist );
 
 gftp_file * copy_fdata 			( gftp_file * fle );
@@ -767,7 +793,8 @@ GList * gftp_sort_filelist 		( GList * filelist,
 					  int column, 
 					  int asds );
 
-char * gftp_gen_ls_string 		( gftp_file * fle, 
+char * gftp_gen_ls_string 		( gftp_request * request,
+					  gftp_file * fle, 
 					  char *file_prefixstr, 
 					  char *file_suffixstr );
 
@@ -801,6 +828,15 @@ char * gftp_get_share_dir 		( void );
 int mkstemps 				( char *template,
 					  int suffix_len );
 
+/* parse-dir-listing.c */
+time_t parse_time 			( char *str,
+					  char **endpos );
+
+int gftp_parse_ls 			( gftp_request * request,
+					  const char *lsoutput, 
+					  gftp_file *fle,
+					  int fd );
+
 /* protocols.c */
 #define GFTP_FTP_NUM				0
 #define GFTP_FTPS_NUM				1
@@ -874,22 +910,18 @@ void gftp_disconnect 			( gftp_request * request );
 
 off_t gftp_get_file 			( gftp_request * request, 
 					  const char *filename, 
-					  int fd,
 					  off_t startsize );
 
 int gftp_put_file 			( gftp_request * request, 
 					  const char *filename, 
-					  int fd,
 					  off_t startsize,
 					  off_t totalsize );
 
 off_t gftp_transfer_file 		( gftp_request *fromreq, 
 					  const char *fromfile, 
-					  int fromfd,
 					  off_t fromsize, 
 					  gftp_request *toreq, 
 					  const char *tofile, 
-					  int tofd,
 					  off_t tosize );
 
 ssize_t gftp_get_next_file_chunk 	( gftp_request * request, 
@@ -902,12 +934,6 @@ ssize_t gftp_put_next_file_chunk 	( gftp_request * request,
 
 int gftp_list_files 			( gftp_request * request );
 
-char * gftp_string_to_utf8		( gftp_request * request, 
-					  const char *str );
-
-char * gftp_string_from_utf8		( gftp_request * request, 
-					  const char *str );
-
 int gftp_parse_bookmark 		( gftp_request * request, 
 					  gftp_request * local_request,
 					  const char * bookmark,
@@ -926,7 +952,8 @@ int gftp_abort_transfer 		( gftp_request * request );
 
 int gftp_stat_filename			( gftp_request * request,
 					  const char *filename,
-					  mode_t * mode );
+					  mode_t * mode,
+					  off_t * filesize );
 
 void gftp_set_hostname 			( gftp_request * request, 
 					  const char *hostname );
@@ -967,7 +994,7 @@ int gftp_set_file_time 			( gftp_request * request,
 					  const char *file, 
 					  time_t datetime );
 
-char gftp_site_cmd 			( gftp_request * request, 
+int gftp_site_cmd 			( gftp_request * request, 
 					  int specify_site,
 					  const char *command );
 
@@ -977,67 +1004,14 @@ off_t gftp_get_file_size 		( gftp_request * request,
 void gftp_calc_kbs 			( gftp_transfer * tdata, 
 				 	  ssize_t num_read );
 
-time_t parse_time 			( char *str,
-					  char **endpos );
-
-int gftp_parse_ls 			( gftp_request * request,
-					  const char *lsoutput, 
-					  gftp_file *fle,
-					  int fd );
-
 int gftp_get_all_subdirs 		( gftp_transfer * transfer, 
 					  void (*update_func) 
 						( gftp_transfer * transfer ));
 
-int gftp_connect_server 		( gftp_request * request, 
-					  char *service,
-					  char *proxy_hostname,
-					  unsigned int proxy_port );
-
-#if !defined (HAVE_GETADDRINFO) || !defined (HAVE_GAI_STRERROR)
-
-struct hostent *r_gethostbyname 	( const char *name, 
-					  struct hostent *result_buf, 
-					  int *h_errnop );
-
-#endif
-
-struct servent *r_getservbyname 	( const char *name, 
-					  const char *proto,
-					  struct servent *result_buf, 
-					  int *h_errnop );
-
 int gftp_set_config_options 		( gftp_request * request );
 
 void print_file_list 			( GList * list );
 
-void gftp_free_getline_buffer 		( gftp_getline_buffer ** rbuf );
-
-ssize_t gftp_get_line 			( gftp_request * request, 
-					  gftp_getline_buffer ** rbuf,
-					  char * str, 
-					  size_t len, 
-					  int fd );
-
-ssize_t gftp_fd_read 			( gftp_request * request, 
-					  void *ptr, 
-					  size_t size, 
-					  int fd );
-
-ssize_t gftp_fd_write 			( gftp_request * request, 
-					  const char *ptr, 
-					  size_t size, 
-					  int fd );
-
-ssize_t gftp_writefmt 			( gftp_request * request, 
-					  int fd, 
-					  const char *fmt, 
-					  ... );
-
-int gftp_fd_set_sockblocking 		( gftp_request * request, 
-					  int fd, 
-					  int non_blocking );
-
 void gftp_swap_socks 			( gftp_request * dest, 
 					  gftp_request * source );
 
@@ -1052,7 +1026,8 @@ int gftp_fd_open 			( gftp_request * request,
 					  int flags,
 					  mode_t perms );
 
-void gftp_setup_startup_directory 	( gftp_request * request );
+void gftp_setup_startup_directory 	( gftp_request * request,
+					  const char *option_name );
 
 unsigned int gftp_protocol_default_port	( gftp_request * request );
 
@@ -1102,3 +1077,65 @@ void gftpui_protocol_update_timeout 	( gftp_request * request );
 
 #endif
 
+/* socket-connect.c */
+int gftp_connect_server 		( gftp_request * request, 
+					  char *service,
+					  char *proxy_hostname,
+					  unsigned int proxy_port );
+
+/* socket-connect-getaddrinfo.c */
+struct addrinfo * lookup_host_with_getaddrinfo
+					( gftp_request *request,
+					  char *service,
+					  char *proxy_hostname,
+					  int proxy_port );
+
+int gftp_connect_server_with_getaddrinfo
+					( gftp_request * request,
+					  char *service,
+					  char *proxy_hostname,
+					  unsigned int proxy_port );
+
+/* socket-connect-gethostbyname.c */
+int lookup_host_with_gethostbyname	( gftp_request *request,
+					  char *proxy_hostname,
+					  struct hostent *hostp );
+
+int gftp_connect_server_legacy		( gftp_request * request,
+					  char *service,
+					  char *proxy_hostname,
+					  unsigned int proxy_port );
+
+/* sockutils.c */
+ssize_t gftp_get_line 			( gftp_request * request, 
+					  /*@out@*/ gftp_getline_buffer ** rbuf,
+					  /*@out@*/ char * str, 
+					  size_t len, 
+					  int fd );
+
+void gftp_free_getline_buffer 		( gftp_getline_buffer ** rbuf );
+
+ssize_t gftp_fd_read 			( gftp_request * request, 
+					  void *ptr, 
+					  size_t size, 
+					  int fd );
+
+ssize_t gftp_fd_write 			( gftp_request * request, 
+					  const char *ptr, 
+					  size_t size, 
+					  int fd );
+
+ssize_t gftp_writefmt 			( gftp_request * request, 
+					  int fd, 
+					  const char *fmt, 
+					  ... );
+
+int gftp_fd_set_sockblocking 		( gftp_request * request, 
+					  int fd, 
+					  int non_blocking );
+
+struct servent * r_getservbyname	( const char *name,
+					  const char *proto,
+					  struct servent *result_buf,
+					  int *h_errnop );
+
