@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  local.c - functions that will use the local system                       */
-/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp.h"
-static const char cvsid[] = "$Id: local.c,v 1.39 2004/09/14 10:16:52 masneyb Exp $";
+static const char cvsid[] = "$Id: local.c 926 2007-04-27 00:40:52Z masneyb $";
 
 typedef struct local_protocol_data_tag
 {
@@ -51,41 +51,83 @@ local_destroy (gftp_request * request)
 }
 
 
+static int
+local_getcwd (gftp_request * request)
+{
+  char tempstr[PATH_MAX], *utf8;
+  size_t destlen;
+  
+  if (request->directory != NULL)
+    g_free (request->directory);
+
+  if (getcwd (tempstr, sizeof (tempstr)) == NULL)
+    {
+      request->logging_function (gftp_logging_error, request,
+                                 _("Could not get current working directory: %s\n"),
+                                 g_strerror (errno));
+      request->directory = NULL;
+      return (GFTP_ERETRYABLE);
+    }
+
+  utf8 = gftp_filename_to_utf8 (request, tempstr, &destlen);
+  if (utf8 != NULL)
+    request->directory = utf8;
+  else
+    request->directory = g_strdup (tempstr);
+
+  return (0);
+}
+
+
+static int
+local_chdir (gftp_request * request, const char *directory)
+{
+  size_t destlen;
+  char *utf8;
+  int ret;
+
+  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
+  g_return_val_if_fail (directory != NULL, GFTP_EFATAL);
+
+  utf8 = gftp_filename_from_utf8 (request, directory, &destlen);
+  if (utf8 != NULL)
+    {
+      ret = chdir (utf8);
+      g_free (utf8);
+    }
+  else
+    ret = chdir (directory);
+
+  if (ret == 0)
+    {
+      request->logging_function (gftp_logging_misc, request,
+                          _("Successfully changed local directory to %s\n"),
+                          directory);
+      ret = local_getcwd (request);
+    }
+  else
+    {
+      request->logging_function (gftp_logging_error, request,
+                                 _("Could not change local directory to %s: %s\n"),
+                                 directory, g_strerror (errno));
+      ret = GFTP_ERETRYABLE;
+    }
+
+  return (ret);
+}
+
+
 static int
 local_connect (gftp_request * request)
 {
-  char tempstr[PATH_MAX];
-
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
 
   if (request->directory != NULL)
-    {
-      if (getcwd (tempstr, sizeof (tempstr)) != NULL &&
-          strcmp (tempstr, request->directory) == 0)
-        return (0);
-
-      if (chdir (request->directory) != 0)
-        {
-          request->logging_function (gftp_logging_error, request,
-                             _("Could not change local directory to %s: %s\n"),
-                             request->directory, g_strerror (errno));
-        }
-      g_free (request->directory);
-      request->directory = NULL;
-    }
-
-  if (getcwd (tempstr, sizeof (tempstr)) != NULL)
-    {
-      tempstr[sizeof (tempstr) - 1] = '\0';
-      request->directory = g_strdup (tempstr);
-    }
+    return (local_chdir (request, request->directory));
   else
-    request->logging_function (gftp_logging_error, request,
-                             _("Could not get current working directory: %s\n"),
-                             g_strerror (errno));
-
-  return (0);
+    return (local_getcwd (request));
 }
 
 
@@ -107,9 +149,11 @@ local_disconnect (gftp_request * request)
 
 
 static off_t
-local_get_file (gftp_request * request, const char *filename, int fd,
+local_get_file (gftp_request * request, const char *filename,
                 off_t startsize)
 {
+  size_t destlen;
+  char *utf8;
   off_t size;
   int flags;
 
@@ -117,18 +161,22 @@ local_get_file (gftp_request * request, const char *filename, int fd,
   g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
   g_return_val_if_fail (filename != NULL, GFTP_EFATAL);
 
-  if (fd <= 0)
-    {
-      flags = O_RDONLY;
+  flags = O_RDONLY;
 #if defined (_LARGEFILE_SOURCE) && defined (O_LARGEFILE)
-      flags |= O_LARGEFILE;
+  flags |= O_LARGEFILE;
 #endif
 
-      if ((request->datafd = gftp_fd_open (request, filename, flags, 0)) == -1)
-        return (GFTP_ERETRYABLE); 
+  utf8 = gftp_filename_from_utf8 (request, filename, &destlen);
+  if (utf8 != NULL)
+    {
+      request->datafd = gftp_fd_open (request, utf8, flags, 0);
+      g_free (utf8);
     }
   else
-    request->datafd = fd;
+    request->datafd = gftp_fd_open (request, filename, flags, 0);
+
+  if (request->datafd == -1)
+    return (GFTP_ERETRYABLE); 
 
   if ((size = lseek (request->datafd, 0, SEEK_END)) == -1)
     {
@@ -153,29 +201,36 @@ local_get_file (gftp_request * request, const char *filename, int fd,
 
 
 static int
-local_put_file (gftp_request * request, const char *filename, int fd,
+local_put_file (gftp_request * request, const char *filename,
                 off_t startsize, off_t totalsize)
 {
-  int flags;
+  int flags, perms;
+  size_t destlen;
+  char *utf8;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
   g_return_val_if_fail (filename != NULL, GFTP_EFATAL);
 
-  if (fd <= 0)
-    {
-      flags = O_WRONLY | O_CREAT;
-      if (startsize > 0)
-         flags |= O_APPEND;
+  flags = O_WRONLY | O_CREAT;
+  if (startsize > 0)
+     flags |= O_APPEND;
 #if defined (_LARGEFILE_SOURCE) && defined (O_LARGEFILE)
-      flags |= O_LARGEFILE;
+  flags |= O_LARGEFILE;
 #endif
 
-      if ((request->datafd = gftp_fd_open (request, filename, flags, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1)
-        return (GFTP_ERETRYABLE);
+  perms = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
+  utf8 = gftp_filename_from_utf8 (request, filename, &destlen);
+  if (utf8 != NULL)
+    {
+      request->datafd = gftp_fd_open (request, utf8, flags, perms);
+      g_free (utf8);
     }
   else
-    request->datafd = fd;
+    request->datafd = gftp_fd_open (request, filename, flags, perms);
+
+  if (request->datafd == -1)
+    return (GFTP_ERETRYABLE);
 
   if (ftruncate (request->datafd, startsize) == -1)
     {
@@ -226,14 +281,28 @@ local_end_transfer (gftp_request * request)
 
 static int
 local_stat_filename (gftp_request * request, const char *filename,
-                     mode_t * mode)
+                     mode_t * mode, off_t * filesize)
 {
   struct stat st;
+  size_t destlen;
+  char *utf8;
+  int ret;
 
-  if (stat (filename, &st) != 0)
+  utf8 = gftp_filename_from_utf8 (request, filename, &destlen);
+  if (utf8 != NULL)
+    {
+      ret = stat (utf8, &st);
+      g_free (utf8);
+    }
+  else
+    ret = stat (filename, &st);
+
+  if (ret != 0)
     return (GFTP_ERETRYABLE);
 
   *mode = st.st_mode;
+  *filesize = st.st_size;
+
   return (0);
 }
 
@@ -303,6 +372,8 @@ local_get_next_file (gftp_request * request, gftp_file * fle, int fd)
       g_hash_table_insert (lpd->grouphash, GUINT_TO_POINTER (st.st_gid), group);
     }
 
+  fle->st_dev = fst.st_dev;
+  fle->st_ino = fst.st_ino;
   fle->st_mode = fst.st_mode;
   fle->datetime = st.st_mtime;
 
@@ -319,8 +390,8 @@ static int
 local_list_files (gftp_request * request)
 {
   local_protocol_data *lpd;
-  char *tempstr;
-  int freeit;
+  char *dir, *utf8;
+  size_t destlen;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->directory != NULL, GFTP_EFATAL);
@@ -331,30 +402,31 @@ local_list_files (gftp_request * request)
   g_return_val_if_fail (lpd != NULL, GFTP_EFATAL);
 
   if (request->directory[strlen (request->directory) - 1] != '/')
+    dir = g_strconcat (request->directory, "/", NULL);
+  else
+    dir = request->directory;
+
+  utf8 = gftp_filename_from_utf8 (request, dir, &destlen);
+  if (utf8 != NULL)
     {
-      tempstr = g_strconcat (request->directory, "/", NULL);
-      freeit = 1;
+      lpd->dir = opendir (utf8);
+      g_free (utf8);
     }
   else
-    {
-      tempstr = request->directory;
-      freeit = 0;
-    }
+    lpd->dir = opendir (dir);
 
-  if ((lpd->dir = opendir (tempstr)) == NULL)
+  if (dir != request->directory)
+    g_free (dir);
+
+  if (lpd->dir == NULL)
     {
       request->logging_function (gftp_logging_error, request,
                            _("Could not get local directory listing %s: %s\n"),
-                           tempstr, g_strerror (errno));
-      if (freeit)
-        g_free (tempstr);
+                           request->directory, g_strerror (errno));
       return (GFTP_ERETRYABLE);
     }
-
-  if (freeit)
-    g_free (tempstr);
-
-  return (0);
+  else
+    return (0);
 }
 
 
@@ -362,63 +434,47 @@ static off_t
 local_get_file_size (gftp_request * request, const char *filename)
 {
   struct stat st;
+  size_t destlen;
+  char *utf8;
+  int ret;
 
-  if (stat (filename, &st) == -1)
-    return (GFTP_ERETRYABLE);
-  return (st.st_size);
-}
-
-
-static int
-local_chdir (gftp_request * request, const char *directory)
-{
-  char tempstr[255];
-
-  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
-  g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
-  g_return_val_if_fail (directory != NULL, GFTP_EFATAL);
-
-  if (chdir (directory) == 0)
+  utf8 = gftp_filename_from_utf8 (request, filename, &destlen);
+  if (utf8 != NULL)
     {
-      request->logging_function (gftp_logging_misc, request,
-                          _("Successfully changed local directory to %s\n"),
-                          directory);
-
-      if (request->directory != directory)
-        {
-          if (getcwd (tempstr, sizeof (tempstr)) == NULL)
-            {
-              request->logging_function (gftp_logging_error, request,
-                             _("Could not get current working directory: %s\n"),
-                             g_strerror (errno));
-	      return (GFTP_ERETRYABLE);
-            }
-
-          if (request->directory)
-            g_free (request->directory);
-          request->directory = g_strdup (tempstr);
-        }
-
-      return (0);
+      ret = stat (utf8, &st);
+      g_free (utf8);
     }
   else
-    {
-      request->logging_function (gftp_logging_error, request,
-                              _("Could not change local directory to %s: %s\n"),
-                              directory, g_strerror (errno));
-      return (GFTP_ERETRYABLE);
-    }
+    ret = stat (filename, &st);
+
+  if (ret == -1)
+    return (GFTP_ERETRYABLE);
+
+  return (st.st_size);
 }
 
 
 static int
 local_rmdir (gftp_request * request, const char *directory)
 {
+  size_t destlen;
+  char *utf8;
+  int ret;
+
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
   g_return_val_if_fail (directory != NULL, GFTP_EFATAL);
 
-  if (rmdir (directory) == 0)
+  utf8 = gftp_filename_from_utf8 (request, directory, &destlen);
+  if (utf8 != NULL)
+    {
+      ret = rmdir (utf8);
+      g_free (utf8);
+    }
+  else
+    ret = rmdir (directory);
+
+  if (ret == 0)
     {
       request->logging_function (gftp_logging_misc, request,
                                  _("Successfully removed %s\n"), directory);
@@ -437,11 +493,24 @@ local_rmdir (gftp_request * request, const char *directory)
 static int
 local_rmfile (gftp_request * request, const char *file)
 {
+  size_t destlen;
+  char *utf8;
+  int ret;
+
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
   g_return_val_if_fail (file != NULL, GFTP_EFATAL);
 
-  if (unlink (file) == 0)
+  utf8 = gftp_filename_from_utf8 (request, file, &destlen);
+  if (utf8 != NULL)
+    {
+      ret = unlink (utf8);
+      g_free (utf8);
+    }
+  else
+    ret = unlink (file);
+
+  if (ret == 0)
     {
       request->logging_function (gftp_logging_misc, request,
                                  _("Successfully removed %s\n"), file);
@@ -460,11 +529,26 @@ local_rmfile (gftp_request * request, const char *file)
 static int
 local_mkdir (gftp_request * request, const char *directory)
 {
+  int ret, perms;
+  size_t destlen;
+  char *utf8;
+
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
   g_return_val_if_fail (directory != NULL, GFTP_EFATAL);
 
-  if (mkdir (directory, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) == 0)
+  perms = S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH;
+
+  utf8 = gftp_filename_from_utf8 (request, directory, &destlen);
+  if (utf8 != NULL)
+    {
+      ret = mkdir (utf8, perms);
+      g_free (utf8);
+    }
+  else
+    ret = mkdir (directory, perms);
+
+  if (ret == 0)
     {
       request->logging_function (gftp_logging_misc, request,
                                  _("Successfully made directory %s\n"),
@@ -485,36 +569,66 @@ static int
 local_rename (gftp_request * request, const char *oldname,
 	      const char *newname)
 {
+  const char *conv_oldname, *conv_newname;
+  char *old_utf8, *new_utf8;
+  size_t destlen;
+  int ret;
+
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
   g_return_val_if_fail (oldname != NULL, GFTP_EFATAL);
   g_return_val_if_fail (newname != NULL, GFTP_EFATAL);
 
-  if (rename (oldname, newname) == 0)
+  old_utf8 = gftp_filename_from_utf8 (request, oldname, &destlen);
+  conv_oldname = old_utf8 != NULL ? old_utf8 : oldname;
+  new_utf8 = gftp_filename_from_utf8 (request, newname, &destlen);
+  conv_newname = new_utf8 != NULL ? new_utf8 : newname;
+
+  if (rename (conv_oldname, conv_newname) == 0)
     {
       request->logging_function (gftp_logging_misc, request,
                                  _("Successfully renamed %s to %s\n"),
                                  oldname, newname);
-      return (0);
+      ret = 0;
     }
   else
     {
       request->logging_function (gftp_logging_error, request,
                                  _("Error: Could not rename %s to %s: %s\n"),
                                  oldname, newname, g_strerror (errno));
-      return (GFTP_ERETRYABLE);
+      ret = GFTP_ERETRYABLE;
     }
+
+  if (old_utf8 != NULL)
+    g_free (old_utf8);
+  if (new_utf8 != NULL)
+    g_free (new_utf8);
+
+  return (ret);
 }
 
 
 static int
 local_chmod (gftp_request * request, const char *file, mode_t mode)
 {
+  size_t destlen;
+  char *utf8;
+  int ret;
+
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
   g_return_val_if_fail (file != NULL, GFTP_EFATAL);
 
-  if (chmod (file, mode) == 0) 
+  utf8 = gftp_filename_from_utf8 (request, file, &destlen);
+  if (utf8 != NULL)
+    {
+      ret = chmod (utf8, mode);
+      g_free (utf8);
+    }
+  else
+    ret = chmod (file, mode);
+
+  if (ret == 0) 
     {
       request->logging_function (gftp_logging_misc, request, 
                                  _("Successfully changed mode of %s to %o\n"),
@@ -536,6 +650,9 @@ local_set_file_time (gftp_request * request, const char *file,
 		     time_t datetime)
 {
   struct utimbuf time_buf;
+  size_t destlen;
+  char *utf8;
+  int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
@@ -543,21 +660,30 @@ local_set_file_time (gftp_request * request, const char *file,
 
   time_buf.modtime = datetime;
   time_buf.actime = datetime;
-  return (utime (file, &time_buf) == 0 ? 0 : GFTP_ERETRYABLE);
-}
 
+  utf8 = gftp_filename_from_utf8 (request, file, &destlen);
+  if (utf8 != NULL)
+    {
+      ret = utime (utf8, &time_buf);
+      g_free (utf8);
+    }
+  else
+    ret = utime (file, &time_buf);
 
-static gint
-hash_compare (gconstpointer path1, gconstpointer path2)
-{
-  return (GPOINTER_TO_UINT (path1) == GPOINTER_TO_UINT (path2));
-}
-
-
-static guint
-hash_function (gconstpointer key)
-{
-  return (GPOINTER_TO_UINT (key));
+  if (ret == 0)
+    {
+      request->logging_function (gftp_logging_misc, request,
+                                 _("Successfully changed the time stamp of %s\n"),
+                                 file);
+      return (0);
+    }
+  else
+    {
+      request->logging_function (gftp_logging_error, request,
+                                 _("Error: Could not change the time stamp of %s: %s\n"),
+                                 file, g_strerror (errno));
+      return (GFTP_ERETRYABLE);
+    }
 }
 
 
@@ -612,11 +738,12 @@ local_init (gftp_request * request)
   request->need_password = 0;
   request->use_cache = 0;
   request->always_connected = 1;
+  request->use_local_encoding = 1;
 
   lpd = g_malloc0 (sizeof (*lpd));
   request->protocol_data = lpd;
-  lpd->userhash = g_hash_table_new (hash_function, hash_compare);
-  lpd->grouphash = g_hash_table_new (hash_function, hash_compare);
+  lpd->userhash = g_hash_table_new (uint_hash_function, uint_hash_compare);
+  lpd->grouphash = g_hash_table_new (uint_hash_function, uint_hash_compare);
 
   if (request->hostname != NULL)
     g_free (request->hostname);
