@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  misc.c - general purpose routines                                        */
-/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -17,10 +17,9 @@
 /*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA      */
 /*****************************************************************************/
 
-static const char cvsid[] = "$Id: misc.c,v 1.66 2004/03/26 21:03:49 masneyb Exp $";
-
 #include "gftp.h"
 #include "options.h"
+static const char cvsid[] = "$Id: misc.c 960 2008-03-28 11:44:36Z masneyb $";
 
 #ifdef HAVE_INTL_PRINTF
 
@@ -28,21 +27,11 @@ char *
 insert_commas (off_t number, char *dest_str, size_t dest_len)
 {
   if (dest_str != NULL)
-    {
-#if defined (_LARGEFILE_SOURCE)
-      g_snprintf (dest_str, dest_len, "%'lld", (long long) number);
-#else
-      g_snprintf (dest_str, dest_len, "%'ld", number);
-#endif
-    }
+    g_snprintf (dest_str, dest_len, GFTP_OFF_T_INTL_PRINTF_MOD,
+                (GFTP_OFF_T_PRINTF_CONVERSION) number);
   else
-    {
-#if defined (_LARGEFILE_SOURCE)
-      dest_str = g_strdup_printf ("%'lld", (long long) number);
-#else
-      dest_str = g_strdup_printf ("%'ld", number);
-#endif
-    }
+    dest_str = g_strdup_printf (GFTP_OFF_T_INTL_PRINTF_MOD,
+                                (GFTP_OFF_T_PRINTF_CONVERSION) number);
 
   return (dest_str);
 }
@@ -53,13 +42,10 @@ char *
 insert_commas (off_t number, char *dest_str, size_t dest_len)
 {
   char *frompos, *topos, src[50], *dest;
-  int len, num, rem, i;
+  size_t num, rem, srclen;
+  size_t len, i;
 
-#if defined (_LARGEFILE_SOURCE)
-  g_snprintf (src, sizeof (src), "%lld", (long long) number);
-#else
-  g_snprintf (src, sizeof (src), "%ld", number);
-#endif
+  g_snprintf (src, sizeof (src), GFTP_OFF_T_PRINTF_MOD, number);
 
   if (dest_str != NULL)
     *dest_str = '\0';
@@ -93,8 +79,10 @@ insert_commas (off_t number, char *dest_str, size_t dest_len)
   else
     dest = dest_str;
 
-  num = strlen (src) / 3 - 1;
-  rem = strlen (src) % 3;
+  srclen = strlen (src);
+  num = srclen / 3 - 1;
+  rem = srclen % 3;
+
   frompos = src;
   topos = dest;
   for (i = 0; i < rem; i++)
@@ -149,7 +137,7 @@ alltrim (char *str)
 
 
 char *
-expand_path (const char *src)
+gftp_expand_path (gftp_request * request, const char *src)
 {
   char *str, *pos, *endpos, *prevpos, *newstr, *tempstr, *ntoken,
        tempchar;
@@ -242,7 +230,7 @@ expand_path (const char *src)
       if ((pos = strchr (newstr, '/')) == NULL)
 	str = g_strdup (pw->pw_dir);
       else
-	str = gftp_build_path (pw->pw_dir, pos, NULL);
+	str = gftp_build_path (request, pw->pw_dir, pos, NULL);
 
       g_free (newstr);
       newstr = str;
@@ -260,34 +248,41 @@ make_nonnull (char **str)
 }
 
 
-/* FIXME - is there a replacement for this */
+/* FIXME - Possible use the libpcre library. If it isn't used, then clean
+   this function up some more. */
 int
-gftp_match_filespec (const char *filename, const char *filespec)
+gftp_match_filespec (gftp_request * request, const char *filename,
+                     const char *filespec)
 {
   const char *filepos, *wcpos, *pos;
   char search_str[20], *newpos;
+  intptr_t show_hidden_files;
   size_t len, curlen;
   
   if (filename == NULL || *filename == '\0' || 
       filespec == NULL || *filespec == '\0') 
-    return(1);
+    return (1);
+
+  gftp_lookup_request_option (request, "show_hidden_files", &show_hidden_files);
+  if (!show_hidden_files && *filename == '.' && strcmp (filename, "..") != 0)
+    return (0);
 
   filepos = filename;
   wcpos = filespec;
-  while(1) 
+  while (1)
     {
       if (*wcpos == '\0') 
         return (1);
       else if (*filepos == '\0') 
-        return(0);
-      else if(*wcpos == '?') 
+        return (0);
+      else if (*wcpos == '?') 
         {
           wcpos++;
           filepos++;
         }
-      else if(*wcpos == '*' && *(wcpos+1) == '\0') 
-        return(1);
-      else if(*wcpos == '*') 
+      else if (*wcpos == '*' && *(wcpos+1) == '\0') 
+        return (1);
+      else if (*wcpos == '*') 
         {
           len = sizeof (search_str);
           for (pos = wcpos + 1, newpos = search_str, curlen = 0;
@@ -296,14 +291,13 @@ gftp_match_filespec (const char *filename, const char *filespec)
           *newpos = '\0';
 
           if ((filepos = strstr (filepos, search_str)) == NULL)
-            return(0);
+            return (0);
           wcpos += curlen + 1;
           filepos += curlen;
         }
       else if(*wcpos++ != *filepos++) 
-        return(0);
+        return (0);
     }
-  return (1);
 }
 
 
@@ -332,6 +326,15 @@ gftp_info (void)
 
   printf ("sizeof (off_t) = %d\n", sizeof (off_t));
 
+#ifdef HAVE_INTL_PRINTF
+  printf ("#define HAVE_INTL_PRINTF\n");
+#endif
+
+  printf ("GFTP_OFF_T_HEX_PRINTF_MOD = %s\n", GFTP_OFF_T_HEX_PRINTF_MOD);
+  printf ("GFTP_OFF_T_INTL_PRINTF_MOD = %s\n", GFTP_OFF_T_INTL_PRINTF_MOD);
+  printf ("GFTP_OFF_T_PRINTF_MOD = %s\n", GFTP_OFF_T_PRINTF_MOD);
+  printf ("GFTP_OFF_T_11PRINTF_MOD = %s\n", GFTP_OFF_T_11PRINTF_MOD);
+
 #ifdef HAVE_GETADDRINFO
   printf ("#define HAVE_GETADDRINFO\n");
 #endif
@@ -360,17 +363,13 @@ gftp_info (void)
   printf ("#define HAVE_GETTEXT\n");
 #endif
 
-#ifdef HAVE_INTL_PRINTF
-  printf ("#define HAVE_INTL_PRINTF\n");
-#endif
-
   printf ("glib version: %d.%d.%d\n", GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION,
           GLIB_MICRO_VERSION);
 
   printf ("PTY implementation: %s\n", gftp_get_pty_impl ());
 
 #ifdef USE_SSL
-  printf ("OpenSSL version: 0x%lx\n", OPENSSL_VERSION_NUMBER);
+  printf ("OpenSSL version: %s\n", OPENSSL_VERSION_TEXT);
 #endif
 
   printf ("Enabled protocols: ");
@@ -438,6 +437,20 @@ string_hash_function (gconstpointer key)
 }
 
 
+gint
+uint_hash_compare (gconstpointer path1, gconstpointer path2)
+{
+  return (GPOINTER_TO_UINT (path1) == GPOINTER_TO_UINT (path2));
+}
+
+
+guint
+uint_hash_function (gconstpointer key)
+{
+  return (GPOINTER_TO_UINT (key));
+}
+
+
 void
 free_file_list (GList * filelist)
 {
@@ -448,34 +461,15 @@ free_file_list (GList * filelist)
   while (templist != NULL)
     {
       tempfle = templist->data;
-      free_fdata (tempfle);
+      templist->data = NULL;
+
+      gftp_file_destroy (tempfle, 1);
       templist = templist->next;
     }
   g_list_free (filelist);
 }
 
 
-void
-free_fdata (gftp_file * fle)
-{
-  if (fle->file)
-    g_free (fle->file);
-  if (fle->utf8_file)
-    g_free (fle->utf8_file);
-  if (fle->user)
-    g_free (fle->user);
-  if (fle->group)
-    g_free (fle->group);
-  if (fle->attribs)
-    g_free (fle->attribs);
-  if (fle->destfile)
-    g_free (fle->destfile);
-  if (fle->fd > 0)
-    close (fle->fd);
-  g_free (fle);
-}
-
-
 gftp_file *
 copy_fdata (gftp_file * fle)
 {
@@ -487,21 +481,16 @@ copy_fdata (gftp_file * fle)
   if (fle->file)
     newfle->file = g_strdup (fle->file);
 
-  if (fle->utf8_file)
-    newfle->utf8_file = g_strdup (fle->utf8_file);
-
   if (fle->user)
     newfle->user = g_strdup (fle->user);
 
   if (fle->group)
     newfle->group = g_strdup (fle->group);
 
-  if (fle->attribs)
-    newfle->attribs = g_strdup (fle->attribs);
-
   if (fle->destfile)
     newfle->destfile = g_strdup (fle->destfile);
 
+  newfle->user_data = NULL;
   return (newfle);
 }
 
@@ -582,6 +571,8 @@ free_tdata (gftp_transfer * tdata)
   if (tdata->toreq != NULL)
     gftp_request_destroy (tdata->toreq, 1);
   free_file_list (tdata->files);
+  if (tdata->thread_id != NULL)
+    g_free (tdata->thread_id);
   g_free (tdata);
 }
 
@@ -603,11 +594,26 @@ gftp_copy_request (gftp_request * req)
     newreq->account = g_strdup (req->account);
   if (req->directory)
     newreq->directory = g_strdup (req->directory);
+  if (req->url_prefix)
+    newreq->url_prefix = g_strdup (req->url_prefix);
   newreq->port = req->port;
   newreq->use_proxy = req->use_proxy;
   newreq->logging_function = req->logging_function;
-  newreq->free_hostp = 0;
-  newreq->hostp = NULL;
+  newreq->ai_family = req->ai_family;
+
+#if defined (HAVE_GETADDRINFO) && defined (HAVE_GAI_STRERROR)
+  if (req->remote_addr == NULL)
+    {
+      newreq->remote_addr = NULL;
+      newreq->remote_addr_len = 0;
+    }
+  else
+    {
+      newreq->remote_addr = g_malloc0 (req->remote_addr_len);
+      memcpy (newreq->remote_addr, req->remote_addr, req->remote_addr_len);
+      newreq->remote_addr_len = req->remote_addr_len;
+    }
+#endif
 
   gftp_copy_local_options (&newreq->local_options_vars, 
                            &newreq->local_options_hash,
@@ -634,7 +640,7 @@ gftp_file_sort_function_as (gconstpointer a, gconstpointer b)
 
   f1 = a;
   f2 = b;
-  return (strcmp (f1->file, f2->file));
+  return (strcasecmp (f1->file, f2->file));
 }
 
 
@@ -646,7 +652,7 @@ gftp_file_sort_function_ds (gconstpointer a, gconstpointer b)
 
   f1 = a;
   f2 = b;
-  ret = strcmp (f1->file, f2->file);
+  ret = strcasecmp (f1->file, f2->file);
   return (ret * -1);
 }
 
@@ -658,7 +664,7 @@ gftp_user_sort_function_as (gconstpointer a, gconstpointer b)
 
   f1 = a;
   f2 = b;
-  return (strcmp (f1->user, f2->user));
+  return (strcasecmp (f1->user, f2->user));
 }
 
 
@@ -670,7 +676,7 @@ gftp_user_sort_function_ds (gconstpointer a, gconstpointer b)
 
   f1 = a;
   f2 = b;
-  ret = strcmp (f1->user, f2->user);
+  ret = strcasecmp (f1->user, f2->user);
   return (ret * -1);
 }
 
@@ -682,7 +688,7 @@ gftp_group_sort_function_as (gconstpointer a, gconstpointer b)
 
   f1 = a;
   f2 = b;
-  return (strcmp (f1->group, f2->group));
+  return (strcasecmp (f1->group, f2->group));
 }
 
 
@@ -694,7 +700,7 @@ gftp_group_sort_function_ds (gconstpointer a, gconstpointer b)
 
   f1 = a;
   f2 = b;
-  ret = strcmp (f1->group, f2->group);
+  ret = strcasecmp (f1->group, f2->group);
   return (ret * -1);
 }
 
@@ -706,7 +712,12 @@ gftp_attribs_sort_function_as (gconstpointer a, gconstpointer b)
 
   f1 = a;
   f2 = b;
-  return (strcmp (f1->attribs, f2->attribs));
+  if (f1->st_mode < f2->st_mode)
+    return (-1);
+  else if (f1->st_mode == f2->st_mode)
+    return (0);
+  else
+    return (1);
 }
 
 
@@ -714,12 +725,15 @@ static gint
 gftp_attribs_sort_function_ds (gconstpointer a, gconstpointer b)
 {
   const gftp_file * f1, * f2;
-  gint ret;
 
   f1 = a;
   f2 = b;
-  ret = strcmp (f1->attribs, f2->attribs);
-  return (ret * -1);
+  if (f1->st_mode < f2->st_mode)
+    return (1);
+  else if (f1->st_mode == f2->st_mode)
+    return (0);
+  else
+    return (-1);
 }
 
 
@@ -830,7 +844,7 @@ gftp_sort_filelist (GList * filelist, int column, int asds)
 
       if (dotdot == NULL && strcmp (tempfle->file, "..") == 0)
         dotdot = insitem;
-      else if (sort_dirs_first && tempfle->isdir)
+      else if (sort_dirs_first && S_ISDIR (tempfle->st_mode))
         {
           insitem->next = dirs;
           dirs = insitem;
@@ -866,68 +880,25 @@ gftp_sort_filelist (GList * filelist, int column, int asds)
 }
 
 
-mode_t
-gftp_parse_attribs (char *attribs)
-{
-  mode_t mode;
-  int cur;
-
-  cur = 0;
-  if (attribs[1] == 'r')
-    cur += 4;
-  if (attribs[2] == 'w')
-    cur += 2;
-  if (attribs[3] == 'x' ||
-      attribs[3] == 's')
-    cur += 1;
-  mode = cur;
-
-  cur = 0;
-  if (attribs[4] == 'r')
-    cur += 4;
-  if (attribs[5] == 'w')
-    cur += 2;
-  if (attribs[6] == 'x' ||
-      attribs[6] == 's')
-    cur += 1;
-  mode = (mode * 10) + cur;
-
-  cur = 0;
-  if (attribs[7] == 'r')
-    cur += 4;
-  if (attribs[8] == 'w')
-    cur += 2;
-  if (attribs[9] == 'x' ||
-      attribs[9] == 's')
-    cur += 1;
-  mode = (mode * 10) + cur;
-
-  return (mode);
-}
-
-
 char *
-gftp_gen_ls_string (gftp_file * fle, char *file_prefixstr, char *file_suffixstr)
+gftp_gen_ls_string (gftp_request * request, gftp_file * fle,
+                    char *file_prefixstr, char *file_suffixstr)
 {
-  char *tempstr1, *tempstr2, *ret, tstr[50];
+  char *tempstr1, *tempstr2, *ret, tstr[50], *attribs, *utf8;
+  size_t destlen;
   struct tm *lt;
   time_t t;
 
   lt = localtime (&fle->datetime);
 
-  tempstr1 = g_strdup_printf ("%10s %8s %8s", fle->attribs, fle->user,
-                              fle->group);
+  attribs = gftp_convert_attributes_from_mode_t (fle->st_mode);
+  tempstr1 = g_strdup_printf ("%10s %8s %8s", attribs, fle->user, fle->group);
+  g_free (attribs);
 
-  if (fle->attribs && (*fle->attribs == 'b' || *fle->attribs == 'c'))
+  if (GFTP_IS_SPECIAL_DEVICE (fle->st_mode))
     tempstr2 = g_strdup_printf ("%d, %d", major (fle->size), minor (fle->size));
   else
-    {
-#if defined (_LARGEFILE_SOURCE)
-      tempstr2 = g_strdup_printf ("%11lld", (long long) fle->size);
-#else
-      tempstr2 = g_strdup_printf ("%11ld", fle->size);
-#endif
-    }
+    tempstr2 = g_strdup_printf (GFTP_OFF_T_11PRINTF_MOD, fle->size);
 
   time (&t);
 
@@ -941,10 +912,16 @@ gftp_gen_ls_string (gftp_file * fle, char *file_prefixstr, char *file_suffixstr)
   if (file_suffixstr == NULL)
     file_suffixstr = "";
 
-  ret = g_strdup_printf ("%s %s %s %s%s%s", tempstr1, tempstr2, tstr, 
-                         file_prefixstr, 
-                         fle->utf8_file != NULL ? fle->utf8_file : fle->file,
-                         file_suffixstr);
+  utf8 = gftp_string_from_utf8 (request, 1, fle->file, &destlen);
+  if (utf8 != NULL)
+    {
+      ret = g_strdup_printf ("%s %s %s %s%s%s", tempstr1, tempstr2, tstr,
+                             file_prefixstr, utf8, file_suffixstr);
+      g_free (utf8);
+    }
+  else
+    ret = g_strdup_printf ("%s %s %s %s%s%s", tempstr1, tempstr2, tstr,
+                           file_prefixstr, fle->file, file_suffixstr);
 
   g_free (tempstr1);
   g_free (tempstr2);
@@ -953,63 +930,6 @@ gftp_gen_ls_string (gftp_file * fle, char *file_prefixstr, char *file_suffixstr)
 }
 
 
-#if !defined (HAVE_GETADDRINFO) || !defined (HAVE_GAI_STRERROR)
-
-struct hostent *
-r_gethostbyname (const char *name, struct hostent *result_buf, int *h_errnop)
-{
-  static GStaticMutex hostfunclock = G_STATIC_MUTEX_INIT; 
-  struct hostent *hent;
-
-  if (g_thread_supported ())
-    g_static_mutex_lock (&hostfunclock);
-
-  if ((hent = gethostbyname (name)) == NULL)
-    {
-      if (h_errnop)
-        *h_errnop = h_errno;
-    }
-  else
-    {
-      *result_buf = *hent;
-      hent = result_buf;
-    }
-
-  if (g_thread_supported ())
-    g_static_mutex_unlock (&hostfunclock);
-
-  return (hent);
-}
-
-#endif /* !HAVE_GETADDRINFO */
-
-struct servent *
-r_getservbyname (const char *name, const char *proto,
-                 struct servent *result_buf, int *h_errnop)
-{
-  static GStaticMutex servfunclock = G_STATIC_MUTEX_INIT;
-  struct servent *sent;
-
-  if (g_thread_supported ())
-    g_static_mutex_lock (&servfunclock);
-
-  if ((sent = getservbyname (name, proto)) == NULL)
-    {
-      if (h_errnop)
-        *h_errnop = h_errno;
-    }
-  else
-    {
-      *result_buf = *sent;
-      sent = result_buf;
-    }
-
-  if (g_thread_supported ())
-    g_static_mutex_unlock (&servfunclock);
-  return (sent);
-}
-
-
 char *
 base64_encode (char *str)
 {
@@ -1018,7 +938,8 @@ base64_encode (char *str)
 
   char *newstr, *newpos, *fillpos, *pos;
   unsigned char table[64], encode[3];
-  int i, num;
+  size_t slen, num;
+  int i;
 
   for (i = 0; i < 26; i++)
     {
@@ -1032,10 +953,12 @@ base64_encode (char *str)
   table[62] = '+';
   table[63] = '/';
 
-  num = strlen (str) / 3;
-  if (strlen (str) % 3 > 0)
+  slen = strlen (str);
+  num = slen / 3;
+  if (slen % 3 > 0)
     num++;
-  newstr = g_malloc (num * 4 + 1);
+
+  newstr = g_malloc0 ((gulong) num * 4 + 1);
   newstr[num * 4] = '\0';
   newpos = newstr;
 
@@ -1059,8 +982,14 @@ base64_encode (char *str)
 
 
 void
-gftp_free_bookmark (gftp_bookmarks_var * entry)
+gftp_free_bookmark (gftp_bookmarks_var * entry, int free_node)
 {
+  gftp_bookmarks_var * tempentry;
+
+  if (entry->path)
+    g_free (entry->path);
+  if (entry->oldpath)
+    g_free (entry->oldpath);
   if (entry->hostname)
     g_free (entry->hostname);
   if (entry->remote_dir)
@@ -1086,6 +1015,15 @@ gftp_free_bookmark (gftp_bookmarks_var * entry)
       entry->local_options_hash = NULL;
       entry->num_local_options_vars = 0;
     }
+
+  if (free_node)
+    g_free (entry);
+  else
+    {
+      tempentry = entry->children;
+      memset (entry, 0, sizeof (*entry));
+      entry->children = tempentry;
+    }
 }
 
 
@@ -1179,7 +1117,7 @@ get_next_selection (GList * selection, GList ** list, int *curnum)
 
 
 char *
-gftp_build_path (const char *first_element, ...) 
+gftp_build_path (gftp_request * request, const char *first_element, ...) 
 {
   const char *element;
   size_t len, retlen;
@@ -1211,9 +1149,11 @@ gftp_build_path (const char *first_element, ...)
         }
       
       retlen += len;
-      ret = g_realloc (ret, retlen + 1);
+      ret = g_realloc (ret, (gulong) retlen + 1);
 
-      if (add_separator)
+      /* Don't append a / for VMS servers... */
+      if (add_separator &&
+          (request == NULL || request->server_type != GFTP_DIRTYPE_VMS))
         strncat (ret, "/", retlen);
 
       strncat (ret, element, retlen);
@@ -1223,17 +1163,6 @@ gftp_build_path (const char *first_element, ...)
 }
 
 
-off_t
-gftp_parse_file_size (char *str)
-{
-#if defined (_LARGEFILE_SOURCE)
-  return (strtoll (str, NULL, 10));
-#else
-  return (strtol (str, NULL, 10));
-#endif
-}
-
-
 void
 gftp_locale_init (void)
 {
@@ -1266,12 +1195,12 @@ gftp_scramble_password (const char *password)
   if (strcmp (password, "@EMAIL@") == 0)
     return (g_strdup (password));
 
-  newstr = g_malloc (strlen(password) * 2 + 2);
+  newstr = g_malloc0 ((gulong) strlen (password) * 2 + 2);
   newpos = newstr;
   
   *newpos++ = '$';
 
-  while (*password != 0)
+  while (*password != '\0')
     {
       *newpos++ = ((*password >> 2) & 0x3c) | 0x41;
       *newpos++ = ((*password << 2) & 0x3c) | 0x41;
@@ -1294,11 +1223,11 @@ gftp_descramble_password (const char *password)
     return (g_strdup (password));
 
   passwordpos = password + 1;
-  newstr = g_malloc (strlen (passwordpos) / 2 + 1);
+  newstr = g_malloc0 ((gulong) strlen (passwordpos) / 2 + 1);
   newpos = newstr;
  
   error = 0;
-  while (*passwordpos != '\0' && (*passwordpos + 1) != '\0')
+  while (*passwordpos != '\0' && *(passwordpos + 1) != '\0')
     {
       if ((*passwordpos & 0xc3) != 0x41 ||
           (*(passwordpos + 1) & 0xc3) != 0x41)
@@ -1331,6 +1260,10 @@ gftp_get_transfer_action (gftp_request * request, gftp_file * fle)
 
   gftp_lookup_request_option (request, "overwrite_default", &overwrite_default);
 
+  /* FIXME - add code to compare the file times and make a decision based
+     on that. Also if overwrite_default is enabled and the file sizes/dates are
+     the same, then skip the file */
+
   if (overwrite_default)
     fle->transfer_action = GFTP_TRANS_ACTION_OVERWRITE;
   else if (fle->startsize == fle->size)
@@ -1343,3 +1276,23 @@ gftp_get_transfer_action (gftp_request * request, gftp_file * fle)
   return (fle->transfer_action);
 }
 
+
+char *
+gftp_get_share_dir (void)
+{
+  static char *gftp_share_dir = NULL;
+  char *envval;
+
+  if (gftp_share_dir == NULL)
+    {
+      envval = getenv ("GFTP_SHARE_DIR");
+
+      if (envval != NULL && *envval != '\0')
+        gftp_share_dir = g_strdup (envval);
+      else
+        gftp_share_dir = SHARE_DIR;
+    }
+
+  return (gftp_share_dir);
+}
+
