@@ -17,15 +17,21 @@
 /*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA      */
 /*****************************************************************************/
 
-/* $Id: options.h,v 1.36 2005/01/16 15:48:24 masneyb Exp $ */
+/* $Id: options.h 871 2006-12-23 18:16:32Z masneyb $ */
 
 #include "gftp.h"
 
-static char *gftp_sort_columns[] = { N_("none"), N_("file"), N_("size"), 
-                                     N_("user"), N_("group"), 
-                                     N_("datetime"), N_("attribs"), NULL };
+typedef /*@null@*/ char *gftp_sort_columns_tag;
+static gftp_sort_columns_tag gftp_sort_columns[] = { N_("none"), N_("file"),
+                                                     N_("size"), N_("user"),
+                                                     N_("group"),
+                                                     N_("datetime"),
+                                                     N_("attribs"), NULL };
 
-static char *gftp_sort_direction[] = { N_("descending"), N_("ascending"), NULL };
+typedef /*@null@*/ char *gftp_sort_direction_tag;
+static gftp_sort_direction_tag gftp_sort_direction[] = { N_("descending"),
+                                                         N_("ascending"),
+                                                         NULL };
 
 static float gftp_maxkbs = 0.0;
 
@@ -39,9 +45,6 @@ gftp_config_vars gftp_global_config_vars[] =
    GFTP_PORT_GTK, NULL},
   {"edit_program", N_("Edit program:"), gftp_option_type_text, "", NULL, 0,
    N_("The default program used to edit files."), GFTP_PORT_GTK, NULL},
-  {"startup_directory", N_("Startup Directory:"), 
-   gftp_option_type_text, "", NULL, 0,
-   N_("The default directory gFTP will go to on startup"), GFTP_PORT_ALL, NULL},
   {"max_log_window_size", N_("Max Log Window Size:"), 
    gftp_option_type_int, 0, NULL, 0, 
    N_("The maximum size of the log window in bytes for the GTK+ port"), 
@@ -103,7 +106,16 @@ gftp_config_vars gftp_global_config_vars[] =
    GFTP_PORT_GTK, NULL},
   {"cmd_in_gui", N_("Allow manual commands in GUI"), 
    gftp_option_type_checkbox, GINT_TO_POINTER(0), NULL, 0,
-   N_("Allow entering manual commands in the GUI (functions like the text port)"), GFTP_PORT_GTK, NULL},
+   N_("Allow entering manual commands in the GUI (functions like the text port)"),
+   GFTP_PORT_GTK, NULL},
+  {"remember_last_directory", N_("Remember last directory"), 
+   gftp_option_type_checkbox, GINT_TO_POINTER(0), NULL, 0,
+   N_("Save the last local and remote directory when the application is closed"),
+   GFTP_PORT_GTK, NULL},
+  {"connect_to_remote_on_startup", N_("Connect to remote server on startup"), 
+   gftp_option_type_checkbox, GINT_TO_POINTER(0), NULL, 0,
+   N_("Automatically connect to the remote server when the application is started."),
+   GFTP_PORT_GTK, NULL},
 
   {"", N_("Network"), gftp_option_type_notebook, NULL, NULL, 
    GFTP_CVARS_FLAGS_SHOW_BOOKMARK, NULL, GFTP_PORT_GTK, NULL},
@@ -168,6 +180,11 @@ gftp_config_vars gftp_global_config_vars[] =
   {"port_value", "", gftp_option_type_text, "", NULL, 0, NULL, 0, NULL},
   {"user_value", "", gftp_option_type_text, "", NULL, 0, NULL, 0, NULL},
 
+  {"local_startup_directory", "", gftp_option_type_text, "", NULL, 0, NULL,
+   0, NULL},
+  {"remote_startup_directory", "", gftp_option_type_text, "", NULL, 0, NULL,
+   0, NULL},
+
   {"local_sortcol", "", 
    gftp_option_type_intcombo, GINT_TO_POINTER(1), gftp_sort_columns, 0,
    N_("The default column to sort by"), GFTP_PORT_TEXT, NULL},
@@ -271,7 +288,7 @@ GList * gftp_file_transfers = NULL,
       
 gftp_bookmarks_var * gftp_bookmarks = NULL;
 
-FILE * gftp_logfd = NULL;
+/*@null@*/ FILE * gftp_logfd = NULL;
 
 int gftp_configuration_changed = 0;
 
