@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  rfc959.c - General purpose routines for the FTP protocol (RFC 959)       */
-/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -20,7 +20,7 @@
 #include "gftp.h"
 #include "ftpcommon.h"
 
-static const char cvsid[] = "$Id: rfc959.c,v 1.93 2005/01/25 02:34:18 masneyb Exp $";
+static const char cvsid[] = "$Id: rfc959.c 957 2008-03-04 12:02:47Z masneyb $";
 
 static gftp_textcomboedt_data gftp_proxy_type[] = {
   {N_("none"), "", 0},
@@ -122,8 +122,13 @@ rfc959_read_response (gftp_request * request, int disconnect_on_42x)
 	  strncpy (code, tempstr, 3);
 	  code[3] = ' ';
 	}
-      request->logging_function (gftp_logging_recv, request,
-				 "%s\n", tempstr);
+
+      if (*tempstr == '4' || *tempstr == '5')
+        request->logging_function (gftp_logging_error, request,
+  				   "%s\n", tempstr);
+      else
+        request->logging_function (gftp_logging_recv, request,
+  				   "%s\n", tempstr);
     }
   while (strncmp (code, tempstr, 4) != 0);
 
@@ -146,7 +151,8 @@ rfc959_read_response (gftp_request * request, int disconnect_on_42x)
 
 int
 rfc959_send_command (gftp_request * request, const char *command, 
-                     int read_response)
+                     ssize_t command_len, int read_response,
+                     int dont_try_to_reconnect)
 {
   int ret;
 
@@ -170,23 +176,75 @@ rfc959_send_command (gftp_request * request, const char *command,
                                  command);
     }
 
-  if ((ret = request->write_function (request, command, strlen (command), 
+  if (command_len == -1)
+    command_len = strlen (command);
+
+  if ((ret = request->write_function (request, command, command_len, 
                                       request->datafd)) < 0)
     return (ret);
 
   if (read_response)
-    return (rfc959_read_response (request, 1));
+    {
+      ret = rfc959_read_response (request, 1);
+      if (ret == GFTP_ETIMEDOUT && !dont_try_to_reconnect)
+        {
+          ret = gftp_connect (request);
+          if (ret < 0)
+            return (ret);
+
+          return (rfc959_send_command (request, command, command_len, 1, 1));
+        }
+      else
+        return (ret);
+    }
   else
     return (0);
 }
 
+static int
+rfc959_generate_and_send_command (gftp_request * request, const char *command,
+                                  const char *argument, int read_response,
+                                  int dont_try_to_reconnect)
+{
+  char *tempstr, *utf8;
+  size_t len;
+  int resp;
+
+  if (argument != NULL)
+    {
+      utf8 = gftp_filename_from_utf8 (request, argument, &len);
+      if (utf8 != NULL)
+        {
+          tempstr = g_strconcat (command, " ", utf8, "\r\n", NULL);
+          g_free (utf8);
+        }
+      else
+        {
+          tempstr = g_strconcat (command, " ", argument, "\r\n", NULL);
+          len = strlen (argument);
+        }
+
+      len += strlen (command) + 3;
+    }
+  else
+    {
+      tempstr = g_strconcat (command, "\r\n", NULL);
+      len = strlen (command) + 2;
+    }
+
+  resp = rfc959_send_command (request, tempstr, len, read_response,
+                              dont_try_to_reconnect);
+  g_free (tempstr);
+  return (resp);
+}
+
 
 static char *
 parse_ftp_proxy_string (gftp_request * request)
 {
   char *startpos, *endpos, *newstr, *newval, tempport[6], *proxy_config, *utf8,
        savechar;
-  size_t len;
+  size_t len, destlen;
   intptr_t tmp;
 
   g_return_val_if_fail (request != NULL, NULL);
@@ -255,7 +313,7 @@ parse_ftp_proxy_string (gftp_request * request)
           *endpos = '\0';
 
           len += strlen (startpos) + 2;
-          newstr = g_realloc (newstr, sizeof (char) * (len + 1));
+          newstr = g_realloc (newstr, (gulong) sizeof (char) * (len + 1));
           strcat (newstr, startpos);
           strcat (newstr, "\r\n");
 
@@ -275,18 +333,18 @@ parse_ftp_proxy_string (gftp_request * request)
       len += strlen (startpos);
       if (!newval)
         {
-          newstr = g_realloc (newstr, sizeof (char) * (len + 1));
+          newstr = g_realloc (newstr, (gulong) sizeof (char) * (len + 1));
           strcat (newstr, startpos);
         }
       else
         {
-          utf8 = gftp_string_from_utf8 (request, newval);
+          utf8 = gftp_string_from_utf8 (request, -1, newval, &destlen);
           if (utf8 != NULL)
             len += strlen (utf8);
           else
             len += strlen (newval);
 
-          newstr = g_realloc (newstr, sizeof (char) * (len + 1));
+          newstr = g_realloc (newstr, (gulong) sizeof (char) * (len + 1));
           strcat (newstr, startpos);
 
           if (utf8 != NULL)
@@ -310,10 +368,11 @@ parse_ftp_proxy_string (gftp_request * request)
 static int
 rfc959_getcwd (gftp_request * request)
 {
-  char *pos, *dir;
+  char *pos, *dir, *utf8;
+  size_t destlen;
   int ret;
 
-  ret = rfc959_send_command (request, "PWD\r\n", 1);
+  ret = rfc959_send_command (request, "PWD\r\n", -1, 1, 0);
   if (ret < 0)
     return (ret);
   else if (ret != '2')
@@ -350,7 +409,12 @@ rfc959_getcwd (gftp_request * request)
   if (request->directory)
     g_free (request->directory);
 
-  request->directory = g_strdup (dir);
+  utf8 = gftp_filename_to_utf8 (request, dir, &destlen);
+  if (utf8 != NULL)
+    request->directory = utf8;
+  else
+    request->directory = g_strdup (dir);
+
   return (0);
 }
 
@@ -358,31 +422,23 @@ rfc959_getcwd (gftp_request * request)
 static int
 rfc959_chdir (gftp_request * request, const char *directory)
 {
-  char *tempstr;
   int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (directory != NULL, GFTP_EFATAL);
 
   if (strcmp (directory, "..") == 0)
-    ret = rfc959_send_command (request, "CDUP\r\n", 1);
+    ret = rfc959_send_command (request, "CDUP\r\n", -1, 1, 0);
   else
-    {
-      tempstr = g_strconcat ("CWD ", directory, "\r\n", NULL);
-      ret = rfc959_send_command (request, tempstr, 1);
-      g_free (tempstr);
-    }
+    ret = rfc959_generate_and_send_command (request, "CWD", directory, 1, 0);
 
   if (ret < 0)
     return (ret);
   else if (ret != '2')
     return (GFTP_ERETRYABLE);
 
-  if (directory != request->directory)
-    {
-      if ((ret = rfc959_getcwd (request)) < 0)
-        return (ret);
-    }
+  if ((ret = rfc959_getcwd (request)) < 0)
+    return (ret);
 
   return (0);
 }
@@ -391,15 +447,13 @@ rfc959_chdir (gftp_request * request, const char *directory)
 static int
 rfc959_syst (gftp_request * request)
 {
-  rfc959_parms * parms;
+  int ret, disable_ls_options;
   char *stpos, *endpos;
-  int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
 
-  parms = request->protocol_data;
-  ret = rfc959_send_command (request, "SYST\r\n", 1);
+  ret = rfc959_send_command (request, "SYST\r\n", -1, 1, 0);
 
   if (ret < 0)
     return (ret);
@@ -415,11 +469,15 @@ rfc959_syst (gftp_request * request)
     return (GFTP_ERETRYABLE);
 
   *endpos = '\0';
+  disable_ls_options = 0;
 
   if (strcmp (stpos, "UNIX") == 0)
     request->server_type = GFTP_DIRTYPE_UNIX;
   else if (strcmp (stpos, "VMS") == 0)
-    request->server_type = GFTP_DIRTYPE_VMS;
+    {
+      disable_ls_options = 1;
+      request->server_type = GFTP_DIRTYPE_VMS;
+    }
   else if (strcmp (stpos, "MVS") == 0 ||
            strcmp (stpos, "OS/MVS") == 0)
     request->server_type = GFTP_DIRTYPE_MVS;
@@ -430,6 +488,16 @@ rfc959_syst (gftp_request * request)
   else
     request->server_type = GFTP_DIRTYPE_OTHER;
 
+  if (strcmp (stpos, "OS/400") == 0)
+    disable_ls_options = 1;
+
+  if (disable_ls_options)
+    {
+      gftp_set_request_option (request, "show_hidden_files",
+                               GINT_TO_POINTER(0));
+      gftp_set_request_option (request, "resolve_symlinks", GINT_TO_POINTER(0));
+    }
+
   return (0);
 }
 
@@ -437,7 +505,7 @@ rfc959_syst (gftp_request * request)
 int
 rfc959_connect (gftp_request * request)
 {
-  char tempchar, *startpos, *endpos, *tempstr, *email, *proxy_hostname, *utf8;
+  char tempchar, *startpos, *endpos, *tempstr, *email, *proxy_hostname;
   intptr_t ascii_transfers, proxy_port;
   rfc959_parms * parms;
   int ret, resp;
@@ -463,7 +531,8 @@ rfc959_connect (gftp_request * request)
       gftp_set_password (request, email);
     }
    
-  if ((ret = gftp_connect_server (request, "ftp", proxy_hostname, proxy_port)) < 0)
+  if ((ret = gftp_connect_server (request, "ftp", proxy_hostname,
+                                  proxy_port)) < 0)
     return (ret);
 
   /* Get the banner */
@@ -497,7 +566,7 @@ rfc959_connect (gftp_request * request)
 	      tempchar = *(endpos + 1);
 	      if (*endpos != '\0')
 		*(endpos + 1) = '\0';
-	      if ((resp = rfc959_send_command (request, startpos, 1)) < 0)
+	      if ((resp = rfc959_send_command (request, startpos, -1, 1, 0)) < 0)
                 return (resp);
 	      if (*endpos != '\0')
 		*(endpos + 1) = tempchar;
@@ -511,34 +580,23 @@ rfc959_connect (gftp_request * request)
     }
   else
     {
-      tempstr = g_strconcat ("USER ", request->username, "\r\n", NULL);
-      resp = rfc959_send_command (request, tempstr, 1);
-      g_free (tempstr);
+      resp = rfc959_generate_and_send_command (request, "USER",
+                                               request->username, 1, 0);
       if (resp < 0)
         return (resp);
 
       if (resp == '3')
 	{
-          utf8 = gftp_string_from_utf8 (request, request->password);
-          if (utf8 != NULL)
-            {
-              tempstr = g_strconcat ("PASS ", utf8, "\r\n", NULL);
-              g_free (utf8);
-            }
-          else
-            tempstr = g_strconcat ("PASS ", request->password, "\r\n", NULL);
-
-	  resp = rfc959_send_command (request, tempstr, 1);
-	  g_free (tempstr);
+          resp = rfc959_generate_and_send_command (request, "PASS",
+                                                   request->password, 1, 0);
           if (resp < 0)
             return (resp);
         }
 
       if (resp == '3' && request->account != NULL)
 	{
-	  tempstr = g_strconcat ("ACCT ", request->account, "\r\n", NULL);
-	  resp = rfc959_send_command (request, tempstr, 1);
-	  g_free (tempstr);
+          resp = rfc959_generate_and_send_command (request, "ACCT",
+                                                   request->account, 1, 0);
           if (resp < 0)
             return (resp);
 	}
@@ -569,7 +627,7 @@ rfc959_connect (gftp_request * request)
       parms->is_ascii_transfer = 0;
     }
 
-  if ((ret = rfc959_send_command (request, tempstr, 1)) < 0)
+  if ((ret = rfc959_send_command (request, tempstr, -1, 1, 0)) < 0)
     return (ret);
 
   ret = -1;
@@ -594,14 +652,27 @@ rfc959_connect (gftp_request * request)
 
 
 static void
-rfc959_disconnect (gftp_request * request)
+rfc959_close_data_connection (gftp_request * request)
 {
   rfc959_parms * parms;
 
   g_return_if_fail (request != NULL);
 
   parms = request->protocol_data;
+  if (parms->data_connection != -1)
+    {
+      close (parms->data_connection);
+      parms->data_connection = -1;
+    }
+}
 
+
+static void
+rfc959_disconnect (gftp_request * request)
+{
+  g_return_if_fail (request != NULL);
+
+  rfc959_close_data_connection (request);
   if (request->datafd > 0)
     {
       request->logging_function (gftp_logging_misc, request,
@@ -610,12 +681,6 @@ rfc959_disconnect (gftp_request * request)
       close (request->datafd);
       request->datafd = -1;
     }
-
-  if (parms->data_connection > 0)
-    {
-      close (parms->data_connection);
-      parms->data_connection = -1;
-    }
 }
 
 
@@ -634,7 +699,8 @@ rfc959_ipv4_data_connection_new (gftp_request * request)
 
   parms = request->protocol_data;
 
-  if ((parms->data_connection = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
+  parms->data_connection = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
+  if (parms->data_connection == -1)
     {
       request->logging_function (gftp_logging_error, request,
 				 _("Failed to create a IPv4 socket: %s\n"),
@@ -659,7 +725,7 @@ rfc959_ipv4_data_connection_new (gftp_request * request)
   gftp_lookup_request_option (request, "passive_transfer", &passive_transfer);
   if (passive_transfer)
     {
-      resp = rfc959_send_command (request, "PASV\r\n", 1);
+      resp = rfc959_send_command (request, "PASV\r\n", -1, 1, 1);
       if (resp < 0)
         return (resp);
       else if (resp != '2')
@@ -701,14 +767,8 @@ rfc959_ipv4_data_connection_new (gftp_request * request)
                                   &ignore_pasv_address);
       if (ignore_pasv_address)
 	{
-#if defined (HAVE_GETADDRINFO)
-          memcpy (&data_addr.sin_addr,
-                  &((struct sockaddr_in *) request->current_hostp->ai_addr)->sin_addr,
-                  sizeof (data_addr.sin_addr));
-#else
-          memcpy (&data_addr.sin_addr, request->hostp->h_addr_list[request->curhost],
-                  request->hostp->h_length);
-#endif
+          memcpy (&data_addr.sin_addr, request->remote_addr,
+                  request->remote_addr_len);
 
           pos = (char *) &data_addr.sin_addr;
           request->logging_function (gftp_logging_error, request,
@@ -778,7 +838,7 @@ rfc959_ipv4_data_connection_new (gftp_request * request)
 				 pos[0] & 0xff, pos[1] & 0xff, pos[2] & 0xff,
 				 pos[3] & 0xff, pos1[0] & 0xff,
 				 pos1[1] & 0xff);
-      resp = rfc959_send_command (request, command, 1);
+      resp = rfc959_send_command (request, command, -1, 1, 1);
       g_free (command);
 
       if (resp < 0)
@@ -805,7 +865,6 @@ rfc959_ipv6_data_connection_new (gftp_request * request)
   struct sockaddr_in6 data_addr;
   char *pos, buf[64], *command;
   intptr_t passive_transfer;
-  socklen_t data_addr_len;
   rfc959_parms * parms;
   unsigned int port;
   int resp;
@@ -829,9 +888,8 @@ rfc959_ipv6_data_connection_new (gftp_request * request)
       return (GFTP_ERETRYABLE);
     }
 
-  data_addr_len = sizeof (data_addr);
   /* This condition shouldn't happen. We better check anyway... */
-  if (data_addr_len != request->current_hostp->ai_addrlen) 
+  if (sizeof (data_addr) != request->remote_addr_len) 
     {
       request->logging_function (gftp_logging_error, request,
 				 _("Error: It doesn't look like we are connected via IPv6. Aborting connection.\n"));
@@ -839,13 +897,13 @@ rfc959_ipv6_data_connection_new (gftp_request * request)
       return (GFTP_EFATAL);
     }
 
-  memset (&data_addr, 0, data_addr_len);
+  memset (&data_addr, 0, sizeof (data_addr));
   data_addr.sin6_family = AF_INET6;
 
   gftp_lookup_request_option (request, "passive_transfer", &passive_transfer);
   if (passive_transfer)
     {
-      resp = rfc959_send_command (request, "EPSV\r\n", 1);
+      resp = rfc959_send_command (request, "EPSV\r\n", -1, 1, 1);
       if (resp < 0)
         return (resp);
       else if (resp != '2')
@@ -877,11 +935,11 @@ rfc959_ipv6_data_connection_new (gftp_request * request)
           return (GFTP_EFATAL);
         }
 
-      memcpy (&data_addr, request->current_hostp->ai_addr, data_addr_len);
+      memcpy (&data_addr, request->remote_addr, request->remote_addr_len);
       data_addr.sin6_port = htons (port);
 
       if (connect (parms->data_connection, (struct sockaddr *) &data_addr, 
-                   data_addr_len) == -1)
+                   request->remote_addr_len) == -1)
         {
           request->logging_function (gftp_logging_error, request,
                                     _("Cannot create a data connection: %s\n"),
@@ -892,11 +950,11 @@ rfc959_ipv6_data_connection_new (gftp_request * request)
     }
   else
     {
-      memcpy (&data_addr, request->current_hostp->ai_addr, data_addr_len);
+      memcpy (&data_addr, request->remote_addr, request->remote_addr_len);
       data_addr.sin6_port = 0;
 
       if (bind (parms->data_connection, (struct sockaddr *) &data_addr, 
-                data_addr_len) == -1)
+                request->remote_addr_len) == -1)
 	{
 	  request->logging_function (gftp_logging_error, request,
 				     _("Cannot bind a port: %s\n"),
@@ -906,7 +964,7 @@ rfc959_ipv6_data_connection_new (gftp_request * request)
 	}
 
       if (getsockname (parms->data_connection, (struct sockaddr *) &data_addr, 
-                       &data_addr_len) == -1)
+                       &request->remote_addr_len) == -1)
         {
           request->logging_function (gftp_logging_error, request,
 				     _("Cannot get socket name: %s\n"),
@@ -937,7 +995,7 @@ rfc959_ipv6_data_connection_new (gftp_request * request)
       command = g_strdup_printf ("EPRT |2|%s|%d|\n", buf,
                                  ntohs (data_addr.sin6_port));
 
-      resp = rfc959_send_command (request, command, 1);
+      resp = rfc959_send_command (request, command, -1, 1, 1);
       g_free (command);
 
       if (resp < 0)
@@ -956,19 +1014,32 @@ rfc959_ipv6_data_connection_new (gftp_request * request)
 
 
 static int
-rfc959_data_connection_new (gftp_request * request)
+rfc959_data_connection_new (gftp_request * request, int dont_try_to_reconnect)
 {
+  int ret;
+
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
 
 #ifdef HAVE_IPV6
   if (request->ai_family == AF_INET6)
-    return (rfc959_ipv6_data_connection_new (request));
+    ret = rfc959_ipv6_data_connection_new (request);
   else
-    return (rfc959_ipv4_data_connection_new (request));
+    ret = rfc959_ipv4_data_connection_new (request);
 #else
-  return (rfc959_ipv4_data_connection_new (request));
+  ret = rfc959_ipv4_data_connection_new (request);
 #endif
+
+  if (ret == GFTP_ETIMEDOUT && !dont_try_to_reconnect)
+    {
+      ret = gftp_connect (request);
+      if (ret < 0)
+        return (ret);
+
+      return (rfc959_data_connection_new (request, 1));
+    }
+  else
+    return (ret);
 }
 
 
@@ -1009,8 +1080,8 @@ rfc959_accept_active_connection (gftp_request * request)
     }
 
   close (parms->data_connection);
-
   parms->data_connection = infd;
+
   if ((ret = gftp_fd_set_sockblocking (request, parms->data_connection, 1)) < 0)
     return (ret);
 
@@ -1018,7 +1089,7 @@ rfc959_accept_active_connection (gftp_request * request)
 }
 
 
-static int
+static unsigned int
 rfc959_is_ascii_transfer (gftp_request * request, const char *filename)
 {
   gftp_config_list_vars * tmplistvar;
@@ -1053,9 +1124,10 @@ rfc959_is_ascii_transfer (gftp_request * request, const char *filename)
 static int
 rfc959_set_data_type (gftp_request * request, const char *filename)
 {
+  unsigned int new_ascii;
   rfc959_parms * parms;
-  int new_ascii, ret;
   char *tempstr;
+  int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
 
@@ -1075,7 +1147,7 @@ rfc959_set_data_type (gftp_request * request, const char *filename)
           parms->is_ascii_transfer = 0;
         }
 
-      if ((ret = rfc959_send_command (request, tempstr, 1)) < 0)
+      if ((ret = rfc959_send_command (request, tempstr, -1, 1, 0)) < 0)
         return (ret);
     }
 
@@ -1083,28 +1155,22 @@ rfc959_set_data_type (gftp_request * request, const char *filename)
 }
 
 
-static off_t
-rfc959_get_file (gftp_request * request, const char *filename, int fd,
-                 off_t startsize)
+static int
+rfc959_setup_file_transfer (gftp_request * request, const char *filename,
+                            off_t startsize, char *transfer_command)
 {
-  char *command, *tempstr;
   intptr_t passive_transfer;
   rfc959_parms * parms;
+  char *command;
   int ret;
 
-  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
-  g_return_val_if_fail (filename != NULL, GFTP_EFATAL);
-  g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
-
   parms = request->protocol_data;
-  if (fd > 0)
-    parms->data_connection = fd;
 
   if ((ret = rfc959_set_data_type (request, filename)) < 0)
     return (ret);
 
   if (parms->data_connection < 0 && 
-      (ret = rfc959_data_connection_new (request)) < 0)
+      (ret = rfc959_data_connection_new (request, 0)) < 0)
     return (ret);
 
   if ((ret = gftp_fd_set_sockblocking (request, parms->data_connection, 1)) < 0)
@@ -1114,30 +1180,30 @@ rfc959_get_file (gftp_request * request, const char *filename, int fd,
     {
       command = g_strdup_printf ("REST " GFTP_OFF_T_PRINTF_MOD "\r\n",
                                  startsize);
-      ret = rfc959_send_command (request, command, 1);
+      ret = rfc959_send_command (request, command, -1, 1, 0);
       g_free (command);
 
       if (ret < 0)
         return (ret);
       else if (ret != '3')
         {
-          close (parms->data_connection);
-          parms->data_connection = -1;
+          rfc959_close_data_connection (request);
 	  return (GFTP_ERETRYABLE);
         }
     }
 
-  tempstr = g_strconcat ("RETR ", filename, "\r\n", NULL);
-  ret = rfc959_send_command (request, tempstr, 1);
-  g_free (tempstr);
-
+  ret = rfc959_generate_and_send_command (request, transfer_command, filename,
+                                          1, 0);
   if (ret < 0)
     return (ret);
   else if (ret != '1')
     {
-      close (parms->data_connection);
-      parms->data_connection = -1;
-      return (GFTP_ERETRYABLE);
+      rfc959_close_data_connection (request);
+
+      if (ret == '5')
+        return (GFTP_EFATAL);
+      else
+        return (GFTP_ERETRYABLE);
     }
 
   gftp_lookup_request_option (request, "passive_transfer", &passive_transfer);
@@ -1145,6 +1211,25 @@ rfc959_get_file (gftp_request * request, const char *filename, int fd,
       (ret = rfc959_accept_active_connection (request)) < 0)
     return (ret);
 
+  return (0);
+}
+
+
+static off_t
+rfc959_get_file (gftp_request * request, const char *filename,
+                 off_t startsize)
+{
+  char *tempstr;
+  int ret;
+
+  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (filename != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
+
+  ret = rfc959_setup_file_transfer (request, filename, startsize, "RETR");
+  if (ret < 0)
+    return (ret);
+
   if ((tempstr = strrchr (request->last_ftp_response, '(')) == NULL)
     {
       tempstr = request->last_ftp_response + 4;
@@ -1159,11 +1244,9 @@ rfc959_get_file (gftp_request * request, const char *filename, int fd,
 
 
 static int
-rfc959_put_file (gftp_request * request, const char *filename, int fd,
+rfc959_put_file (gftp_request * request, const char *filename,
                  off_t startsize, off_t totalsize)
 {
-  intptr_t passive_transfer;
-  char *command, *tempstr;
   rfc959_parms * parms;
   int ret;
 
@@ -1171,58 +1254,19 @@ rfc959_put_file (gftp_request * request, const char *filename, int fd,
   g_return_val_if_fail (filename != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
 
-  parms = request->protocol_data;
-  if (fd > 0)
-    fd = parms->data_connection;
-
   if ((ret = rfc959_set_data_type (request, filename)) < 0)
     return (ret);
 
+  parms = request->protocol_data;
   if (parms->data_connection < 0 && 
-      (ret = rfc959_data_connection_new (request)) < 0)
+      (ret = rfc959_data_connection_new (request, 0)) < 0)
     return (ret);
 
-  if ((ret = gftp_fd_set_sockblocking (request, parms->data_connection, 1)) < 0)
-    return (ret);
-
-  if (startsize > 0)
-    {
-      command = g_strdup_printf ("REST " GFTP_OFF_T_PRINTF_MOD "\r\n",
-                                 startsize);
-      ret = rfc959_send_command (request, command, 1);
-      g_free (command);
-      if (ret < 0)
-        return (ret);
-      else if (ret != '3')
-        {
-          close (parms->data_connection);
-          parms->data_connection = -1;
-	  return (GFTP_ERETRYABLE);
-        }
-    }
-
-  tempstr = g_strconcat ("STOR ", filename, "\r\n", NULL);
-  ret = rfc959_send_command (request, tempstr, 1);
-  g_free (tempstr);
-  if (ret < 0)
-    return (ret);
-  else if (ret != '1')
-    {
-      close (parms->data_connection);
-      parms->data_connection = -1;
-      return (GFTP_ERETRYABLE);
-    }
-
-  gftp_lookup_request_option (request, "passive_transfer", &passive_transfer);
-  if (!passive_transfer &&
-      (ret = rfc959_accept_active_connection (request)) < 0)
-    return (ret);
-
-  return (0);
+  return (rfc959_setup_file_transfer (request, filename, startsize, "STOR"));
 }
 
 
-static long 
+static off_t
 rfc959_transfer_file (gftp_request *fromreq, const char *fromfile, 
                       off_t fromsize, gftp_request *toreq, 
                       const char *tofile, off_t tosize)
@@ -1238,7 +1282,7 @@ rfc959_transfer_file (gftp_request *fromreq, const char *fromfile,
   g_return_val_if_fail (fromreq->datafd > 0, GFTP_EFATAL);
   g_return_val_if_fail (toreq->datafd > 0, GFTP_EFATAL);
 
-  if ((ret = rfc959_send_command (fromreq, "PASV\r\n", 1)) < 0)
+  if ((ret = rfc959_send_command (fromreq, "PASV\r\n", -1, 1, 0)) < 0)
     return (ret);
   else if (ret != '2')
     return (GFTP_ERETRYABLE);
@@ -1256,7 +1300,7 @@ rfc959_transfer_file (gftp_request *fromreq, const char *fromfile,
     *endpos = '\0';
 
   tempstr = g_strconcat ("PORT ", pos, "\r\n", NULL);
-  ret = rfc959_send_command (toreq, tempstr, 1);
+  ret = rfc959_send_command (toreq, tempstr, -1, 1, 0);
   g_free (tempstr);
 
   if (ret < 0)
@@ -1264,17 +1308,11 @@ rfc959_transfer_file (gftp_request *fromreq, const char *fromfile,
   else if (ret != '2')
     return (GFTP_ERETRYABLE);
 
-  tempstr = g_strconcat ("RETR ", fromfile, "\r\n", NULL);
-  ret = rfc959_send_command (fromreq, tempstr, 0);
-  g_free (tempstr);
-
+  ret = rfc959_generate_and_send_command (fromreq, "RETR", fromfile, 0, 0);
   if (ret < 0)
     return (ret);
 
-  tempstr = g_strconcat ("STOR ", tofile, "\r\n", NULL);
-  ret = rfc959_send_command (toreq, tempstr, 0);
-  g_free (tempstr);
-
+  ret = rfc959_generate_and_send_command (toreq, "STOR", tofile, 0, 0);
   if (ret < 0)
     return (ret);
 
@@ -1306,11 +1344,7 @@ rfc959_end_transfer (gftp_request * request)
   parms = request->protocol_data;
   parms->is_fxp_transfer = 0;
 
-  if (parms->data_connection > 0)
-    {
-      close (parms->data_connection);
-      parms->data_connection = -1;
-    }
+  rfc959_close_data_connection (request);
 
   ret = rfc959_read_response (request, 1);
 
@@ -1326,22 +1360,15 @@ rfc959_end_transfer (gftp_request * request)
 static int
 rfc959_abort_transfer (gftp_request * request)
 {
-  rfc959_parms * parms;
   int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
 
-  parms = request->protocol_data;
-
-  if ((ret = rfc959_send_command (request, "ABOR\r\n", 0)) < 0)
+  if ((ret = rfc959_send_command (request, "ABOR\r\n", -1, 0, 0)) < 0)
     return (ret);
 
-  if (parms->data_connection > 0)
-    {
-      close (parms->data_connection);
-      parms->data_connection = -1;
-    }
+  rfc959_close_data_connection (request);
 
   if (request->datafd > 0)
     {
@@ -1363,7 +1390,7 @@ rfc959_list_files (gftp_request * request)
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
 
-  if ((ret = rfc959_data_connection_new (request)) < 0)
+  if ((ret = rfc959_data_connection_new (request, 0)) < 0)
     return (ret);
 
   gftp_lookup_request_option (request, "show_hidden_files", &show_hidden_files);
@@ -1376,7 +1403,7 @@ rfc959_list_files (gftp_request * request)
   tempstr = g_strconcat ("LIST", *parms != '\0' ? " -" : "", parms, "\r\n", 
                          NULL); 
 
-  ret = rfc959_send_command (request, tempstr, 1);
+  ret = rfc959_send_command (request, tempstr, -1, 1, 0);
   g_free (tempstr);
 
   if (ret < 0)
@@ -1431,13 +1458,10 @@ rfc959_get_next_file_chunk (gftp_request * request, char *buf, size_t size)
 static ssize_t
 rfc959_put_next_file_chunk (gftp_request * request, char *buf, size_t size)
 {
+  ssize_t num_wrote, ret;
   rfc959_parms * parms;
+  char *tempstr, *pos;
   size_t rsize, i, j;
-  ssize_t num_wrote;
-  char *tempstr;
-
-  if (size == 0)
-    return (0);
 
   parms = request->protocol_data;
 
@@ -1446,17 +1470,16 @@ rfc959_put_next_file_chunk (gftp_request * request, char *buf, size_t size)
 
   if (parms->is_ascii_transfer)
     {
-      rsize = 0;
-      for (i = 0; i < size; i++)
+      rsize = size;
+      for (i = 1; i < size; i++)
         {
-          rsize++;
-          if (i > 0 && buf[i] == '\n' && buf[i - 1] != '\r')
+          if (buf[i] == '\n' && buf[i - 1] != '\r')
             rsize++;
         }
 
       if (rsize != size)
         {
-          tempstr = g_malloc (rsize);
+          tempstr = g_malloc0 (rsize);
 
           for (i = 0, j = 0; i < size; i++)
             {
@@ -1474,13 +1497,32 @@ rfc959_put_next_file_chunk (gftp_request * request, char *buf, size_t size)
       tempstr = buf;
     }
 
-  num_wrote = parms->data_conn_write (request, tempstr, rsize,
-                                      parms->data_connection);
+  /* I need to ensure that the entire buffer has been transfered properly due
+     to the ascii conversion that may occur. 
+     FIXME - the ascii conversion doesn't occur properly when the \n occurs
+     at the beginning of the buffer. I need to remember the last character
+     in the previous buffer */
+
+  ret = rsize;
+  pos = tempstr;
+  while (rsize > 0)
+    {
+      num_wrote = parms->data_conn_write (request, pos, rsize,
+                                          parms->data_connection);
+      if (num_wrote < 0)
+        {
+          ret = num_wrote;
+          break;
+        }
+
+      pos += num_wrote;
+      rsize -= num_wrote;
+    }
 
   if (tempstr != buf)
     g_free (tempstr);
 
-  return (num_wrote);
+  return (ret);
 }
 
 
@@ -1564,21 +1606,19 @@ rfc959_get_next_file (gftp_request * request, gftp_file * fle, int fd)
 static off_t
 rfc959_get_file_size (gftp_request * request, const char *filename)
 {
-  char *tempstr;
   int ret;
 
   g_return_val_if_fail (request != NULL, 0);
   g_return_val_if_fail (filename != NULL, 0);
   g_return_val_if_fail (request->datafd > 0, 0);
 
-  tempstr = g_strconcat ("SIZE ", filename, "\r\n", NULL);
-  ret = rfc959_send_command (request, tempstr, 1);
-  g_free (tempstr);
+  ret = rfc959_generate_and_send_command (request, "SIZE", filename, 1, 0);
   if (ret < 0)
     return (ret);
 
   if (*request->last_ftp_response != '2')
     return (0);
+
   return (strtol (request->last_ftp_response + 4, NULL, 10));
 }
 
@@ -1586,17 +1626,13 @@ rfc959_get_file_size (gftp_request * request, const char *filename)
 static int
 rfc959_rmdir (gftp_request * request, const char *directory)
 {
-  char *tempstr;
   int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (directory != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
 
-  tempstr = g_strconcat ("RMD ", directory, "\r\n", NULL);
-  ret = rfc959_send_command (request, tempstr, 1);
-  g_free (tempstr);
-
+  ret = rfc959_generate_and_send_command (request, "RMD", directory, 1, 0);
   if (ret < 0)
     return (ret);
   else if (ret == '2')
@@ -1609,17 +1645,13 @@ rfc959_rmdir (gftp_request * request, const char *directory)
 static int
 rfc959_rmfile (gftp_request * request, const char *file)
 {
-  char *tempstr;
   int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (file != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
 
-  tempstr = g_strconcat ("DELE ", file, "\r\n", NULL);
-  ret = rfc959_send_command (request, tempstr, 1);
-  g_free (tempstr);
-
+  ret = rfc959_generate_and_send_command (request, "DELE", file, 1, 0);
   if (ret < 0)
     return (ret);
   else if (ret == '2')
@@ -1632,17 +1664,13 @@ rfc959_rmfile (gftp_request * request, const char *file)
 static int
 rfc959_mkdir (gftp_request * request, const char *directory)
 {
-  char *tempstr;
   int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (directory != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
 
-  tempstr = g_strconcat ("MKD ", directory, "\r\n", NULL);
-  ret = rfc959_send_command (request, tempstr, 1);
-  g_free (tempstr);
-
+  ret = rfc959_generate_and_send_command (request, "MKD", directory, 1, 0);
   if (ret < 0)
     return (ret);
   else if (ret == '2')
@@ -1656,7 +1684,6 @@ static int
 rfc959_rename (gftp_request * request, const char *oldname,
 	       const char *newname)
 {
-  char *tempstr;
   int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
@@ -1664,19 +1691,13 @@ rfc959_rename (gftp_request * request, const char *oldname,
   g_return_val_if_fail (newname != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
 
-  tempstr = g_strconcat ("RNFR ", oldname, "\r\n", NULL);
-  ret = rfc959_send_command (request, tempstr, 1);
-  g_free (tempstr);
-
+  ret = rfc959_generate_and_send_command (request, "RNFR", oldname, 1, 0);
   if (ret < 0)
     return (ret);
   else if (ret != '3')
     return (GFTP_ERETRYABLE);
 
-  tempstr = g_strconcat ("RNTO ", newname, "\r\n", NULL);
-  ret = rfc959_send_command (request, tempstr, 1);
-  g_free (tempstr);
-
+  ret = rfc959_generate_and_send_command (request, "RNTO", newname, 1, 0);
   if (ret < 0)
     return (ret);
   else if (ret == '2')
@@ -1689,18 +1710,24 @@ rfc959_rename (gftp_request * request, const char *oldname,
 static int
 rfc959_chmod (gftp_request * request, const char *file, mode_t mode)
 {
-  rfc959_parms * parms;
-  char *tempstr;
+  char *tempstr, *utf8;
+  size_t destlen;
   int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (file != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
 
-  parms = request->protocol_data;
-  tempstr = g_strdup_printf ("SITE CHMOD %o %s\r\n", mode, file);
+  utf8 = gftp_filename_from_utf8 (request, file, &destlen);
+  if (utf8 != NULL)
+    {
+      tempstr = g_strdup_printf ("SITE CHMOD %o %s\r\n", mode, utf8);
+      g_free (utf8);
+    }
+  else
+    tempstr = g_strdup_printf ("SITE CHMOD %o %s\r\n", mode, file);
 
-  ret = rfc959_send_command (request, tempstr, 1);
+  ret = rfc959_send_command (request, tempstr, -1, 1, 0);
   g_free (tempstr);
 
   if (ret < 0)
@@ -1715,75 +1742,45 @@ rfc959_chmod (gftp_request * request, const char *file, mode_t mode)
 static int
 rfc959_site (gftp_request * request, int specify_site, const char *command)
 {
-  char *tempstr;
+  char *tempstr, *utf8;
+  size_t len;
   int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (command != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
 
-  if (specify_site)
-    tempstr = g_strconcat ("SITE ", command, "\r\n", NULL);
-  else
-    tempstr = g_strconcat (command, "\r\n", NULL);
-
-  ret = rfc959_send_command (request, tempstr, 1);
-  g_free (tempstr);
-
-  if (ret < 0)
-    return (ret);
-  else if (ret == '2')
-    return (0);
-  else
-    return (GFTP_ERETRYABLE);
-}
-
-
-static char *
-rfc959_time_t_to_mdtm (gftp_request * request, time_t datetime)
-{
-  struct tm gt;
-  char *ret;
-
-  if (localtime_r (&datetime, &gt) != NULL)
+  utf8 = gftp_string_from_utf8 (request, -1, command, &len);
+  if (utf8 != NULL)
     {
-      ret = g_strdup_printf ("%04d%02d%02d%02d%02d%02d", gt.tm_year + 1900,
-                             gt.tm_mon + 1, gt.tm_mday, gt.tm_hour, gt.tm_min,
-                             gt.tm_sec);
-      return (ret);
+      if (specify_site)
+        {
+          len += 7;
+          tempstr = g_strconcat ("SITE ", utf8, "\r\n", NULL);
+        }
+      else
+        {
+          len += 2;
+          tempstr = g_strconcat (utf8, "\r\n", NULL);
+        }
+
+      g_free (utf8);
     }
   else
     {
-      request->logging_function (gftp_logging_error, request,
-                                 "Cannot parse UNIX timestamp %d: %s\n",
-                                 datetime, g_strerror (errno));
-      return (NULL);
+      if (specify_site)
+        {
+          len = strlen (command) + 7;
+          tempstr = g_strconcat ("SITE ", command, "\r\n", NULL);
+        }
+      else
+        {
+          len = strlen (command) + 2;
+          tempstr = g_strconcat (command, "\r\n", NULL);
+        }
     }
-}
 
-
-static int
-rfc959_set_file_time (gftp_request * request, const char *file, time_t datetime)
-{
-  char *tempstr, *datestr;
-  rfc959_parms * parms;
-  int ret;
-
-  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
-  g_return_val_if_fail (file != NULL, GFTP_EFATAL);
-  g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
-
-  parms = request->protocol_data;
-
-  datestr = rfc959_time_t_to_mdtm (request, datetime);
-  if (datestr == NULL)
-    return (GFTP_EFATAL);
-
-  tempstr = g_strconcat ("SITE UTIME ", datestr, " ", file, "\r\n", NULL);
-
-  g_free (datestr);
-
-  ret = rfc959_send_command (request, tempstr, 1);
+  ret = rfc959_send_command (request, tempstr, len, 1, 0);
   g_free (tempstr);
 
   if (ret < 0)
@@ -1911,7 +1908,7 @@ rfc959_init (gftp_request * request)
   request->mkdir = rfc959_mkdir;
   request->rename = rfc959_rename;
   request->chmod = rfc959_chmod;
-  request->set_file_time = rfc959_set_file_time;
+  request->set_file_time = NULL;
   request->site = rfc959_site;
   request->parse_url = NULL;
   request->swap_socks = NULL;
@@ -1922,6 +1919,7 @@ rfc959_init (gftp_request * request)
   request->need_password = 1;
   request->use_cache = 1;
   request->always_connected = 0;
+  request->use_local_encoding = 0;
 
   request->protocol_data = g_malloc0 (sizeof (rfc959_parms));
   parms = request->protocol_data;
