@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  gftp-gtk.c - GTK+ 1.2 port of gftp                                       */
-/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,22 +18,22 @@
 /*****************************************************************************/
 
 #include "gftp-gtk.h"
-static const char cvsid[] = "$Id: gftp-gtk.c,v 1.66 2005/01/25 02:34:19 masneyb Exp $";
+static const char cvsid[] = "$Id: gftp-gtk.c 949 2008-01-24 23:26:45Z masneyb $";
 
 static GtkItemFactory *log_factory, *dl_factory;
 static GtkWidget * local_frame, * remote_frame, * log_table, * transfer_scroll,
-                 * gftpui_command_toolbar;
+                 * gftpui_command_toolbar, * protocol_menu;
 
 gftp_window_data window1, window2, *other_wdata, *current_wdata;
 GtkWidget * stop_btn, * hostedit, * useredit, * passedit, * portedit, * logwdw,
-          * dlwdw, * protocol_menu, * optionmenu, * gftpui_command_widget,
-          * download_left_arrow, * upload_right_arrow, * openurl_btn;
+          * dlwdw, * optionmenu, * gftpui_command_widget, * download_left_arrow,
+          * upload_right_arrow, * openurl_btn;
 GtkTooltips * openurl_tooltip;
 GtkAdjustment * logwdw_vadj;
 #if GTK_MAJOR_VERSION > 1
 GtkTextMark * logwdw_textmark;
 #endif
-int local_start, remote_start, trans_start, log_start, tools_start;
+int local_start, remote_start, trans_start;
 GHashTable * graphic_hash_table = NULL;
 GtkItemFactoryEntry * menus = NULL;
 GtkItemFactory * factory = NULL;
@@ -57,7 +57,9 @@ get_column (GtkCListColumn * col)
 static void
 _gftp_exit (GtkWidget * widget, gpointer data)
 {
+  intptr_t remember_last_directory;
   const char *tempstr;
+  GtkWidget * tempwid;
   intptr_t ret;
 
   ret = GTK_WIDGET (local_frame)->allocation.width;
@@ -109,6 +111,21 @@ _gftp_exit (GtkWidget * widget, gpointer data)
   tempstr = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (useredit)->entry));
   gftp_set_global_option ("user_value", tempstr);
 
+  gftp_lookup_global_option ("remember_last_directory",
+                             &remember_last_directory);
+  if (remember_last_directory)
+    {
+      tempstr = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (window1.combo)->entry));
+      gftp_set_global_option ("local_startup_directory", tempstr);
+
+      tempstr = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (window2.combo)->entry));
+      gftp_set_global_option ("remote_startup_directory", tempstr);
+    }
+
+  tempwid = gtk_menu_get_active (GTK_MENU (protocol_menu));
+  ret = GPOINTER_TO_INT (gtk_object_get_user_data (GTK_OBJECT (tempwid)));
+  gftp_set_global_option ("default_protocol", gftp_protocols[ret].name);
+
   gftp_shutdown ();
   exit (0);
 }
@@ -159,14 +176,18 @@ change_setting (gftp_window_data * wdata, int menuitem, GtkWidget * checkmenu)
     case GFTP_MENU_ITEM_WIN1:
       current_wdata = &window1;
       other_wdata = &window2;
+
       if (wdata->request != NULL)
         update_window_info ();
+
       break;
     case GFTP_MENU_ITEM_WIN2:
       current_wdata = &window2;
       other_wdata = &window1;
+
       if (wdata->request != NULL)
         update_window_info ();
+
       break;
     }
 }
@@ -195,7 +216,7 @@ openurl_dialog (gpointer data)
   gftp_window_data * wdata;
 
   wdata = data;
-  MakeEditDialog (_("Connect via URL"), _("Enter a URL to connect to"),
+  MakeEditDialog (_("Open Location"), _("Enter a URL to connect to"),
                   NULL, 1, NULL, gftp_dialog_button_connect,
                   _gftpui_gtk_do_openurl, wdata,
                   NULL, NULL);
@@ -211,7 +232,7 @@ tb_openurl_dialog (gpointer data)
     {
       ftp_log (gftp_logging_error, NULL,
                _("%s: Please hit the stop button first to do anything else\n"),
-               _("OpenURL"));
+               _("Open Location"));
       return;
     }
 
@@ -236,105 +257,115 @@ gftp_gtk_refresh (gftp_window_data * wdata)
 static GtkWidget *
 CreateMenus (GtkWidget * parent)
 {
-  int local_len, remote_len, len, i, trans_len, log_len, tools_len;
-  GtkAccelGroup *accel_group;
+  int local_len, remote_len, len, i, trans_len, log_len, tools_len, log_start;
+  GtkAccelGroup * accel_group;
   intptr_t ascii_transfers;
   GtkWidget * tempwid;
   static GtkItemFactoryEntry menu_items[] = {
     {N_("/_FTP"), NULL, 0, 0, MN_("<Branch>")},
     {N_("/FTP/tearoff"), NULL, 0, 0, MN_("<Tearoff>")},
-    {N_("/FTP/Window 1"), NULL, change_setting, GFTP_MENU_ITEM_WIN1,
+    {N_("/FTP/Window _1"), "<control>1", change_setting, GFTP_MENU_ITEM_WIN1,
      MN_("<RadioItem>")},
-    {N_("/FTP/Window 2"), NULL, change_setting, GFTP_MENU_ITEM_WIN2,
+    {N_("/FTP/Window _2"), "<control>2", change_setting, GFTP_MENU_ITEM_WIN2,
      MN_("/FTP/Window 1")},
     {N_("/FTP/sep"), NULL, 0, 0, MN_("<Separator>")},
-    {N_("/FTP/Ascii"), NULL, change_setting, GFTP_MENU_ITEM_ASCII,
+    {N_("/FTP/_Ascii"), NULL, change_setting, GFTP_MENU_ITEM_ASCII,
      MN_("<RadioItem>")},
-    {N_("/FTP/Binary"), NULL, change_setting, GFTP_MENU_ITEM_BINARY,
+    {N_("/FTP/_Binary"), NULL, change_setting, GFTP_MENU_ITEM_BINARY,
      MN_("/FTP/Ascii")},
     {N_("/FTP/sep"), NULL, 0, 0, MN_("<Separator>")},
-    {N_("/FTP/_Options..."), "<control>O", options_dialog, 0,
+    {N_("/FTP/_Preferences..."), NULL, options_dialog, 0,
 	MS_(GTK_STOCK_PREFERENCES)},
     {N_("/FTP/sep"), NULL, 0, 0, MN_("<Separator>")},
     {N_("/FTP/_Quit"), "<control>Q", _gftp_menu_exit, 0, MS_(GTK_STOCK_QUIT)},
     {N_("/_Local"), NULL, 0, 0, MN_("<Branch>")},
     {N_("/Local/tearoff"), NULL, 0, 0, MN_("<Tearoff>")},
-    {N_("/Local/Open URL..."), "", openurl_dialog, 0, MS_(GTK_STOCK_OPEN)},
-    {N_("/Local/Disconnect"), "", gftpui_disconnect, 0, MS_(GTK_STOCK_CLOSE)},
+    {N_("/Local/_Open Location..."), "<control><shift>O", openurl_dialog, 0,
+        MS_(GTK_STOCK_OPEN)},
+    {N_("/Local/D_isconnect"), "<control><shift>I", gftpui_disconnect, 0, MS_(GTK_STOCK_CLOSE)},
     {N_("/Local/sep"), NULL, 0, 0, MN_("<Separator>")},
-    {N_("/Local/Change Filespec..."), NULL, change_filespec, 0, MN_(NULL)},
-    {N_("/Local/Show selected"), NULL, show_selected, 0, MN_(NULL)},
-    {N_("/Local/Select All"), NULL, selectall, 0, MN_(NULL)},
+    {N_("/Local/Change _Filespec..."), "<control><shift>F", change_filespec, 0,
+     MN_(NULL)},
+    {N_("/Local/_Show selected"), NULL, show_selected, 0, MN_(NULL)},
+    {N_("/Local/Select _All"), "<control><shift>A", selectall, 0, MN_(NULL)},
     {N_("/Local/Select All Files"), NULL, selectallfiles, 0, MN_(NULL)},
     {N_("/Local/Deselect All"), NULL, deselectall, 0, MN_(NULL)},
     {N_("/Local/sep"), NULL, 0, 0, MN_("<Separator>")},
     {N_("/Local/Save Directory Listing..."), NULL, save_directory_listing, 0, MN_(NULL)},
     {N_("/Local/Send SITE Command..."), NULL, gftpui_site_dialog, 0, MN_(NULL)},
-    {N_("/Local/Change Directory"), NULL, gftpui_chdir_dialog, 0, MN_(NULL)},
-    {N_("/Local/Chmod..."), NULL, chmod_dialog, 0, MN_(NULL)},
-    {N_("/Local/Make Directory..."), NULL, gftpui_mkdir_dialog, 0, MN_(NULL)},
-    {N_("/Local/Rename..."), NULL, gftpui_rename_dialog, 0, MN_(NULL)},
-    {N_("/Local/Delete..."), NULL, delete_dialog, 0, MN_(NULL)},
-    {N_("/Local/Edit..."), NULL, edit_dialog, 0, MN_(NULL)},
-    {N_("/Local/View..."), NULL, view_dialog, 0, MN_(NULL)},
-    {N_("/Local/Refresh"), NULL, gftp_gtk_refresh, 0, MS_(GTK_STOCK_REFRESH)},
+    {N_("/Local/_Change Directory"), NULL, gftpui_chdir_dialog, 0, MN_(NULL)},
+    {N_("/Local/_Permissions..."), "<control><shift>P", chmod_dialog, 0,
+        MN_(NULL)},
+    {N_("/Local/_New Folder..."), "<control><shift>N", gftpui_mkdir_dialog, 0, MN_(NULL)},
+    {N_("/Local/Rena_me..."), "<control><shift>M", gftpui_rename_dialog, 0,
+        MN_(NULL)},
+    {N_("/Local/_Delete..."), "<control><shift>D", delete_dialog, 0,
+        MN_(NULL)},
+    {N_("/Local/_Edit..."), "<control><shift>E", edit_dialog, 0, MN_(NULL)},
+    {N_("/Local/_View..."), "<control><shift>V", view_dialog, 0, MN_(NULL)},
+    {N_("/Local/_Refresh"), "<control><shift>R", gftp_gtk_refresh, 0,
+        MS_(GTK_STOCK_REFRESH)},
     {N_("/_Remote"), NULL, 0, 0, MN_("<Branch>")},
     {N_("/Remote/tearoff"), NULL, 0, 0, MN_("<Tearoff>")},
-    {N_("/Remote/Open _URL..."), "<control>U", openurl_dialog, 0,
+    {N_("/Remote/_Open Location..."), "<control>O", openurl_dialog, 0,
 	MS_(GTK_STOCK_OPEN)},
-    {N_("/Remote/Disconnect"), "<control>D", gftpui_disconnect, 0,
+    {N_("/Remote/D_isconnect"), "<control>D", gftpui_disconnect, 0,
 	MS_(GTK_STOCK_CLOSE)},
     {N_("/Remote/sep"), NULL, 0, 0, MN_("<Separator>")},
-    {N_("/Remote/Change Filespec..."), NULL, change_filespec, 0, MN_(NULL)},
-    {N_("/Remote/Show selected"), NULL, show_selected, 0, MN_(NULL)},
-    {N_("/Remote/Select All"), NULL, selectall, 0, MN_(NULL)},
+    {N_("/Remote/Change _Filespec..."), "<control>F", change_filespec, 0,
+        MN_(NULL)},
+    {N_("/Remote/_Show selected"), NULL, show_selected, 0, MN_(NULL)},
+    {N_("/Remote/Select _All"), "<control>A", selectall, 0, MN_(NULL)},
     {N_("/Remote/Select All Files"), NULL, selectallfiles, 0, MN_(NULL)},
     {N_("/Remote/Deselect All"), NULL, deselectall, 0, MN_(NULL)},
     {N_("/Remote/sep"), NULL, 0, 0, MN_("<Separator>")},
     {N_("/Remote/Save Directory Listing..."), NULL, save_directory_listing, 0, MN_(NULL)},
     {N_("/Remote/Send SITE Command..."), NULL, gftpui_site_dialog, 0, MN_(NULL)},
-    {N_("/Remote/Change Directory"), NULL, gftpui_chdir_dialog, 0, MN_(NULL)},
-    {N_("/Remote/Chmod..."), NULL, chmod_dialog, 0, MN_(NULL)},
-    {N_("/Remote/Make Directory..."), NULL, gftpui_mkdir_dialog, 0, MN_(NULL)},
-    {N_("/Remote/Rename..."), NULL, gftpui_rename_dialog, 0, MN_(NULL)},
-    {N_("/Remote/Delete..."), NULL, delete_dialog, 0, MN_(NULL)},
-    {N_("/Remote/Edit..."), NULL, edit_dialog, 0, MN_(NULL)},
-    {N_("/Remote/View..."), NULL, view_dialog, 0, MN_(NULL)},
-    {N_("/Remote/Refresh"), NULL, gftp_gtk_refresh, 0, MS_(GTK_STOCK_REFRESH)},
+    {N_("/Remote/_Change Directory"), NULL, gftpui_chdir_dialog, 0, MN_(NULL)},
+    {N_("/Remote/_Permisssions..."), "<control>P", chmod_dialog, 0, MN_(NULL)},
+    {N_("/Remote/_New Folder..."), "<control>N", gftpui_mkdir_dialog, 0,
+        MN_(NULL)},
+    {N_("/Remote/Rena_me..."), "<control>M", gftpui_rename_dialog, 0,
+        MN_(NULL)},
+    {N_("/Remote/_Delete..."), "<control>D", delete_dialog, 0, MN_(NULL)},
+    {N_("/Remote/_Edit..."), "<control>E", edit_dialog, 0, MN_(NULL)},
+    {N_("/Remote/_View..."), "<control>V", view_dialog, 0, MN_(NULL)},
+    {N_("/Remote/_Refresh"), "<control>R", gftp_gtk_refresh, 0,
+        MS_(GTK_STOCK_REFRESH)},
     {N_("/_Bookmarks"), NULL, 0, 0, MN_("<Branch>")},
     {N_("/Bookmarks/tearoff"), NULL, 0, 0, MN_("<Tearoff>")},
-    {N_("/Bookmarks/Add bookmark"), "<control>A", add_bookmark, 0,
+    {N_("/Bookmarks/Add _Bookmark"), "<control>B", add_bookmark, 0,
 	MS_(GTK_STOCK_ADD)},
-    {N_("/Bookmarks/Edit bookmarks"), NULL, edit_bookmarks, 0, MN_(NULL)},
+    {N_("/Bookmarks/Edit Bookmarks"), NULL, edit_bookmarks, 0, MN_(NULL)},
     {N_("/Bookmarks/sep"), NULL, 0, 0, MN_("<Separator>")},
-    {N_("/_Transfers"), NULL, 0, 0, MN_("<Branch>")},
-    {N_("/Transfers/tearoff"), NULL, 0, 0, MN_("<Tearoff>")},
-    {N_("/Transfers/Start Transfer"), NULL, start_transfer, 0, MN_(NULL)},
-    {N_("/Transfers/Stop Transfer"), NULL, stop_transfer, 0,
+    {N_("/_Transfer"), NULL, 0, 0, MN_("<Branch>")},
+    {N_("/Transfer/tearoff"), NULL, 0, 0, MN_("<Tearoff>")},
+    {N_("/Transfer/_Start"), NULL, start_transfer, 0, MN_(NULL)},
+    {N_("/Transfer/St_op"), NULL, stop_transfer, 0,
 	MS_(GTK_STOCK_STOP)},
-    {N_("/Transfers/sep"), NULL, 0, 0, MN_("<Separator>")},
-    {N_("/Transfers/Skip Current File"), NULL, skip_transfer, 0, MN_(NULL)},
-    {N_("/Transfers/Remove File"), NULL, remove_file_transfer, 0,
+    {N_("/Transfer/sep"), NULL, 0, 0, MN_("<Separator>")},
+    {N_("/Transfer/Skip _Current File"), NULL, skip_transfer, 0, MN_(NULL)},
+    {N_("/Transfer/_Remove File"), NULL, remove_file_transfer, 0,
 	MS_(GTK_STOCK_DELETE)},
-    {N_("/Transfers/Move File Up"), NULL, move_transfer_up, 0,
+    {N_("/Transfer/Move File _Up"), NULL, move_transfer_up, 0,
 	MS_(GTK_STOCK_GO_UP)},
-    {N_("/Transfers/Move File Down"), NULL, move_transfer_down, 0,
+    {N_("/Transfer/Move File _Down"), NULL, move_transfer_down, 0,
 	MS_(GTK_STOCK_GO_DOWN)},
-    {N_("/Transfers/sep"), NULL, 0, 0, MN_("<Separator>")},
-    {N_("/Transfers/Retrieve Files"), "<control>R", get_files, 0, MN_(NULL)},
-    {N_("/Transfers/Put Files"), "<control>P", put_files, 0, MN_(NULL)},
-    {N_("/L_ogging"), NULL, 0, 0, MN_("<Branch>")},
-    {N_("/Logging/tearoff"), NULL, 0, 0, MN_("<Tearoff>")},
-    {N_("/Logging/Clear"), NULL, clearlog, 0, MS_(GTK_STOCK_CLEAR)},
-    {N_("/Logging/View log"), NULL, viewlog, 0, MN_(NULL)},
-    {N_("/Logging/Save log..."), NULL, savelog, 0, MS_(GTK_STOCK_SAVE)},
+    {N_("/Transfer/sep"), NULL, 0, 0, MN_("<Separator>")},
+    {N_("/Transfer/_Retrieve Files"), "<control>R", get_files, 0, MN_(NULL)},
+    {N_("/Transfer/_Put Files"), "<control>P", put_files, 0, MN_(NULL)},
+    {N_("/L_og"), NULL, 0, 0, MN_("<Branch>")},
+    {N_("/Log/tearoff"), NULL, 0, 0, MN_("<Tearoff>")},
+    {N_("/Log/_Clear"), NULL, clearlog, 0, MS_(GTK_STOCK_CLEAR)},
+    {N_("/Log/_View"), NULL, viewlog, 0, MN_(NULL)},
+    {N_("/Log/_Save..."), NULL, savelog, 0, MS_(GTK_STOCK_SAVE)},
     {N_("/Tool_s"), NULL, 0, 0, MN_("<Branch>")},
     {N_("/Tools/tearoff"), NULL, 0, 0, MN_("<Tearoff>")},
-    {N_("/Tools/Compare Windows"), NULL, compare_windows, 0, MN_(NULL)},
-    {N_("/Tools/Clear Cache"), NULL, clear_cache, 0, MS_(GTK_STOCK_CLEAR)},
+    {N_("/Tools/C_ompare Windows"), NULL, compare_windows, 0, MN_(NULL)},
+    {N_("/Tools/_Clear Cache"), NULL, clear_cache, 0, MS_(GTK_STOCK_CLEAR)},
     {N_("/Help"), NULL, 0, 0, MN_("<Branch>")},
     {N_("/Help/tearoff"), NULL, 0, 0, MN_("<Tearoff>")},
-    {N_("/Help/About"), NULL, about_dialog, 0, MS_(GTK_STOCK_HELP)}
+    {N_("/Help/_About"), NULL, about_dialog, 0, MS_(GTK_STOCK_ABOUT)}
   };
 
   menus = menu_items;
@@ -365,7 +396,7 @@ CreateMenus (GtkWidget * parent)
   create_item_factory (factory, len, menu_items + i, &window2);
 
   i += len;
-  /* Transfers Menu */
+  /* Transfer Menu */
   trans_start = i;
   trans_len = 12;
   create_item_factory (factory, trans_len, menu_items + i, NULL);
@@ -378,7 +409,6 @@ CreateMenus (GtkWidget * parent)
 
   i += log_len;
   /* Tools Menu */
-  tools_start = i;
   tools_len = 4;
   create_item_factory (factory, tools_len, menu_items + i, NULL);
 
@@ -393,16 +423,16 @@ CreateMenus (GtkWidget * parent)
   gftp_lookup_global_option ("ascii_transfers", &ascii_transfers);
   if (ascii_transfers)
     {
-      tempwid = gtk_item_factory_get_widget (factory, menu_items[5].path);
+      tempwid = gtk_item_factory_get_widget (factory, "/FTP/Ascii");
       gtk_check_menu_item_set_state (GTK_CHECK_MENU_ITEM (tempwid), TRUE);
     }
   else
     {
-      tempwid = gtk_item_factory_get_widget (factory, menu_items[6].path);
+      tempwid = gtk_item_factory_get_widget (factory, "/FTP/Binary");
       gtk_check_menu_item_set_state (GTK_CHECK_MENU_ITEM (tempwid), TRUE);
     }
 
-  tempwid = gtk_item_factory_get_widget (factory, menu_items[3].path);
+  tempwid = gtk_item_factory_get_widget (factory, "/FTP/Window 2");
   gtk_check_menu_item_set_state (GTK_CHECK_MENU_ITEM (tempwid), TRUE);
 
   window1.ifactory = item_factory_new (GTK_TYPE_MENU, "<local>", NULL, "/Local");
@@ -414,7 +444,7 @@ CreateMenus (GtkWidget * parent)
   log_factory = item_factory_new (GTK_TYPE_MENU, "<log>", NULL, "/Logging");
   create_item_factory (log_factory, log_len - 2, menu_items + log_start + 2, NULL);
 
-  dl_factory = item_factory_new (GTK_TYPE_MENU, "<download>", NULL, "/Transfers");
+  dl_factory = item_factory_new (GTK_TYPE_MENU, "<download>", NULL, "/Transfer");
   create_item_factory (dl_factory, trans_len - 2, menu_items + trans_start + 2, NULL);
 
   return (factory->widget);
@@ -432,7 +462,7 @@ CreateConnectToolbar (GtkWidget * parent)
   GtkWidget *toolbar, *box, *tempwid;
   gftp_config_list_vars * tmplistvar;
   char *default_protocol, *tempstr;
-  int i, num;
+  int i, j, num;
 
   toolbar = gtk_handle_box_new ();
 
@@ -551,14 +581,16 @@ CreateConnectToolbar (GtkWidget * parent)
   num = 0;
   gftp_lookup_global_option ("default_protocol", &default_protocol);
   protocol_menu = gtk_menu_new ();
-  for (i = 0; gftp_protocols[i].name; i++)
+  for (i = 0, j = 0; gftp_protocols[i].name; i++)
     {
       if (!gftp_protocols[i].shown)
         continue;
 
       if (default_protocol != NULL &&
           strcmp (gftp_protocols[i].name, default_protocol) == 0)
-        num = i;
+        num = j;
+ 
+      j++;
 
       tempwid = gtk_menu_item_new_with_label (gftp_protocols[i].name);
       gtk_object_set_user_data (GTK_OBJECT (tempwid), GINT_TO_POINTER(i));
@@ -730,6 +762,15 @@ select_row_callback (GtkWidget *widget, gint row, gint column,
 }
 
 
+void
+gftp_gtk_init_request (gftp_window_data * wdata)
+{
+  wdata->request->logging_function = ftp_log;
+  wdata->filespec = g_malloc0 (2);
+  *wdata->filespec = '*';
+}
+
+
 static GtkWidget *
 CreateFTPWindow (gftp_window_data * wdata)
 {
@@ -738,9 +779,9 @@ CreateFTPWindow (gftp_window_data * wdata)
     {"text/plain", 0, 0},
     {"application/x-rootwin-drop", 0, 1}
   };
+  char *titles[7], tempstr[50], *startup_directory;
   GtkWidget *box, *scroll_list, *parent;
   intptr_t listbox_file_height, colwidth;
-  char *titles[7], tempstr[50];
 
   titles[0] = "";
   titles[1] = _("Filename");
@@ -751,9 +792,7 @@ CreateFTPWindow (gftp_window_data * wdata)
   titles[6] = _("Attribs");
 
   wdata->request = gftp_request_new ();
-  wdata->request->logging_function = ftp_log;
-  wdata->filespec = g_malloc0 (2);
-  *wdata->filespec = '*';
+  gftp_gtk_init_request (wdata);
 
   parent = gtk_frame_new (NULL);
   
@@ -777,7 +816,12 @@ CreateFTPWindow (gftp_window_data * wdata)
   if (*wdata->history)
     gtk_combo_set_popdown_strings (GTK_COMBO (wdata->combo), *wdata->history);
   gtk_combo_disable_activate (GTK_COMBO (wdata->combo));
-  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (wdata->combo)->entry), "");
+
+  g_snprintf (tempstr, sizeof (tempstr), "%s_startup_directory",
+              wdata->prefix_col_str);
+  gftp_lookup_global_option (tempstr, &startup_directory);
+  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (wdata->combo)->entry),
+                      startup_directory);
 
   wdata->hoststxt = gtk_label_new (NULL);
   gtk_misc_set_alignment (GTK_MISC (wdata->hoststxt), 0, 0);
@@ -1035,16 +1079,13 @@ CreateFTPWindows (GtkWidget * ui)
 }
 
 
-static void
-init_gftp (int argc, char *argv[], GtkWidget * parent)
+static int
+_get_selected_protocol ()
 {
-  if (argc == 2 && strncmp (argv[1], "--", 2) != 0)
-    {
-      if (gftp_parse_url (window2.request, argv[1]) == 0)
-	ftp_connect (&window2, window2.request, 1);
-      else
-	gftp_usage ();
-    }
+  GtkWidget * tempwid;
+  
+  tempwid = gtk_menu_get_active (GTK_MENU (protocol_menu));
+  return (GPOINTER_TO_INT (gtk_object_get_user_data (GTK_OBJECT (tempwid))));
 }
 
 
@@ -1053,7 +1094,6 @@ toolbar_hostedit (GtkWidget * widget, gpointer data)
 {
   int (*init) (gftp_request * request);
   gftp_config_list_vars * tmplistvar;
-  GtkWidget *tempwid;
   const char *txt;
   int num;
 
@@ -1068,8 +1108,7 @@ toolbar_hostedit (GtkWidget * widget, gpointer data)
   if (GFTP_IS_CONNECTED (current_wdata->request))
     gftp_disconnect (current_wdata->request);
 
-  tempwid = gtk_menu_get_active (GTK_MENU (protocol_menu));
-  num = GPOINTER_TO_INT (gtk_object_get_user_data (GTK_OBJECT (tempwid)));
+  num = _get_selected_protocol ();
   init = gftp_protocols[num].init;
   if (init (current_wdata->request) < 0)
     return;
@@ -1103,7 +1142,7 @@ toolbar_hostedit (GtkWidget * widget, gpointer data)
       gftp_parse_url (current_wdata->request, 
                       current_wdata->request->hostname) == 0)
     {
-      ftp_connect (current_wdata, current_wdata->request, 1);
+      ftp_connect (current_wdata, current_wdata->request);
       return;
     }
  
@@ -1132,7 +1171,7 @@ toolbar_hostedit (GtkWidget * widget, gpointer data)
   add_history (current_wdata->combo, current_wdata->history, 
                current_wdata->histlen, current_wdata->request->directory);
 
-  ftp_connect (current_wdata, current_wdata->request, 1);
+  ftp_connect (current_wdata, current_wdata->request);
 }
 
 
@@ -1209,7 +1248,6 @@ sortrows (GtkCList * clist, gint column, gpointer data)
 
   wdata->sorted = 1;
   gtk_clist_thaw (clist);
-  update_window_info ();
 }
 
 
@@ -1220,9 +1258,15 @@ stop_button (GtkWidget * widget, gpointer data)
 
   memset (&comptid, 0, sizeof (comptid));
   if (!pthread_equal (comptid, window1.tid))
-    pthread_kill (window1.tid, SIGINT);
+    {
+      window1.request->cancel = 1;
+      pthread_kill (window1.tid, SIGINT);
+    }
   else if (!pthread_equal (comptid, window2.tid))
-    pthread_kill (window2.tid, SIGINT);
+    {
+      window2.request->cancel = 1;
+      pthread_kill (window2.tid, SIGINT);
+    }
 }
 
 
@@ -1237,7 +1281,7 @@ gftp_gtk_config_file_read_color (char *str, gftp_config_vars * cv, int line)
 
   gftp_config_parse_args (str, 3, line, &red, &green, &blue);
 
-  color = g_malloc (sizeof (*color));
+  color = g_malloc0 (sizeof (*color));
   color->red = strtol (red, NULL, 16);
   color->green = strtol (green, NULL, 16);
   color->blue = strtol (blue, NULL, 16);
@@ -1265,6 +1309,53 @@ gftp_gtk_config_file_write_color (gftp_config_vars * cv, char *buf,
 }
 
 
+static void
+_setup_window1 ()
+{
+  if (gftp_protocols[GFTP_LOCAL_NUM].init (window1.request) == 0)
+    {
+      gftp_setup_startup_directory (window1.request,
+                                    "local_startup_directory");
+      gftp_connect (window1.request);
+      ftp_list_files (&window1);
+    }
+}
+
+
+static void
+_setup_window2 (int argc, char **argv)
+{
+  intptr_t connect_to_remote_on_startup;
+
+  gftp_lookup_request_option (window2.request, "connect_to_remote_on_startup",
+                              &connect_to_remote_on_startup);
+
+  if (argc == 2 && strncmp (argv[1], "--", 2) != 0)
+    {
+      if (gftp_parse_url (window2.request, argv[1]) == 0)
+        ftp_connect (&window2, window2.request);
+      else
+        gftp_usage ();
+    }
+  else if (connect_to_remote_on_startup)
+    {
+      if (gftp_protocols[_get_selected_protocol ()].init (current_wdata->request) == 0)
+        {
+          gftp_setup_startup_directory (window2.request,
+                                        "remote_startup_directory");
+          gftp_connect (window2.request);
+          ftp_list_files (&window2);
+        }
+    }
+  else
+    {
+      /* On the remote window, even though we aren't connected, draw the sort
+         icon on that side */
+      sortrows (GTK_CLIST (window2.listbox), -1, &window2);
+    }
+}
+
+
 int
 main (int argc, char **argv)
 {
@@ -1284,7 +1375,7 @@ main (int argc, char **argv)
 #if GTK_MAJOR_VERSION > 1
   gdk_threads_init();
 #endif
-
+  GDK_THREADS_ENTER ();
   main_thread_id = pthread_self ();
   gtk_set_locale ();
   gtk_init (&argc, &argv);
@@ -1320,20 +1411,10 @@ main (int argc, char **argv)
   gftpui_common_about (ftp_log, NULL);
 
   gtk_timeout_add (1000, update_downloads, NULL);
-  if (gftp_protocols[GFTP_LOCAL_NUM].init (window1.request) == 0)
-    {
-      gftp_setup_startup_directory (window1.request);
-      gftp_connect (window1.request);
-      ftp_list_files (&window1);
-    }
 
-  /* On the remote window, even though we aren't connected, draw the sort
-     icon on that side */
-  sortrows (GTK_CLIST (window2.listbox), -1, &window2);
+  _setup_window1 ();
+  _setup_window2 (argc, argv);
 
-  init_gftp (argc, argv, window);
-
-  GDK_THREADS_ENTER ();
   gtk_main ();
   GDK_THREADS_LEAVE ();
 
