@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  misc-gtk.c - misc stuff for the gtk+ 1.2 port of gftp                    */
-/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp-gtk.h"
-static const char cvsid[] = "$Id: misc-gtk.c,v 1.56 2005/01/25 02:34:19 masneyb Exp $";
+static const char cvsid[] = "$Id: misc-gtk.c 924 2007-04-26 23:47:34Z masneyb $";
 
 static GtkWidget * statuswid;
 
@@ -54,6 +54,7 @@ ftp_log (gftp_logging_level level, gftp_request * request,
   GtkTextIter iter, iter2;
   const char *descr;
   char *utf8_str;
+  size_t destlen;
 #endif
 
   va_start (argp, string);
@@ -70,7 +71,7 @@ ftp_log (gftp_logging_level level, gftp_request * request,
   va_end (argp);
 
 #if GTK_MAJOR_VERSION > 1
-  if ((utf8_str = gftp_string_to_utf8 (request, logstr)) != NULL)
+  if ((utf8_str = gftp_string_to_utf8 (request, logstr, &destlen)) != NULL)
     {
       if (free_logstr)
         g_free (logstr);
@@ -209,9 +210,8 @@ void
 update_window_info (void)
 {
   char *tempstr, empty[] = "";
+  unsigned int port, i, j;
   GtkWidget * tempwid;
-  unsigned int port;
-  int i;
 
   if (current_wdata->request != NULL)
     {
@@ -240,13 +240,17 @@ update_window_info (void)
           else
             gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (portedit)->entry), "");
     
-          for (i=0; gftp_protocols[i].init != NULL; i++)
+          for (i=0, j=0; gftp_protocols[i].init != NULL; i++)
             {
+              if (!gftp_protocols[i].shown) 
+                continue;
+
               if (current_wdata->request->init == gftp_protocols[i].init)
                 {
                   gtk_option_menu_set_history (GTK_OPTION_MENU (optionmenu), i);
                   break;
                 }
+              j++;
             }
 
           gtk_tooltips_set_tip (GTK_TOOLTIPS(openurl_tooltip), openurl_btn,
@@ -260,7 +264,7 @@ update_window_info (void)
   update_window (&window1);
   update_window (&window2);
 
-  tempwid = gtk_item_factory_get_widget (factory, menus[tools_start+2].path);
+  tempwid = gtk_item_factory_get_widget (factory, "/Tools/Compare Windows");
   gtk_widget_set_sensitive (tempwid, GFTP_IS_CONNECTED (window1.request) 
 			    && GFTP_IS_CONNECTED (window2.request));
 }
@@ -295,7 +299,7 @@ set_menu_sensitive (gftp_window_data * wdata, char *path, int sensitive)
 void
 update_window (gftp_window_data * wdata)
 {
-  char *dir, *tempstr, *temp1str, *fspec, *utf8_directory;
+  char *tempstr, *hostname, *fspec;
   int connected, start;
 
   connected = GFTP_IS_CONNECTED (wdata->request);
@@ -303,33 +307,22 @@ update_window (gftp_window_data * wdata)
     {
       fspec = wdata->show_selected ? "Selected" : strcmp (wdata->filespec, "*") == 0 ?  _("All Files") : wdata->filespec;
 
-      if ((temp1str = wdata->request->hostname) == NULL ||
+      if (wdata->request->hostname == NULL ||
           wdata->request->protonum == GFTP_LOCAL_NUM)
-	temp1str = "";
-      tempstr = g_strconcat (temp1str, *temp1str == '\0' ? "[" : " [",
-		     gftp_protocols[wdata->request->protonum].name,
-		     wdata->request->cached ? _("] (Cached) [") : "] [",
-                     fspec, "]", current_wdata == wdata ? "*" : "", NULL);
+        hostname = "";
+      else
+        hostname = wdata->request->hostname;
+
+      tempstr = g_strconcat (hostname, *hostname == '\0' ? "[" : " [",
+                             gftp_protocols[wdata->request->protonum].name,
+                             wdata->request->cached ? _("] (Cached) [") : "] [",
+                             fspec, "]", current_wdata == wdata ? "*" : "", NULL);
       gtk_label_set (GTK_LABEL (wdata->hoststxt), tempstr);
       g_free (tempstr);
 
-      utf8_directory = NULL;
-      if ((dir = wdata->request->directory) == NULL)
-        temp1str = "";
-      else
-        {
-          utf8_directory = gftp_string_to_utf8 (wdata->request, 
-                                                wdata->request->directory);
-          if (utf8_directory != NULL)
-            temp1str = utf8_directory;
-          else
-            temp1str = dir;
-        }
-
-      gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (wdata->combo)->entry),temp1str);
-
-      if (utf8_directory != NULL)
-        g_free (utf8_directory);
+      if (wdata->request->directory != NULL)
+        gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (wdata->combo)->entry),
+                            wdata->request->directory);
     }
   else if (wdata->hoststxt != NULL)
     {
@@ -337,7 +330,6 @@ update_window (gftp_window_data * wdata)
                              current_wdata == wdata ? "*" : "", NULL);
       gtk_label_set (GTK_LABEL (wdata->hoststxt), tempstr);
       g_free (tempstr);
-      gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (wdata->combo)->entry), "");
     }
 
   if (wdata == &window1)
@@ -499,8 +491,9 @@ gftp_get_pixmap (GtkWidget * widget, char *filename, GdkPixmap ** pix,
 
 
 int
-check_status (char *name, gftp_window_data *wdata, int check_other_stop,
-              int only_one, int at_least_one, int func)
+check_status (char *name, gftp_window_data *wdata,
+              unsigned int check_other_stop, unsigned int only_one,
+              unsigned int at_least_one, unsigned int func)
 {
   gftp_window_data * owdata;
 
@@ -594,12 +587,12 @@ item_factory_new (GtkType container_type, const char *path,
 
 
 void
-create_item_factory (GtkItemFactory * ifactory, guint n_entries,
-		     GtkItemFactoryEntry * entries, gpointer callback_data)
+create_item_factory (GtkItemFactory * ifactory, gint n_entries,
+                     GtkItemFactoryEntry * entries, gpointer callback_data)
 {
   const char *strip_prefix;
   size_t strip_prefix_len;
-  size_t i;
+  int i;
 
   strip_prefix = gtk_object_get_data (GTK_OBJECT (ifactory), "gftp-strip-prefix");
   if (strip_prefix)
@@ -676,7 +669,7 @@ check_reconnect (gftp_window_data *wdata)
 {
   return (wdata->request->cached && wdata->request->datafd < 0 && 
           !wdata->request->always_connected &&
-	  !ftp_connect (wdata, wdata->request, 0) ? -1 : 0);
+	  !ftp_connect (wdata, wdata->request) ? -1 : 0);
 }
 
 
@@ -687,25 +680,19 @@ add_file_listbox (gftp_window_data * wdata, gftp_file * fle)
   char *tempstr, *str, *pos, *attribs;
   gftp_config_list_vars * tmplistvar;
   gftp_file_extensions * tempext;
-  intptr_t show_hidden_files;
   GdkBitmap * bitmap;
   GList * templist;
   GdkPixmap * pix;
   int clist_num;
   size_t stlen;
 
-  gftp_lookup_request_option (wdata->request, "show_hidden_files", 
-                              &show_hidden_files);
-
   if (wdata->show_selected)
     {
       fle->shown = fle->was_sel;
       if (!fle->shown)
         return;
     }
-  else if ((!show_hidden_files && *fle->file == '.' && 
-            strcmp (fle->file, "..") != 0) ||
-           !gftp_match_filespec (fle->file, wdata->filespec))
+  else if (!gftp_match_filespec (wdata->request, fle->file, wdata->filespec))
     {
       fle->shown = 0;
       fle->was_sel = 0;
@@ -760,10 +747,7 @@ add_file_listbox (gftp_window_data * wdata, gftp_file * fle)
    
   gtk_clist_set_pixmap (GTK_CLIST (wdata->listbox), clist_num, 0, pix, bitmap);
 
-  if (fle->utf8_file)
-    gtk_clist_set_text (GTK_CLIST (wdata->listbox), clist_num, 1, 
-                        fle->utf8_file);
-  else if (fle->file)
+  if (fle->file != NULL && fle->filename_utf8_encoded)
     gtk_clist_set_text (GTK_CLIST (wdata->listbox), clist_num, 1, fle->file);
 
   if (GFTP_IS_SPECIAL_DEVICE (fle->st_mode))
@@ -925,7 +909,6 @@ MakeEditDialog (char *diagtxt, char *infotxt, char *deftext, int passwd_item,
 #if GTK_MAJOR_VERSION == 1
   dialog = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (dialog), diagtxt);
-  gtk_grab_add (dialog);
   gtk_container_border_width (GTK_CONTAINER
 			      (GTK_DIALOG (dialog)->action_area), 5);
   gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->action_area), 15);
@@ -954,16 +937,17 @@ MakeEditDialog (char *diagtxt, char *infotxt, char *deftext, int passwd_item,
     }
 
   dialog = gtk_dialog_new_with_buttons (_(diagtxt), NULL, 0,
-                                        yes_text,
-                                        GTK_RESPONSE_YES,
                                         GTK_STOCK_CANCEL,
                                         GTK_RESPONSE_NO,
+                                        yes_text,
+                                        GTK_RESPONSE_YES,
                                         NULL);
 #endif
   gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), 10);
   gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 5);
   gtk_window_set_wmclass (GTK_WINDOW(dialog), "edit", "gFTP");
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
+  gtk_grab_add (dialog);
   gtk_widget_realize (dialog);
 
   if (gftp_icon != NULL)
@@ -1073,7 +1057,6 @@ MakeYesNoDialog (char *diagtxt, char *infotxt,
 
 #if GTK_MAJOR_VERSION == 1
   dialog = gtk_dialog_new ();
-  gtk_grab_add (dialog);
   gtk_window_set_title (GTK_WINDOW (dialog), diagtxt);
   gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->action_area), 
                               5);
@@ -1081,16 +1064,17 @@ MakeYesNoDialog (char *diagtxt, char *infotxt,
   gtk_box_set_homogeneous (GTK_BOX (GTK_DIALOG (dialog)->action_area), TRUE);
 #else
   dialog = gtk_dialog_new_with_buttons (_(diagtxt), NULL, 0,
-                                        GTK_STOCK_YES,
-                                        GTK_RESPONSE_YES,
                                         GTK_STOCK_NO,
                                         GTK_RESPONSE_NO,
+                                        GTK_STOCK_YES,
+                                        GTK_RESPONSE_YES,
                                         NULL);
 #endif
   gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), 10);
   gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 5);
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
   gtk_window_set_wmclass (GTK_WINDOW(dialog), "yndiag", "gFTP");
+  gtk_grab_add (dialog);
   gtk_widget_realize (dialog);
 
   if (gftp_icon != NULL)
@@ -1285,7 +1269,7 @@ get_xpm_path (char *filename, int quit_on_err)
 
 	      printf (_("gFTP Error: Cannot find file %s in %s or %s\n"),
 		      filename, share_dir, BASE_CONF_DIR);
-	      exit (1);
+	      exit (EXIT_FAILURE);
 	    }
 	}
     }
