@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  menu-items.c - menu callbacks                                            */
-/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp-gtk.h"
-static const char cvsid[] = "$Id: options_dialog.c,v 1.25 2004/12/02 22:22:31 masneyb Exp $";
+static const char cvsid[] = "$Id: options_dialog.c 893 2007-03-13 01:52:50Z masneyb $";
 
 static GtkWidget * proxy_list, * new_proxy_domain, * network1,
                  * network2, * network3, * network4, * netmask1, * netmask2, 
@@ -83,7 +83,7 @@ _gen_input_widget (gftp_options_dialog_data * option_data, char *label, char *ti
 
 
 static void *
-_print_option_type_newtable (gftp_config_vars * cv, void *user_data, void *value)
+_print_option_type_newtable (void *user_data)
 {
   gftp_options_dialog_data * option_data;
 
@@ -232,7 +232,7 @@ _textcomboedt_toggle (GtkList * list, GtkWidget * child, gpointer data)
 #if GTK_MAJOR_VERSION > 1
   GtkTextIter iter, iter2;
   GtkTextBuffer * textbuf;
-  guint len;
+  gint len;
 #endif
 
   widdata = data;
@@ -289,7 +289,7 @@ _gftp_convert_to_newlines (char *str)
       *endpos = '\0';
 
       len += strlen (stpos) + 1;
-      ret = g_realloc (ret, len + 1);
+      ret = g_realloc (ret, (gulong) len + 1);
       strcat (ret, stpos);
       strcat (ret, "\n");
 
@@ -299,7 +299,7 @@ _gftp_convert_to_newlines (char *str)
   if (stpos != NULL && *stpos != '\0')
     {
       len += strlen (stpos);
-      ret = g_realloc (ret, len + 1);
+      ret = g_realloc (ret, (gulong) len + 1);
       strcat (ret, stpos);
     }
 
@@ -324,7 +324,7 @@ _gftp_convert_from_newlines (char *str)
       *endpos = '\0';
 
       len += strlen (stpos) + 2;
-      ret = g_realloc (ret, len + 1);
+      ret = g_realloc (ret, (gulong) len + 1);
       strcat (ret, stpos);
       strcat (ret, "%n");
 
@@ -334,7 +334,7 @@ _gftp_convert_from_newlines (char *str)
   if (stpos != NULL && *stpos != '\0')
     {
       len += strlen (stpos);
-      ret = g_realloc (ret, len + 1);
+      ret = g_realloc (ret, (gulong) len + 1);
       strcat (ret, stpos);
     }
 
@@ -572,7 +572,7 @@ _print_option_type_checkbox (gftp_config_vars * cv, void *user_data, void *value
   option_data = user_data;
 
   if (option_data->last_option != gftp_option_type_checkbox)
-    _print_option_type_newtable (NULL, user_data, NULL);
+    _print_option_type_newtable (user_data);
 
   if (option_data->tbl_col_num == 0)
     {
@@ -674,7 +674,7 @@ _print_option_type_notebook (gftp_config_vars * cv, void *user_data, void *value
   gtk_notebook_append_page (GTK_NOTEBOOK (option_data->notebook), 
                             option_data->box, tempwid);
 
-  _print_option_type_newtable (NULL, user_data, NULL);
+  _print_option_type_newtable (user_data);
   
   return (NULL);
 }
@@ -758,9 +758,6 @@ options_action (GtkWidget * widget, gint response, gpointer user_data)
 {
   switch (response)
     {
-      case GTK_RESPONSE_APPLY:
-        apply_changes (widget, NULL);
-        break;
       case GTK_RESPONSE_OK:
         apply_changes (widget, NULL);
         /* no break */
@@ -1002,7 +999,7 @@ add_proxy_host (GtkWidget * widget, gpointer data)
   gtk_widget_show (box);
   
 #if GTK_MAJOR_VERSION == 1
-  tempwid = gtk_label_new (_("Type:"))
+  tempwid = gtk_label_new (_("Type:"));
 #else
   tempwid = gtk_label_new_with_mnemonic (_("_Type:"));
 #endif
@@ -1373,8 +1370,6 @@ options_dialog (gpointer data)
   gtk_box_set_homogeneous (GTK_BOX (GTK_DIALOG (gftp_option_data->dialog)->action_area), TRUE);
 #else
   gftp_option_data->dialog = gtk_dialog_new_with_buttons (_("Options"), NULL, 0,
-                                        GTK_STOCK_APPLY,
-                                        GTK_RESPONSE_APPLY,
                                         GTK_STOCK_CANCEL,
                                         GTK_RESPONSE_CANCEL,
                                         GTK_STOCK_OK,
@@ -1520,7 +1515,7 @@ gftp_gtk_setup_bookmark_options (GtkWidget * notebook, gftp_bookmarks_var * bm)
 
 
 void
-gftp_gtk_save_bookmark_options (gftp_bookmarks_var * bm)
+gftp_gtk_save_bookmark_options ()
 {
   gftp_config_vars * cv;
   GList * templist;
