@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  transfer.c - functions to handle transfering files                       */
-/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp-gtk.h"
-static const char cvsid[] = "$Id: transfer.c,v 1.76 2005/01/25 02:34:19 masneyb Exp $";
+static const char cvsid[] = "$Id: transfer.c 949 2008-01-24 23:26:45Z masneyb $";
 
 static int num_transfers_in_progress = 0;
 
@@ -57,7 +57,7 @@ ftp_list_files (gftp_window_data * wdata)
 
 
 int
-ftp_connect (gftp_window_data * wdata, gftp_request * request, int getdir)
+ftp_connect (gftp_window_data * wdata, gftp_request * request)
 {
   if (wdata->request == request)
     gtk_label_set (GTK_LABEL (wdata->hoststxt), _("Connecting..."));
@@ -128,7 +128,7 @@ transfer_window_files (gftp_window_data * fromwdata, gftp_window_data * towdata)
       gftp_swap_socks (transfer->fromreq, fromwdata->request);
       gftp_swap_socks (transfer->toreq, towdata->request);
 
-      ret = gftp_gtk_get_subdirs (transfer, &fromwdata->tid);
+      ret = gftp_gtk_get_subdirs (transfer);
       if (ret < 0)
         disconnect = 1;
       else
@@ -219,7 +219,7 @@ _gftp_getdir_thread (gftpui_callback_data * cdata)
 
 
 int
-gftp_gtk_get_subdirs (gftp_transfer * transfer, pthread_t *tid)
+gftp_gtk_get_subdirs (gftp_transfer * transfer)
 {
   gftpui_callback_data * cdata; 
   long numfiles, numdirs;
@@ -254,14 +254,17 @@ gftp_gtk_get_subdirs (gftp_transfer * transfer, pthread_t *tid)
 
 
 static void
-remove_file (char *filename)
+remove_file (gftp_viewedit_data * ve_proc)
 {
-  if (unlink (filename) == 0)
+  if (ve_proc->remote_filename == NULL)
+    return;
+
+  if (unlink (ve_proc->filename) == 0)
     ftp_log (gftp_logging_misc, NULL, _("Successfully removed %s\n"),
-             filename);
+             ve_proc->filename);
   else
     ftp_log (gftp_logging_error, NULL,
-             _("Error: Could not remove file %s: %s\n"), filename,
+             _("Error: Could not remove file %s: %s\n"), ve_proc->filename,
              g_strerror (errno));
 }
 
@@ -287,7 +290,7 @@ free_edit_data (gftp_viewedit_data * ve_proc)
 static void
 dont_upload (gftp_viewedit_data * ve_proc, gftp_dialog_data * ddata)
 {
-  remove_file (ve_proc->filename);
+  remove_file (ve_proc);
   free_edit_data (ve_proc);
 }
 
@@ -319,83 +322,126 @@ do_upload (gftp_viewedit_data * ve_proc, gftp_dialog_data * ddata)
 }
 
 
+static int
+_check_viewedit_process_status (gftp_viewedit_data * ve_proc, int ret)
+{
+  int procret;
+
+  if (WIFEXITED (ret))
+    {
+      procret = WEXITSTATUS (ret);
+      if (procret != 0)
+        {
+          ftp_log (gftp_logging_error, NULL,
+                   _("Error: Child %d returned %d\n"), ve_proc->pid, procret);
+          if (ve_proc->view)
+            remove_file (ve_proc);
+
+          return (0);
+        }
+      else
+        {
+          ftp_log (gftp_logging_misc, NULL,
+                   _("Child %d returned successfully\n"), ve_proc->pid);
+          return (1);
+        }
+    }
+  else
+    {
+      ftp_log (gftp_logging_error, NULL,
+               _("Error: Child %d did not terminate properly\n"),
+               ve_proc->pid);
+      return (0);
+    }
+}
+
+
+static int
+_prompt_to_upload_edited_file (gftp_viewedit_data * ve_proc)
+{
+  struct stat st;
+  char *str;
+
+  if (stat (ve_proc->filename, &st) == -1)
+    {
+      ftp_log (gftp_logging_error, NULL,
+               _("Error: Cannot get information about file %s: %s\n"),
+	       ve_proc->filename, g_strerror (errno));
+      return (0);
+    }
+  else if (st.st_mtime == ve_proc->st.st_mtime)
+    {
+      ftp_log (gftp_logging_misc, NULL, _("File %s was not changed\n"),
+	       ve_proc->filename);
+      remove_file (ve_proc);
+      return (0);
+    }
+  else
+    {
+      memcpy (&ve_proc->st, &st, sizeof (ve_proc->st));
+      str = g_strdup_printf (_("File %s has changed.\nWould you like to upload it?"),
+                             ve_proc->remote_filename);
+
+      MakeYesNoDialog (_("Edit File"), str, do_upload, ve_proc, dont_upload,
+                       ve_proc);
+      g_free (str);
+      return (1);
+    }
+}
+
+
 static void
-check_done_process (void)
+do_check_done_process (pid_t pid, int ret)
 {
   gftp_viewedit_data * ve_proc;
   GList * curdata, *deldata;
-  int ret, procret;
-  struct stat st;
-  char *str;
+  int ok;
+
+  curdata = viewedit_processes;
+  while (curdata != NULL)
+    {
+      ve_proc = curdata->data;
+      if (ve_proc->pid != pid)
+        continue;
+
+      deldata = curdata;
+      curdata = curdata->next;
+
+      viewedit_processes = g_list_remove_link (viewedit_processes, 
+                                               deldata);
+
+      ok = _check_viewedit_process_status (ve_proc, ret);
+      if (!ve_proc->view && ve_proc->dontupload)
+        gftpui_refresh (ve_proc->fromwdata, 1);
+
+      if (ok && !ve_proc->view && !ve_proc->dontupload)
+	{
+	  /* We were editing the file. Upload it */
+          if (_prompt_to_upload_edited_file (ve_proc))
+            break; /* Don't free the ve_proc structure */
+        }
+      else if (ve_proc->view && ve_proc->rm)
+        {
+	  /* After viewing the file delete the tmp file */
+	  remove_file (ve_proc);
+	}
+
+      free_edit_data (ve_proc);
+      break;
+    }
+}
+
+
+static void
+check_done_process (void)
+{
   pid_t pid;
+  int ret;
 
   gftpui_common_child_process_done = 0;
   while ((pid = waitpid (-1, &ret, WNOHANG)) > 0)
     {
-      curdata = viewedit_processes;
-      while (curdata != NULL)
-        {
-	  ve_proc = curdata->data;
-          deldata = curdata;
-          curdata = curdata->next;
-	  if (ve_proc->pid == pid)
-	    {
-	      viewedit_processes = g_list_remove_link (viewedit_processes, 
-                                                       deldata);
-              if (WIFEXITED (ret))
-                {
-                  procret = WEXITSTATUS (ret);
-                  if (procret != 0)
-                    {
-                      ftp_log (gftp_logging_error, NULL,
-                               _("Error: Child %d returned %d\n"), pid, procret);
-                      remove_file (ve_proc->filename);
-                      continue;
-                    }
-                  else
-                    ftp_log (gftp_logging_misc, NULL,
-                             _("Child %d returned successfully\n"), pid);
-                }
-              else
-                ftp_log (gftp_logging_error, NULL,
-                         _("Error: Child %d did not terminate properly\n"),
-                         pid);
-
-	      if (!ve_proc->view && !ve_proc->dontupload)
-		{
-		  /* We was editing the file. Upload it */
-		  ret = stat (ve_proc->filename, &st);
-
-                  if (ret == -1)
-		    ftp_log (gftp_logging_error, NULL,
-		         _("Error: Cannot get information about file %s: %s\n"),
-			 ve_proc->filename, g_strerror (errno));
-		  else if (st.st_mtime == ve_proc->st.st_mtime)
-                    {
-		      ftp_log (gftp_logging_misc, NULL,
-		  	       _("File %s was not changed\n"),
-			       ve_proc->filename);
-                      remove_file (ve_proc->filename);
-                    }
-		  else
-		    {
-		      memcpy (&ve_proc->st, &st, sizeof (ve_proc->st));
-		      str = g_strdup_printf (
-			_("File %s has changed.\nWould you like to upload it?"),
-                        ve_proc->remote_filename);
-
-		      MakeYesNoDialog (_("Edit File"), str, 
-                                       do_upload, ve_proc, 
-                                       dont_upload, ve_proc);
-		      g_free (str);
-		      continue;
-		    }
-		}
-
-              free_edit_data (ve_proc);
-	      continue;
-	    }
-	}
+      do_check_done_process (pid, ret);
     }
 }
 
@@ -403,7 +449,6 @@ check_done_process (void)
 static void
 on_next_transfer (gftp_transfer * tdata)
 {
-  int fd;
   intptr_t refresh_files;
   gftp_file * tempfle;
 
@@ -412,33 +457,12 @@ on_next_transfer (gftp_transfer * tdata)
     {
       tempfle = tdata->updfle->data;
 
-      if (tempfle->is_fd)
-        fd = tempfle->fd;
-      else
-        fd = 0;
-
-      if (tempfle->done_view)
+      if (tempfle->done_view || tempfle->done_edit)
         {
           if (tempfle->transfer_action != GFTP_TRANS_ACTION_SKIP)
-            view_file (tempfle->destfile, fd, 1, tempfle->done_rm, 1, 0,
-                       tempfle->file, NULL);
-
-          if (tempfle->is_fd)
             {
-              close (tempfle->fd);
-              tempfle->fd = -1;
-            }
-        }
-      else if (tempfle->done_edit)
-        {
-          if (tempfle->transfer_action != GFTP_TRANS_ACTION_SKIP)
-	    view_file (tempfle->destfile, fd, 0, tempfle->done_rm, 1, 0,
-                       tempfle->file, NULL);
-
-          if (tempfle->is_fd)
-            {
-              close (tempfle->fd);
-              tempfle->fd = -1;
+              view_file (tempfle->destfile, 0, tempfle->done_view,
+                         tempfle->done_rm, 1, 0, tempfle->file, NULL);
             }
         }
       else if (tempfle->done_rm)
@@ -475,22 +499,7 @@ cancel_get_trans_password (gftp_transfer * tdata, gftp_dialog_data * ddata)
   if (tdata->fromreq->stopable == 0)
     return;
 
-  g_static_mutex_lock (&tdata->structmutex);
-  if (tdata->started)
-    {
-      tdata->cancel = 1;
-      tdata->fromreq->cancel = 1;
-      tdata->toreq->cancel = 1;
-    }
-  else
-    tdata->done = 1;
-
-  tdata->fromreq->stopable = 0;
-  tdata->toreq->stopable = 0;
-  g_static_mutex_unlock (&tdata->structmutex);
-
-  ftp_log (gftp_logging_misc, NULL, _("Stopping the transfer of %s\n"),
-	   ((gftp_file *) tdata->curfle->data)->file);
+  gftpui_common_cancel_file_transfer (tdata);
 }
 
 
@@ -515,7 +524,7 @@ show_transfer (gftp_transfer * tdata)
                                        opendir_pixmap, opendir_bitmap, 
                                        FALSE, 
                                        tdata->numdirs + tdata->numfiles < 50);
-  transdata = g_malloc (sizeof (*transdata));
+  transdata = g_malloc0 (sizeof (*transdata));
   transdata->transfer = tdata;
   transdata->curfle = NULL;
   gtk_ctree_node_set_row_data (GTK_CTREE (dlwdw), tdata->user_data, transdata);
@@ -540,7 +549,7 @@ show_transfer (gftp_transfer * tdata)
                                              tdata->user_data, 
                                              NULL, text, 5, NULL, NULL, NULL, 
                                              NULL, FALSE, FALSE);
-      transdata = g_malloc (sizeof (*transdata));
+      transdata = g_malloc0 (sizeof (*transdata));
       transdata->transfer = tdata;
       transdata->curfle = templist;
       gtk_ctree_node_set_row_data (GTK_CTREE (dlwdw), tempfle->user_data, 
@@ -609,7 +618,8 @@ transfer_done (GList * node)
       num_transfers_in_progress--;
     }
 
-  if (!tdata->show && tdata->started)
+  if ((!tdata->show && tdata->started) ||
+      (tdata->done && !tdata->started))
     {
       transdata = gtk_ctree_node_get_row_data (GTK_CTREE (dlwdw), 
                                                tdata->user_data);
@@ -653,32 +663,99 @@ _gftpui_transfer_files (void *data)
 static void
 create_transfer (gftp_transfer * tdata)
 {
-  pthread_t tid;
+  if (tdata->fromreq->stopable)
+    return;
 
-  if (!tdata->fromreq->stopable)
+  if (GFTP_IS_SAME_HOST_START_TRANS ((gftp_window_data *) tdata->fromwdata,
+                                     tdata->fromreq))
     {
-      if (GFTP_IS_SAME_HOST_START_TRANS ((gftp_window_data *) tdata->fromwdata,
-                                         tdata->fromreq))
+      gftp_swap_socks (tdata->fromreq, 
+                       ((gftp_window_data *) tdata->fromwdata)->request);
+      update_window (tdata->fromwdata);
+    }
+
+  if (GFTP_IS_SAME_HOST_START_TRANS ((gftp_window_data *) tdata->towdata,
+                                     tdata->toreq))
+    {
+      gftp_swap_socks (tdata->toreq, 
+                       ((gftp_window_data *) tdata->towdata)->request);
+      update_window (tdata->towdata);
+    }
+
+  num_transfers_in_progress++;
+  tdata->started = 1;
+  tdata->stalled = 1;
+  gtk_ctree_node_set_text (GTK_CTREE (dlwdw), tdata->user_data, 1,
+                           _("Connecting..."));
+
+  if (tdata->thread_id == NULL)
+    tdata->thread_id = g_malloc0 (sizeof (pthread_t));
+
+  pthread_create (tdata->thread_id, NULL, _gftpui_transfer_files, tdata);
+}
+
+
+static void
+_setup_dlstr (gftp_transfer * tdata, gftp_file * fle, char *dlstr,
+              size_t dlstr_len)
+{
+  int hours, mins, secs, stalled, usesentdescr;
+  unsigned long remaining_secs, lkbs;
+  char gotstr[50], ofstr[50];
+  struct timeval tv;
+
+  stalled = 1;
+  gettimeofday (&tv, NULL);
+  usesentdescr = (tdata->fromreq->protonum == GFTP_LOCAL_NUM);
+
+  insert_commas (fle->size, ofstr, sizeof (ofstr));
+  insert_commas (tdata->curtrans + tdata->curresumed, gotstr, sizeof (gotstr));
+
+  if (tv.tv_sec - tdata->lasttime.tv_sec <= 5)
+    {
+      remaining_secs = (fle->size - tdata->curtrans - tdata->curresumed) / 1024;
+
+      lkbs = (unsigned long) tdata->kbs;
+      if (lkbs > 0)
+        remaining_secs /= lkbs;
+
+      hours = remaining_secs / 3600;
+      remaining_secs -= hours * 3600;
+      mins = remaining_secs / 60;
+      remaining_secs -= mins * 60;
+      secs = remaining_secs;
+
+      if (!(hours < 0 || mins < 0 || secs < 0))
         {
-          gftp_swap_socks (tdata->fromreq, 
-                           ((gftp_window_data *) tdata->fromwdata)->request);
-          update_window (tdata->fromwdata);
+          stalled = 0;
+          if (usesentdescr)
+            {
+              g_snprintf (dlstr, dlstr_len,
+                          _("Sent %s of %s at %.2fKB/s, %02d:%02d:%02d est. time remaining"), gotstr, ofstr, tdata->kbs, hours, mins, secs);
+            }
+          else
+            {
+              g_snprintf (dlstr, dlstr_len,
+                          _("Recv %s of %s at %.2fKB/s, %02d:%02d:%02d est. time remaining"), gotstr, ofstr, tdata->kbs, hours, mins, secs);
+            }
         }
+    }
 
-      if (GFTP_IS_SAME_HOST_START_TRANS ((gftp_window_data *) tdata->towdata,
-                                         tdata->toreq))
-        {
-          gftp_swap_socks (tdata->toreq, 
-                           ((gftp_window_data *) tdata->towdata)->request);
-	  update_window (tdata->towdata);
-	}
-
-      num_transfers_in_progress++;
-      tdata->started = 1;
+  if (stalled)
+    {
       tdata->stalled = 1;
-      gtk_ctree_node_set_text (GTK_CTREE (dlwdw), tdata->user_data, 1,
-			       _("Connecting..."));
-      pthread_create (&tid, NULL, _gftpui_transfer_files, tdata);
+      if (usesentdescr)
+        {
+          g_snprintf (dlstr, dlstr_len,
+                      _("Sent %s of %s, transfer stalled, unknown time remaining"),
+                      gotstr, ofstr);
+        }
+      else
+        {
+          g_snprintf (dlstr, dlstr_len,
+                      _("Recv %s of %s, transfer stalled, unknown time remaining"),
+                      gotstr, ofstr);
+        }
     }
 }
 
@@ -686,18 +763,15 @@ create_transfer (gftp_transfer * tdata)
 static void
 update_file_status (gftp_transfer * tdata)
 {
-  char totstr[150], dlstr[150], winstr[150], gotstr[50], ofstr[50];
+  char totstr[150], winstr[150], dlstr[150];
   unsigned long remaining_secs, lkbs;
-  int hours, mins, secs, pcent, st;
+  int hours, mins, secs, pcent;
   intptr_t show_trans_in_title;
   gftp_file * tempfle;
-  struct timeval tv;
   
   g_static_mutex_lock (&tdata->statmutex);
   tempfle = tdata->curfle->data;
 
-  gettimeofday (&tv, NULL);
-
   remaining_secs = (tdata->total_bytes - tdata->trans_bytes - tdata->resumed_bytes) / 1024;
 
   lkbs = (unsigned long) tdata->kbs;
@@ -733,43 +807,7 @@ update_file_status (gftp_transfer * tdata)
 
   *dlstr = '\0';
   if (!tdata->stalled)
-    {
-      insert_commas (tdata->curtrans + tdata->curresumed, gotstr, sizeof (gotstr));
-      insert_commas (tempfle->size, ofstr, sizeof (ofstr));
-      st = 1;
-      if (tv.tv_sec - tdata->lasttime.tv_sec <= 5)
-        {
-          if (tdata->curfle->next != NULL)
-            {
-              remaining_secs = (tempfle->size - tdata->curtrans - tdata->curresumed) / 1024;
-
-              lkbs = (unsigned long) tdata->kbs;
-              if (lkbs > 0)
-                remaining_secs /= lkbs;
-
-              hours = remaining_secs / 3600;
-              remaining_secs -= hours * 3600;
-              mins = remaining_secs / 60;
-              remaining_secs -= mins * 60;
-              secs = remaining_secs;
-            }
-
-          if (!(hours < 0 || mins < 0 || secs < 0))
-            {
-              g_snprintf (dlstr, sizeof (dlstr),
-                          _("Recv %s of %s at %.2fKB/s, %02d:%02d:%02d est. time remaining"), gotstr, ofstr, tdata->kbs, hours, mins, secs);
-              st = 0;
-            }
-        }
-
-      if (st)
-        {
-          tdata->stalled = 1;
-          g_snprintf (dlstr, sizeof (dlstr),
-	  	  _("Recv %s of %s, transfer stalled, unknown time remaining"),
-		  gotstr, ofstr);
-        }
-    }
+    _setup_dlstr (tdata, tempfle, dlstr, sizeof (dlstr));
 
   g_static_mutex_unlock (&tdata->statmutex);
 
@@ -795,7 +833,7 @@ update_window_transfer_bytes (gftp_window_data * wdata)
 
   if (wdata->request->gotbytes == -1)
     {
-      update_window_info ();
+      update_window (wdata);
       wdata->request->gotbytes = 0;
     }
   else
@@ -905,23 +943,10 @@ stop_transfer (gpointer data)
 	      _("There are no file transfers selected\n"));
       return;
     }
+
   node = GTK_CLIST (dlwdw)->selection->data;
   transdata = gtk_ctree_node_get_row_data (GTK_CTREE (dlwdw), node);
-
-  g_static_mutex_lock (&transdata->transfer->structmutex);
-  if (transdata->transfer->started)
-    {
-      transdata->transfer->cancel = 1;
-      transdata->transfer->fromreq->cancel = 1;
-      transdata->transfer->toreq->cancel = 1;
-      transdata->transfer->skip_file = 0;
-    }
-  else
-    transdata->transfer->done = 1;
-  g_static_mutex_unlock (&transdata->transfer->structmutex);
-
-  ftp_log (gftp_logging_misc, NULL, _("Stopping the transfer on host %s\n"),
-	   transdata->transfer->fromreq->hostname);
+  gftpui_common_cancel_file_transfer (transdata->transfer);
 }
 
 
@@ -930,8 +955,6 @@ skip_transfer (gpointer data)
 {
   gftpui_common_curtrans_data * transdata;
   GtkCTreeNode * node;
-  gftp_file * curfle;
-  char *file;
 
   if (GTK_CLIST (dlwdw)->selection == NULL)
     {
@@ -939,30 +962,12 @@ skip_transfer (gpointer data)
 	      _("There are no file transfers selected\n"));
       return;
     }
+
   node = GTK_CLIST (dlwdw)->selection->data;
   transdata = gtk_ctree_node_get_row_data (GTK_CTREE (dlwdw), node);
 
-  g_static_mutex_lock (&transdata->transfer->structmutex);
-  if (transdata->transfer->curfle != NULL)
-    {
-      curfle = transdata->transfer->curfle->data;
-      if (transdata->transfer->started)
-        {
-          transdata->transfer->cancel = 1;
-          transdata->transfer->fromreq->cancel = 1;
-          transdata->transfer->toreq->cancel = 1;
-          transdata->transfer->skip_file = 1;
-        }
-
-      curfle->transfer_action = GFTP_TRANS_ACTION_SKIP;
-      file = curfle->file;
-    }
-  else
-    file = NULL;
-  g_static_mutex_unlock (&transdata->transfer->structmutex);
-
-  ftp_log (gftp_logging_misc, NULL, _("Skipping file %s on host %s\n"), 
-           file, transdata->transfer->fromreq->hostname);
+  gftpui_common_skip_file_transfer (transdata->transfer,
+                                    transdata->transfer->curfle->data);
 }
 
 
@@ -983,39 +988,14 @@ remove_file_transfer (gpointer data)
   node = GTK_CLIST (dlwdw)->selection->data;
   transdata = gtk_ctree_node_get_row_data (GTK_CTREE (dlwdw), node);
 
-
   if (transdata->curfle == NULL || transdata->curfle->data == NULL)
     return;
 
   curfle = transdata->curfle->data;
+  gftpui_common_skip_file_transfer (transdata->transfer, curfle);
 
-  if (curfle->transfer_action & GFTP_TRANS_ACTION_SKIP)
-    return;
-
-  g_static_mutex_lock (&transdata->transfer->structmutex);
-
-  curfle->transfer_action = GFTP_TRANS_ACTION_SKIP;
-
-  if (transdata->transfer->started &&
-      transdata->curfle == transdata->transfer->curfle)
-    {
-      transdata->transfer->cancel = 1;
-      transdata->transfer->fromreq->cancel = 1;
-      transdata->transfer->toreq->cancel = 1;
-      transdata->transfer->skip_file = 1;
-    }
-  else if (transdata->curfle != transdata->transfer->curfle &&
-           !curfle->transfer_done)
-    {
-      gtk_ctree_node_set_text (GTK_CTREE (dlwdw), curfle->user_data, 1,
-                               _("Skipped"));
-      transdata->transfer->total_bytes -= curfle->size;
-    }
-
-  g_static_mutex_unlock (&transdata->transfer->structmutex);
-
-  ftp_log (gftp_logging_misc, NULL, _("Skipping file %s on host %s\n"),
-           curfle->file, transdata->transfer->fromreq->hostname);
+  gtk_ctree_node_set_text (GTK_CTREE (dlwdw), curfle->user_data, 1,
+                           _("Skipped"));
 }
 
 
