@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  view_dialog.c - view dialog box and ftp routines                         */
-/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,39 +18,44 @@
 /*****************************************************************************/
 
 #include "gftp-gtk.h"
-static const char cvsid[] = "$Id: view_dialog.c,v 1.17 2004/03/21 13:22:41 masneyb Exp $";
+static const char cvsid[] = "$Id: view_dialog.c 913 2007-03-28 02:31:06Z masneyb $";
 
 static gftp_file * curfle;
 
-void
-view_dialog (gpointer data)
+static void
+do_view_or_edit_file (gftp_window_data * fromwdata, int is_view)
 {
   GList * templist, * filelist, * newfile;
-  gftp_window_data * fromwdata, * towdata;
+  gftp_window_data * towdata;
   gftp_file * new_fle;
   char *suffix;
   int num;
 
-  fromwdata = data;
-  towdata = fromwdata == &window1 ? &window2 : &window1;
-  if (!check_status (_("View"), fromwdata, 0, 1, 1, 1))
+  if (!check_status (is_view ? _("View") : _("Edit"), fromwdata, 0, 1, 1, 1))
     return;
 
+  towdata = fromwdata == &window1 ? &window2 : &window1;
+
   templist = GTK_CLIST (fromwdata->listbox)->selection;
   num = 0;
   filelist = fromwdata->files;
   templist = get_next_selection (templist, &filelist, &num);
   curfle = filelist->data;
 
-  if (curfle->isdir)
+  if (S_ISDIR (curfle->st_mode))
     {
-      ftp_log (gftp_logging_misc, NULL,
-	       _("View: %s is a directory. Cannot view it.\n"), curfle->file);
+      if (is_view)
+        ftp_log (gftp_logging_error, NULL,
+                 _("View: %s is a directory. Cannot view it.\n"), curfle->file);
+      else
+        ftp_log (gftp_logging_error, NULL,
+                 _("Edit: %s is a directory. Cannot edit it.\n"), curfle->file);
+
       return;
     }
 
   if (strcmp (gftp_protocols[fromwdata->request->protonum].name, "Local") == 0)
-    view_file (curfle->file, 0, 1, 0, 1, 1, NULL, fromwdata);
+    view_file (curfle->file, 0, is_view, 0, 1, 1, NULL, fromwdata);
   else
     {
       new_fle = copy_fdata (curfle);
@@ -72,18 +77,23 @@ view_dialog (gpointer data)
 		
       if (new_fle->fd < 0)
         {
-          ftp_log (gftp_logging_misc, NULL, 
+          ftp_log (gftp_logging_error, NULL, 
                    _("Error: Cannot open %s for writing: %s\n"),  
                    new_fle->destfile, g_strerror (errno));
-          free_fdata (new_fle);
+          gftp_file_destroy (new_fle, 1);
           return;
         }
 
       fchmod (new_fle->fd, S_IRUSR | S_IWUSR);
 
-      new_fle->is_fd = 1;
-      new_fle->done_view = 1;
-      new_fle->done_rm = 1;
+      if (is_view)
+        {
+          new_fle->done_view = 1;
+          new_fle->done_rm = 1;
+        }
+      else
+        new_fle->done_edit = 1;
+
       newfile = g_list_append (NULL, new_fle); 
       gftpui_common_add_file_transfer (fromwdata->request, towdata->request,
                                        fromwdata, towdata, newfile);
@@ -91,88 +101,37 @@ view_dialog (gpointer data)
 }
 
 
+void
+view_dialog (gpointer data)
+{
+  do_view_or_edit_file (data, 1);
+}
+
+
 void
 edit_dialog (gpointer data)
 {
-  gftp_window_data * fromwdata, * towdata;
-  GList * templist, * filelist, * newfile;
-  gftp_file * new_fle;
-  char *edit_program, *suffix;
-  int num;
-
-  fromwdata = data;
-  towdata = fromwdata == &window1 ? &window2 : &window1;
-  if (!check_status (_("Edit"), fromwdata, 0, 1, 1, 1))
-    return;
+  gftp_window_data * fromwdata = data;
+  char *edit_program;
 
   gftp_lookup_request_option (fromwdata->request, "edit_program", 
                               &edit_program);
 
   if (*edit_program == '\0')
     {
-      ftp_log (gftp_logging_misc, NULL,
+      ftp_log (gftp_logging_error, NULL,
 	       _("Edit: You must specify an editor in the options dialog\n"));
       return;
     }
 
-  templist = GTK_CLIST (fromwdata->listbox)->selection;
-  num = 0;
-  filelist = fromwdata->files;
-  templist = get_next_selection (templist, &filelist, &num);
-  curfle = filelist->data;
-
-  if (curfle->isdir)
-    {
-      ftp_log (gftp_logging_misc, NULL,
-	       _("Edit: %s is a directory. Cannot edit it.\n"), curfle->file);
-      return;
-    }
-
-  if (strcmp (gftp_protocols[fromwdata->request->protonum].name, "Local") == 0)
-    view_file (curfle->file, 0, 0, 0, 1, 1, NULL, fromwdata);
-  else
-    {
-      new_fle = copy_fdata (curfle);
-      if (new_fle->destfile)
-        g_free (new_fle->destfile);
-
-      if ((suffix = strrchr (curfle->file, '.')) != NULL)
-        {
-          new_fle->destfile = g_strconcat (g_get_tmp_dir (),
-                                           "/gftp-view.XXXXXX", suffix, NULL);
-          new_fle->fd = mkstemps (new_fle->destfile, strlen (suffix));
-	}
-      else
-        {
-	  new_fle->destfile = g_strconcat (g_get_tmp_dir (),
-                                           "/gftp-view.XXXXXX", NULL);		
-          new_fle->fd = mkstemps (new_fle->destfile, 0);
-	}
-
-      if (new_fle->fd < 0)
-        {
-          ftp_log (gftp_logging_misc, NULL, 
-                   _("Error: Cannot open %s for writing: %s\n"),
-                   new_fle->destfile, g_strerror (errno));
-          free_fdata (new_fle);
-          return;
-        }
-
-      fchmod (new_fle->fd, S_IRUSR | S_IWUSR);
-
-      new_fle->is_fd = 1;
-      new_fle->done_edit = 1;
-      newfile = g_list_append (NULL, new_fle); 
-      gftpui_common_add_file_transfer (fromwdata->request, towdata->request,
-                                       fromwdata, towdata, newfile);
-    }
+  do_view_or_edit_file (data, 0);
 }
 
 
 static gftp_viewedit_data *
 fork_process (char *proc, char *filename, int fd, char *remote_filename, 
-              int viewedit, int del_file, int dontupload, 
-              gftp_window_data * wdata)
+              unsigned int viewedit, unsigned int del_file,
+              unsigned int dontupload, gftp_window_data * wdata)
 {
   gftp_viewedit_data * newproc;
   char *pos, *endpos, **argv;
@@ -186,14 +145,22 @@ fork_process (char *proc, char *filename, int fd, char *remote_filename,
     {
       *endpos = '\0';
       n++;
-      argv = g_realloc (argv, n * sizeof (char *));
+      argv = g_realloc (argv, (gulong) n * sizeof (char *));
       argv[n - 1] = g_strdup (pos);
       *endpos = ' ';
       pos = endpos + 1;
     }
-  argv = g_realloc (argv, (n + 3) * sizeof (char *));
+  argv = g_realloc (argv, (gulong) (n + 3) * sizeof (char *));
   argv[n] = g_strdup (pos);
-  argv[n + 1] = g_strdup (filename);
+
+  if (wdata != NULL)
+    {
+      if ((argv[n + 1] = gftp_filename_from_utf8 (wdata->request, filename, NULL)) == NULL)
+        argv[n + 1] = g_strdup (filename);
+    }
+  else
+    argv[n + 1] = g_strdup (filename);
+
   argv[n + 2] = NULL;
 
   newproc = NULL;
@@ -239,8 +206,9 @@ fork_process (char *proc, char *filename, int fd, char *remote_filename,
 
 
 void
-view_file (char *filename, int fd, int viewedit, int del_file, int start_pos,
-	   int dontupload, char *remote_filename, gftp_window_data * wdata)
+view_file (char *filename, int fd, unsigned int viewedit, unsigned int del_file,
+           unsigned int start_pos, unsigned int dontupload,
+           char *remote_filename, gftp_window_data * wdata)
 {
   GtkWidget * dialog, * view, * table, * tempwid;
   char buf[8192], *view_program, *edit_program;
@@ -248,13 +216,14 @@ view_file (char *filename, int fd, int viewedit, int del_file, int start_pos,
   gftp_file_extensions * tempext;
   gftp_viewedit_data * newproc;
   GtkAdjustment * vadj;
-  int stlen, doclose;
   GList * templist;
+  size_t stlen;
+  int doclose;
   ssize_t n;
+  char * non_utf8;
 #if GTK_MAJOR_VERSION > 1
   GtkTextBuffer * textbuf;
   GtkTextIter iter;
-  guint len;
 #endif
 
   doclose = 1;
@@ -280,11 +249,14 @@ view_file (char *filename, int fd, int viewedit, int del_file, int start_pos,
     {
       gftp_lookup_request_option (wdata->request, "view_program", &view_program);
       gftp_lookup_request_option (wdata->request, "edit_program", &edit_program);
+      if ((non_utf8 = gftp_filename_from_utf8 (wdata->request, filename, NULL)) == NULL) /* freeme later! */
+        non_utf8 = filename;
     }
   else
     {
       gftp_lookup_global_option ("view_program", &view_program);
       gftp_lookup_global_option ("edit_program", &edit_program);
+      non_utf8 = filename;
     }
 
   if (viewedit && *view_program != '\0')
@@ -292,6 +264,8 @@ view_file (char *filename, int fd, int viewedit, int del_file, int start_pos,
       /* Open the file with the default file viewer */
       fork_process (view_program, filename, fd, remote_filename, viewedit,
                     del_file, dontupload, wdata);
+      if (non_utf8 != filename && non_utf8)
+        g_free (non_utf8);
       return;
     }
   else if (!viewedit && *edit_program != '\0')
@@ -299,7 +273,9 @@ view_file (char *filename, int fd, int viewedit, int del_file, int start_pos,
       /* Open the file with the default file editor */
       newproc = fork_process (edit_program, filename, fd, remote_filename, 
                               viewedit, del_file, dontupload, wdata);
-      stat (filename, &newproc->st);
+      stat (non_utf8, &newproc->st);
+      if (non_utf8 != filename && non_utf8)
+        g_free (non_utf8);
       return;
     }
 
@@ -307,11 +283,13 @@ view_file (char *filename, int fd, int viewedit, int del_file, int start_pos,
 
   if (fd == 0)
     {
-      if ((fd = open (filename, O_RDONLY)) < 0)
+      if ((fd = open (non_utf8, O_RDONLY)) < 0)
         {
-          ftp_log (gftp_logging_misc, NULL, 
-                   _("View: Cannot open file %s: %s\n"), filename, 
+          ftp_log (gftp_logging_error, NULL, 
+                   _("View: Cannot open file %s: %s\n"), non_utf8, 
                    g_strerror (errno));
+          if (non_utf8 != filename && non_utf8)
+            g_free (non_utf8);
           return;
         }
       doclose = 1;
@@ -324,7 +302,7 @@ view_file (char *filename, int fd, int viewedit, int del_file, int start_pos,
 
   if (del_file)
     {
-      if (unlink (filename) == 0)
+      if (unlink (non_utf8) == 0)
         ftp_log (gftp_logging_misc, NULL, _("Successfully removed %s\n"), 
                  filename);
       else
@@ -333,6 +311,9 @@ view_file (char *filename, int fd, int viewedit, int del_file, int start_pos,
                  g_strerror (errno));
     }
 
+  if (non_utf8 != filename && non_utf8)
+    g_free (non_utf8);
+
 #if GTK_MAJOR_VERSION == 1
   dialog = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (dialog), filename);
@@ -422,9 +403,8 @@ view_file (char *filename, int fd, int viewedit, int del_file, int start_pos,
       gtk_text_insert (GTK_TEXT (view), NULL, NULL, NULL, buf, -1);
 #else
       textbuf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
-      len = gtk_text_buffer_get_char_count (textbuf);
-      gtk_text_buffer_get_iter_at_offset (textbuf, &iter, len);
-      gtk_text_buffer_insert (textbuf, &iter, buf, n);
+      gtk_text_buffer_get_iter_at_offset (textbuf, &iter, -1);
+      gtk_text_buffer_insert (textbuf, &iter, buf, -1);
 #endif
     }
 
