@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  gftp-text.c - text port of gftp                                          */
-/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp-text.h"
-static const char cvsid[] = "$Id: gftp-text.c,v 1.45 2005/01/25 01:11:00 masneyb Exp $";
+static const char cvsid[] = "$Id: gftp-text.c 893 2007-03-13 01:52:50Z masneyb $";
 
 unsigned int
 gftp_text_get_win_size (void)
@@ -40,10 +40,11 @@ gftp_text_write_string (gftp_request * request, char *string)
 {
   gchar *stpos, *endpos, *locale_str, savechar;
   unsigned int sw;
+  size_t destlen;
 
   sw = gftp_text_get_win_size ();
 
-  locale_str = gftp_string_from_utf8 (request, string);
+  locale_str = gftp_string_from_utf8 (request, 1, string, &destlen);
   if (locale_str == NULL)
     stpos = string;
   else
@@ -92,7 +93,7 @@ static void
 gftp_text_log (gftp_logging_level level, gftp_request * request, 
                const char *string, ...)
 {
-  char tempstr[512], *utf8_str = NULL, *outstr;
+  char tempstr[512];
   va_list argp;
 
   g_return_if_fail (string != NULL);
@@ -117,19 +118,9 @@ gftp_text_log (gftp_logging_level level, gftp_request * request,
   g_vsnprintf (tempstr, sizeof (tempstr), string, argp);
   va_end (argp);
 
-#if GLIB_MAJOR_VERSION > 1
-  if (!g_utf8_validate (tempstr, -1, NULL))
-    utf8_str = gftp_string_to_utf8 (request, tempstr);
-#endif
-
-  if (utf8_str != NULL)
-    outstr = utf8_str;
-  else
-    outstr = tempstr;
-
   if (gftp_logfd != NULL && level != gftp_logging_misc_nolog)
     {
-      fwrite (outstr, 1, strlen (outstr), gftp_logfd);
+      fwrite (tempstr, 1, strlen (tempstr), gftp_logfd);
       if (ferror (gftp_logfd))
         {
           fclose (gftp_logfd);
@@ -140,24 +131,23 @@ gftp_text_log (gftp_logging_level level, gftp_request * request,
     }
 
   if (level == gftp_logging_misc_nolog)
-    printf ("%s", outstr);
+    printf ("%s", tempstr);
   else
-    gftp_text_write_string (request, outstr);
+    gftp_text_write_string (request, tempstr);
   
   printf ("%s", GFTPUI_COMMON_COLOR_DEFAULT);
-
-  if (utf8_str != NULL)
-    g_free (utf8_str);
 }
 
 
 char *
-gftp_text_ask_question (const char *question, int echo, char *buf, size_t size)
+gftp_text_ask_question (gftp_request * request, const char *question, int echo,
+                        char *buf, size_t size)
 {
   struct termios term, oldterm;
   gchar *locale_question;
   sigset_t sig, sigsave;
   char *pos, *termname;
+  size_t destlen;
   int singlechar;
   FILE *infd;
 
@@ -185,7 +175,7 @@ gftp_text_ask_question (const char *question, int echo, char *buf, size_t size)
   else
     infd = stdin;
 
-  locale_question = g_locale_from_utf8 (question, -1, NULL, NULL, NULL);
+  locale_question = gftp_string_from_utf8 (request, 1, question, &destlen);
   if (locale_question != NULL)
     {
       printf ("%s%s%s ", GFTPUI_COMMON_COLOR_BLUE, locale_question,
@@ -263,7 +253,8 @@ main (int argc, char **argv)
 
   if (gftp_protocols[GFTP_LOCAL_NUM].init (gftp_text_locreq) == 0)
     {
-      gftp_setup_startup_directory (gftp_text_locreq);
+      gftp_setup_startup_directory (gftp_text_locreq,
+                                    "local_startup_directory");
       gftp_connect (gftp_text_locreq);
     }
 
@@ -292,6 +283,9 @@ main (int argc, char **argv)
       gftpui_common_cmd_open (remuidata, gftp_text_remreq,
                               locuidata, gftp_text_locreq,
                               argv[1]);
+
+      gftp_setup_startup_directory (gftp_text_remreq,
+                                    "remote_startup_directory");
     }
 
 #if HAVE_LIBREADLINE
