@@ -17,7 +17,7 @@
 /*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
 /*****************************************************************************/
 
-/* $Id: gftp-text.h,v 1.11 2004/01/24 11:45:10 masneyb Exp $ */
+/* $Id: gftp-text.h 854 2006-11-03 03:07:44Z masneyb $ */
 
 #ifndef __GFTP_TEXT_H
 #define __GFTP_TEXT_H
@@ -30,9 +30,10 @@
 #include <readline/history.h>
 #endif
 
-int gftp_text_get_win_size 			( void );
+unsigned int gftp_text_get_win_size 		( void );
 
-char * gftp_text_ask_question 			( const char *question,
+char * gftp_text_ask_question 			( gftp_request * request,
+						  const char *question,
 						  int echo,
 						  char *buf,
 						  size_t size );
