@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  textui.c - Text UI related functions for gFTP                            */
-/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
+/*  Copyright (C) 1998-2007 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp-text.h"
-static const char cvsid[] = "$Id: textui.c,v 1.13 2004/07/28 02:47:24 masneyb Exp $";
+static const char cvsid[] = "$Id: textui.c 893 2007-03-13 01:52:50Z masneyb $";
 
 void
 gftpui_lookup_file_colors (gftp_file * fle, char **start_color,
@@ -64,6 +64,11 @@ gftpui_check_reconnect (gftpui_callback_data * cdata)
   return (1);
 }
 
+void
+gftpui_show_busy (gboolean busy)
+{
+  /* do nothing for text based */
+}
 
 void
 gftpui_prompt_username (void *uidata, gftp_request * request)
@@ -71,7 +76,8 @@ gftpui_prompt_username (void *uidata, gftp_request * request)
   char tempstr[256];
 
   gftp_set_username (request, 
-                     gftp_text_ask_question (_("Username [anonymous]:"), 1,
+                     gftp_text_ask_question (request,
+                                             _("Username [anonymous]:"), 1,
                                              tempstr, sizeof (tempstr)));
 }
 
@@ -83,7 +89,7 @@ gftpui_prompt_password (void *uidata, gftp_request * request)
   char tempstr[256];
 
   gftp_set_password (request,
-                     gftp_text_ask_question (_("Password:"), 0,
+                     gftp_text_ask_question (request, _("Password:"), 0,
                                              tempstr, sizeof (tempstr)));
 }
 
@@ -94,6 +100,15 @@ gftpui_add_file_to_transfer (gftp_transfer * tdata, GList * curfle)
 }
 
 
+void
+gftpui_cancel_file_transfer (gftp_transfer * tdata)
+{
+  tdata->cancel = 1;
+  tdata->fromreq->cancel = 1;
+  tdata->toreq->cancel = 1;
+}
+
+
 void
 gftpui_ask_transfer (gftp_transfer * tdata)
 {
@@ -142,7 +157,7 @@ gftpui_ask_transfer (gftp_transfer * tdata)
     
           g_snprintf (question, sizeof (question), _("%s already exists. (%s source size, %s destination size):\n(o)verwrite, (r)esume, (s)kip, (O)verwrite All, (R)esume All, (S)kip All: (%c)"), pos, srcsize, destsize, defaction);
 
-          gftp_text_ask_question (question, 1, &buf, 1);
+          gftp_text_ask_question (tdata->fromreq, question, 1, &buf, 1);
 
           switch (buf)
             {
@@ -184,7 +199,7 @@ _gftpui_text_print_status (gftp_transfer * tdata)
 {
   static int progress_pos = 0;
   char *progress = "|/-\\";
-  int sw, tot, i;
+  unsigned int sw, tot, i;
 
   printf ("\r%c [", progress[progress_pos++]);
 
@@ -192,15 +207,15 @@ _gftpui_text_print_status (gftp_transfer * tdata)
     progress_pos = 0;
 
   sw = gftp_text_get_win_size () - 20;
-  tot = (float) tdata->curtrans / (float) tdata->tot_file_trans * (float) sw;
+  tot = (unsigned int) ((float) tdata->curtrans / (float) tdata->tot_file_trans * (float) sw);
                         
   if (tot > sw)
     tot = sw;
 
-  for (i=0; i<tot; i++)
+  for (i = 0; i < tot; i++)
     printf ("=");
 
-  for (i=0; i<sw-tot; i++)
+  for (i = 0; i < sw - tot; i++)
     printf (" ");
 
   printf ("] @ %.2fKB/s", tdata->kbs);
@@ -256,7 +271,7 @@ gftpui_protocol_ask_yes_no (gftp_request * request, char *title, char *question)
 
   do
     {
-      gftp_text_ask_question (question, 1, buf, sizeof (buf));
+      gftp_text_ask_question (request, question, 1, buf, sizeof (buf));
       if (strcasecmp (buf, "yes") == 0)
         ret = 1;
       else if (strcasecmp (buf, "no") == 0)
@@ -278,7 +293,7 @@ gftpui_protocol_ask_user_input (gftp_request * request, char *title,
 
   do
     {
-      gftp_text_ask_question (question, shown, buf, sizeof (buf));
+      gftp_text_ask_question (request, question, shown, buf, sizeof (buf));
     }
   while (*buf == '\0');
 
