@@ -17,7 +17,7 @@
 /*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA      */
 /*****************************************************************************/
 
-/* $Id: gftpui.h,v 1.17 2004/12/02 23:19:31 masneyb Exp $ */
+/* $Id: gftpui.h 884 2007-02-07 02:43:57Z masneyb $ */
 
 #ifndef __GFTPUI_H
 #define __GFTPUI_H
@@ -133,6 +133,13 @@ gftp_transfer * gftpui_common_add_file_transfer ( gftp_request * fromreq,
 						  void *touidata,
 						  GList * files );
 
+void gftpui_cancel_file_transfer 	( gftp_transfer * tdata );
+
+void gftpui_common_skip_file_transfer	( gftp_transfer * tdata,
+					  gftp_file * curfle );
+
+void gftpui_common_cancel_file_transfer	( gftp_transfer * tdata );
+
 int gftpui_common_transfer_files 	( gftp_transfer * tdata );
 
 /* gftpuicallback.c */
@@ -167,6 +174,8 @@ void gftpui_refresh 			( void *uidata,
 void *gftpui_generic_thread 		( void *(*run_function)(void *data),
 					  void *data);
 
+void gftpui_show_busy           (gboolean busy);
+
 void gftpui_prompt_username		( void *uidata,
 					  gftp_request * request );
 
